// #############################################################
// #														   #
// #                PriceActionSwingOscillator                 #
// #														   #
// #     15.06.2014 by dorschden, die.unendlichkeit@gmx.de     #
// #														   #
// #        Paypal thanks to "die.unendlichkeit@gmx.de"        #
// #    or other thanks and comments are highly appreciated    #
// #														   #
// #############################################################
// 

#region Updates
// Version 15 - 24.09.2012 - Initial public release
// - Includes PriceActionSwingTrend
// - Add developing swing volume (repainting)
// Version 17 - 05.06.2014
// - Changed the code structure
// Version 18 - 15.06.2014
// - Fixed minimum swing size to < 1 for percentage settings
#endregion

#region Using declarations
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using PriceActionSwing.Base;
using System;
using System.ComponentModel;
using System.Drawing;
using System.Xml.Serialization;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// PriceActionSwingOscillator shows the trend direction, swing relation or developing 
    /// swing volume. !!! The volume is repainting. !!!
    /// </summary>
    [Description("PriceActionSwingOscillator shows the trend direction, swing relation or developing swing volume. !!! The volume is repainting. !!!")]
    public class PriceActionSwingOscillator : Indicator
    {
        #region Variables
        //#########################################################################################
        #region Parameters
        //=========================================================================================
        /// <summary>
        /// Represents the double top/-bottom strength for the swings.
        /// </summary>
        private int dtbStrength = 20;
        /// <summary>
        /// Represents the swing size for the swings. E.g. 1 = small and 5 = bigger swings.
        /// </summary>
        private double swingSize = 7;
        /// <summary>
        /// Represents the swing type for the swings.
        /// </summary>
        private SwingStyle swingType = SwingStyle.Standard;
        /// <summary>
        /// Indicates if high and low prices are used for the swing calculations or close values.
        /// </summary>
        private bool useCloseValues = false;
        //=========================================================================================
        #endregion

        #region DataSeries
        //=========================================================================================
        private DataSeries swingTrend;
        private DataSeries swingRelation;
        private DataSeries volumeHighSeries;
        private DataSeries volumeLowSeries;
        //=========================================================================================
        #endregion

        #region Misc
        //=====================================================================
        /// <summary>
        /// Indicates if the Gann swings are updated if the last swing high/low is broken. 
        /// </summary>
        private bool useBreakouts = true;
        /// <summary>
        /// Indicates if inside bars are ignored for the Gann swing calculation. If set to 
        /// true it is possible that between consecutive up/down bars are inside bars.
        /// </summary>
        private bool ignoreInsideBars = true;
        /// <summary>
        /// Represents the number of decimal places for the instrument
        /// </summary>
        private int decimalPlaces;
        //=====================================================================
        #endregion

        #region Settings
        //=========================================================================================
        private Show showOscillator = Show.Volume;
        private bool useOldTrend = true;
        private int oldTrend = 0;
        private bool ignoreSwings = true;
        //=========================================================================================
        #endregion

        #region Class objects
        //=========================================================================================
        /// <summary>
        /// Represents the properties for the swing.
        /// </summary>
        private SwingProperties swingProperties;
        /// <summary>
        /// Represents the values for the current swing.
        /// </summary>
        private SwingCurrent swingCurrent = new SwingCurrent();
        /// <summary>
        /// Represents the swing high values.
        /// </summary>
        private Swings swingHigh = new Swings();
        /// <summary>
        /// Represents the swing low values.
        /// </summary>
        private Swings swingLow = new Swings();
        //=========================================================================================
        #endregion
        //#########################################################################################
        #endregion

        #region Initialize()
        //#########################################################################################
        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy 
        /// method is called.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Firebrick, 20), PlotStyle.Square, "RTDoubleTop"));
            Add(new Plot(new Pen(Color.Red, 20), PlotStyle.Square, "RTDownTrend"));
            Add(new Plot(new Pen(Color.Gold, 20), PlotStyle.Square, "RTNoWhere"));
            Add(new Plot(new Pen(Color.Green, 20), PlotStyle.Square, "RTUpTrend"));
            Add(new Plot(new Pen(Color.Lime, 20), PlotStyle.Square, "RTDoubleBottom"));

            Add(new Plot(new Pen(Color.Lime, 2), PlotStyle.Bar, "VHigh"));
            Add(new Plot(new Pen(Color.Red, 2), PlotStyle.Bar, "VLow"));
            Add(new Plot(new Pen(Color.Lime, 2), PlotStyle.Square, "VHighCurrent"));
            Add(new Plot(new Pen(Color.Red, 2), PlotStyle.Square, "VLowCurrent"));

            AutoScale = true;
            BarsRequired = 2;
            CalculateOnBarClose = true;
            Overlay = false;
            PriceTypeSupported = false;

            swingTrend = new DataSeries(this);
            swingRelation = new DataSeries(this);
            volumeHighSeries = new DataSeries(this);
            volumeLowSeries = new DataSeries(this);
        }
        //#########################################################################################
        #endregion

        #region OnStartUp()
        //#########################################################################################
        /// <summary>
        /// This method is used to initialize any variables or resources. This method is called 
        /// only once immediately prior to the start of the script, but after the Initialize() 
        /// method.
        /// </summary>
        protected override void OnStartUp()
        {
            // Calculate decimal places
            decimal increment = Convert.ToDecimal(Instrument.MasterInstrument.TickSize);
            int incrementLength = increment.ToString().Length;
            decimalPlaces = 0;
            if (incrementLength == 1)
                decimalPlaces = 0;
            else if (incrementLength > 2)
                decimalPlaces = incrementLength - 2;

            if (swingType != SwingStyle.Percent)
            {
                swingSize = Convert.ToInt32(Math.Round(swingSize, 0,
                    MidpointRounding.AwayFromZero));
                if (swingSize <= 0)
                    swingSize = 1;
            }

            swingProperties = new SwingProperties(swingSize, dtbStrength);
        }
        //#########################################################################################
        #endregion

        #region OnBarUpdate()
        //#########################################################################################
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            #region Initialize varibles
            //=====================================================================================
            if (FirstTickOfBar)
            {
                swingCurrent.StopOutsideBarCalc = false;

                if (CurrentBar == 1)
                {
                    swingHigh.CurBar = swingLow.CurBar = CurrentBar;
                    swingHigh.CurPrice = High[1];
                    swingLow.CurPrice = Low[1];
                }
                volumeHighSeries.Set(0);
                volumeLowSeries.Set(0);
            }
            // Set new/update high/low back to false, to avoid function 
            // calls which depends on them
            swingHigh.New = swingLow.New = swingHigh.Update = swingLow.Update = false;

            // Checks to ensure there are enough bars before beginning
            if (CurrentBars[BarsInProgress] <= BarsRequired
                || CurrentBars[BarsInProgress] < swingSize)
                return;
            //=====================================================================================
            #endregion

            #region Swing calculation
            //=====================================================================================
            switch (swingType)
	        {
                case SwingStyle.Standard:
                    CalculateSwingStandard(swingHigh, swingLow, swingCurrent, swingProperties,
                        decimalPlaces, useCloseValues);
                    break;
                case SwingStyle.Gann:
                    CalculateSwingGann(swingHigh, swingLow, swingCurrent, swingProperties, 
                        decimalPlaces);
                    break;
                case SwingStyle.Ticks:
                    CalculateSwingTicks(swingHigh, swingLow, swingCurrent, swingProperties,
                        decimalPlaces, useCloseValues);
                    break;
                case SwingStyle.Percent:
                    CalculateSwingPercent(swingHigh, swingLow, swingCurrent, swingProperties,
                        decimalPlaces, useCloseValues);
                    break;
            }
            //=====================================================================================
            #endregion

            #region Swing trend
            //=====================================================================================
            if ((swingHigh.CurRelation == 1 && swingLow.CurRelation == 1)
                    || (swingTrend[1] == 1 && swingCurrent.SwingSlope == 1)
                    || (ignoreSwings && swingTrend[1] == 1 && swingLow.CurRelation == 1)
                    || (((swingTrend[1] == 1) || (swingCurrent.SwingSlope == 1 
                        && swingHigh.CurRelation == 1))
                        && swingLow.CurRelation == 0))
                swingTrend.Set(1);
            else if ((swingHigh.CurRelation == -1 && swingLow.CurRelation == -1)
                    || (swingTrend[1] == -1 && swingCurrent.SwingSlope == -1)
                    || (ignoreSwings && swingTrend[1] == -1 && swingHigh.CurRelation == -1)
                    || (((swingTrend[1] == -1) || (swingCurrent.SwingSlope == -1 
                        && swingLow.CurRelation == -1))
                        && swingHigh.CurRelation == 0))
                swingTrend.Set(-1);
            else
                swingTrend.Set(0);
            //=====================================================================================
            #endregion

            #region Swing relation
            //=====================================================================================
            if (swingLow.CurRelation == 0)
                swingRelation.Set(2);
            else if (swingHigh.CurRelation == 0)
                swingRelation.Set(-2);
            else if (swingHigh.CurRelation == 1 && swingLow.CurRelation == 1)
                swingRelation.Set(1);
            else if (swingHigh.CurRelation == -1 && swingLow.CurRelation == -1)
                swingRelation.Set(-1);
            else
                swingRelation.Set(0);
            //=====================================================================================
            #endregion

            #region Draw
            //=====================================================================================
            switch (showOscillator)
            {
                case Show.Trend:
                    #region Trend
                    //-----------------------------------------------------------------------------
                    int trend = Convert.ToInt32(swingTrend[0]);
                    switch (trend)
                    {
                        case -1:
                            RTDownTrend.Set(1);
                            oldTrend = -1;
                            break;
                        case 1:
                            oldTrend = 1;
                            RTUpTrend.Set(1);
                            break;
                        default:
                            if (useOldTrend)
                            {
                                if (oldTrend == 1)
                                    RTDownTrend.Set(1);
                                else if (oldTrend == -1)
                                    RTUpTrend.Set(1);
                            }
                            else
                                RTNoWhere.Set(1);
                            break;
                    }
                    //-----------------------------------------------------------------------------
                    #endregion
                    break;
                case Show.Relation:
                    #region Relation
                    //-----------------------------------------------------------------------------
                    int relation = Convert.ToInt32(swingRelation[0]);
                    switch (relation)
                    {
                        case -2:
                            RTDoubleTop.Set(relation);
                            break;
                        case -1:
                            RTDownTrend.Set(relation);
                            break;
                        case 0:
                            RTNoWhere.Set(relation);
                            break;
                        case 1:
                            RTUpTrend.Set(relation);
                            break;
                        case 2:
                            RTDoubleBottom.Set(relation);
                            break;
                        default:
                            RTNoWhere.Set(relation);
                            break;
                    }
                    //-----------------------------------------------------------------------------
                    #endregion
                    break;
                case Show.Volume:
                    #region Volume
                    //-----------------------------------------------------------------------------
                    if (swingHigh.New == true)
                    {
                        if (swingHigh.Update == false)
                        {
                            double swingVolume = 0.0;
                            for (int i = CurrentBar - swingLow.CurBar - 1; i > -1; i--)
                            {
                                swingVolume = swingVolume + Volume[i];
                                volumeHighSeries.Set(i, swingVolume);
                                volumeLowSeries.Set(i, 0);

                                VLow.Reset(i);
                                VHighCurrent.Reset(i);
                                VHigh.Set(i, swingVolume);
                            }
                            swingVolume = 0.0;
                            for (int i = CurrentBar - swingHigh.CurBar - 1; i > -1; i--)
                            {
                                swingVolume = swingVolume + Volume[i];
                                volumeLowSeries.Set(i, swingVolume);
                                VLowCurrent.Set(i, swingVolume);
                            }
                        }
                        else
                        {
                            double tmp = volumeHighSeries[1] + Volume[0];
                            volumeHighSeries.Set(tmp);
                            for (int i = CurrentBar - swingLow.CurBar - 1; i > -1; i--)
                            {
                                VLowCurrent.Reset(i);
                                volumeLowSeries.Set(i, 0);
                            }
                            VHigh.Set(tmp);
                        }
                    }
                    else if (swingLow.New == true)
                    {
                        if (swingLow.Update == false)
                        {
                            double swingVolume = 0.0;
                            for (int i = CurrentBar - swingHigh.CurBar - 1; i > -1; i--)
                            {
                                swingVolume = swingVolume + Volume[i];
                                volumeLowSeries.Set(i, swingVolume);
                                volumeHighSeries.Set(i, 0);

                                VHigh.Reset(i);
                                VLowCurrent.Reset(i);
                                VLow.Set(i, swingVolume);
                            }
                            swingVolume = 0.0;
                            for (int i = CurrentBar - swingLow.CurBar - 1; i > -1; i--)
                            {
                                swingVolume = swingVolume + Volume[i];
                                volumeHighSeries.Set(i, swingVolume);
                                VHighCurrent.Set(i, swingVolume);
                            }
                        }
                        else
                        {
                            double tmp = volumeLowSeries[1] + Volume[0];
                            volumeLowSeries.Set(tmp);
                            for (int i = CurrentBar - swingHigh.CurBar - 1; i > -1; i--)
                            {
                                VHighCurrent.Reset(i);
                                volumeHighSeries.Set(i, 0);
                            }
                            VLow.Set(tmp);
                        }
                    }
                    else
                    {
                        double tmpH = volumeHighSeries[1] + Volume[0];
                        double tmpL = volumeLowSeries[1] + Volume[0];
                        volumeHighSeries.Set(tmpH);
                        volumeLowSeries.Set(tmpL);

                        if (swingCurrent.SwingSlope == -1)
                        {
                            VLow.Set(tmpL);
                            VHighCurrent.Set(tmpH);
                        }
                        else
                        {
                            VHigh.Set(tmpH);
                            VLowCurrent.Set(tmpL);
                        }

                    }
                    //-----------------------------------------------------------------------------
                    #endregion
                    break;
            }
            //=====================================================================================
            #endregion
        }
        //#########################################################################################
        #endregion

        #region Properties
        //#########################################################################################
        #region Plots
        //=====================================================================
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RTDoubleTop
        {
            get { return Values[0]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RTDownTrend
        {
            get { return Values[1]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RTNoWhere
        {
            get { return Values[2]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RTUpTrend
        {
            get { return Values[3]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RTDoubleBottom
        {
            get { return Values[4]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VHigh
        {
            get { return Values[5]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VLow
        {
            get { return Values[6]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VHighCurrent
        {
            get { return Values[7]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VLowCurrent
        {
            get { return Values[8]; }
        }
        //===================================================================
        #endregion

        #region DataSeries
        // DataSeries =========================================================
        /// <summary>
        /// Represents the price position in relation to the swings.
        /// -2 = DT | -1 = LL and LH | 0 = price is nowhere | 1 = HH and HL | 2 = DB
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public DataSeries SwingRelation
        {
            get { return swingRelation; }
        }
        /// <summary>
        /// Represents the trend. -1 = LL/LH | 0 = no trend | 1 = HH/HL
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public DataSeries SwingTrend
        {
            get { return swingTrend; }
        }
        /// <summary>
        /// Represents the swing high volume. 
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public DataSeries VolumeHighSeries
        {
            get { return volumeHighSeries; }
        }
        /// <summary>
        /// Represents the swing low volume. 
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public DataSeries VolumeLowSeries
        {
            get { return volumeLowSeries; }
        }
        //=====================================================================
        #endregion

        #region Parameters
        //=========================================================================================
        /// <summary>
        /// Represents the swing type for the swings.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Represents the swing type for the swings.")]
        [Gui.Design.DisplayName("01. Swing type")]
        public SwingStyle SwingType
        {
            get { return swingType; }
            set { swingType = value; }
        }
        /// <summary>
        /// Represents the swing size. e.g. 1 = small swings and 5 = bigger swings.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Represents the swing size. e.g. 1 = small swings and 5 = bigger swings.")]
        [Gui.Design.DisplayName("02. Swing size")]
        public double SwingSize
        {
            get { return swingSize; }
            set { swingSize = Math.Max(Double.MinValue, value); }
        }
        /// <summary>
        /// Represents the double top/-bottom strength.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Represents the double top/-bottom strength. Increase the value to get more DB/DT.")]
        [Gui.Design.DisplayName("03. Double top/-bottom strength")]
        public int DtbStrength
        {
            get { return dtbStrength; }
            set { dtbStrength = Math.Max(1, value); }
        }
        /// <summary>
        /// Indicates if high and low prices are used for the swing calculations or close values.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Indicates if high and low prices are used for the swing calculations or close values.")]
        [Gui.Design.DisplayName("04. Use close values")]
        public bool UseCloseValues
        {
            get { return useCloseValues; }
            set { useCloseValues = value; }
        }
        //=========================================================================================
        #endregion

        #region Settings
        //=========================================================================================
        [Category("01. Settings")]
        [Description("Represents which swing indication is shown. Trend | Relation | Volume (repainting).")]
        [Gui.Design.DisplayName("01. Choose indication")]
        public Show ShowOscillator
        {
            get { return showOscillator; }
            set { showOscillator = value; }
        }
        [Category("01. Settings")]
        [Description("Indicates if the trend direction is changed when the old trend ends or whether a new trend must start first.")]
        [Gui.Design.DisplayNameAttribute("02. Trend change")]
        public bool UseOldTrend
        {
            get { return useOldTrend; }
            set { useOldTrend = value; }
        }
        //=========================================================================================
        #endregion

        #region Gann Swings
        //=========================================================================================
        [Category("02. Gann Swings")]
        [Description("Indicates if inside bars are ignored. If set to true it is possible that between consecutive up/down bars are inside bars. Only used if calculationSize > 1.")]
        [Gui.Design.DisplayName("01. Ignore inside Bars")]
        public bool IgnoreInsideBars
        {
            get { return ignoreInsideBars; }
            set { ignoreInsideBars = value; }
        }
        [Category("02. Gann Swings")]
        [Description("Indicates if the swings are updated if the last swing high/low is broken. Only used if calculationSize > 1.")]
        [Gui.Design.DisplayName("02. Use breakouts")]
        public bool UseBreakouts
        {
            get { return useBreakouts; }
            set { useBreakouts = value; }
        }
        //=========================================================================================
        #endregion
        //#########################################################################################
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PriceActionSwingOscillator[] cachePriceActionSwingOscillator = null;

        private static PriceActionSwingOscillator checkPriceActionSwingOscillator = new PriceActionSwingOscillator();

        /// <summary>
        /// PriceActionSwingOscillator shows the trend direction, swing relation or developing swing volume. !!! The volume is repainting. !!!
        /// </summary>
        /// <returns></returns>
        public PriceActionSwingOscillator PriceActionSwingOscillator(int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return PriceActionSwingOscillator(Input, dtbStrength, swingSize, swingType, useCloseValues);
        }

        /// <summary>
        /// PriceActionSwingOscillator shows the trend direction, swing relation or developing swing volume. !!! The volume is repainting. !!!
        /// </summary>
        /// <returns></returns>
        public PriceActionSwingOscillator PriceActionSwingOscillator(Data.IDataSeries input, int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            if (cachePriceActionSwingOscillator != null)
                for (int idx = 0; idx < cachePriceActionSwingOscillator.Length; idx++)
                    if (cachePriceActionSwingOscillator[idx].DtbStrength == dtbStrength && Math.Abs(cachePriceActionSwingOscillator[idx].SwingSize - swingSize) <= double.Epsilon && cachePriceActionSwingOscillator[idx].SwingType == swingType && cachePriceActionSwingOscillator[idx].UseCloseValues == useCloseValues && cachePriceActionSwingOscillator[idx].EqualsInput(input))
                        return cachePriceActionSwingOscillator[idx];

            lock (checkPriceActionSwingOscillator)
            {
                checkPriceActionSwingOscillator.DtbStrength = dtbStrength;
                dtbStrength = checkPriceActionSwingOscillator.DtbStrength;
                checkPriceActionSwingOscillator.SwingSize = swingSize;
                swingSize = checkPriceActionSwingOscillator.SwingSize;
                checkPriceActionSwingOscillator.SwingType = swingType;
                swingType = checkPriceActionSwingOscillator.SwingType;
                checkPriceActionSwingOscillator.UseCloseValues = useCloseValues;
                useCloseValues = checkPriceActionSwingOscillator.UseCloseValues;

                if (cachePriceActionSwingOscillator != null)
                    for (int idx = 0; idx < cachePriceActionSwingOscillator.Length; idx++)
                        if (cachePriceActionSwingOscillator[idx].DtbStrength == dtbStrength && Math.Abs(cachePriceActionSwingOscillator[idx].SwingSize - swingSize) <= double.Epsilon && cachePriceActionSwingOscillator[idx].SwingType == swingType && cachePriceActionSwingOscillator[idx].UseCloseValues == useCloseValues && cachePriceActionSwingOscillator[idx].EqualsInput(input))
                            return cachePriceActionSwingOscillator[idx];

                PriceActionSwingOscillator indicator = new PriceActionSwingOscillator();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.DtbStrength = dtbStrength;
                indicator.SwingSize = swingSize;
                indicator.SwingType = swingType;
                indicator.UseCloseValues = useCloseValues;
                Indicators.Add(indicator);
                indicator.SetUp();

                PriceActionSwingOscillator[] tmp = new PriceActionSwingOscillator[cachePriceActionSwingOscillator == null ? 1 : cachePriceActionSwingOscillator.Length + 1];
                if (cachePriceActionSwingOscillator != null)
                    cachePriceActionSwingOscillator.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachePriceActionSwingOscillator = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// PriceActionSwingOscillator shows the trend direction, swing relation or developing swing volume. !!! The volume is repainting. !!!
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceActionSwingOscillator PriceActionSwingOscillator(int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return _indicator.PriceActionSwingOscillator(Input, dtbStrength, swingSize, swingType, useCloseValues);
        }

        /// <summary>
        /// PriceActionSwingOscillator shows the trend direction, swing relation or developing swing volume. !!! The volume is repainting. !!!
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceActionSwingOscillator PriceActionSwingOscillator(Data.IDataSeries input, int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return _indicator.PriceActionSwingOscillator(input, dtbStrength, swingSize, swingType, useCloseValues);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// PriceActionSwingOscillator shows the trend direction, swing relation or developing swing volume. !!! The volume is repainting. !!!
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceActionSwingOscillator PriceActionSwingOscillator(int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return _indicator.PriceActionSwingOscillator(Input, dtbStrength, swingSize, swingType, useCloseValues);
        }

        /// <summary>
        /// PriceActionSwingOscillator shows the trend direction, swing relation or developing swing volume. !!! The volume is repainting. !!!
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceActionSwingOscillator PriceActionSwingOscillator(Data.IDataSeries input, int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PriceActionSwingOscillator(input, dtbStrength, swingSize, swingType, useCloseValues);
        }
    }
}
#endregion
