// #############################################################
// #														   #
// #                    PriceActionSwingPro                    #
// #														   #
// #     15.06.2014 by dorschden, die.unendlichkeit@gmx.de     #
// #														   #
// #        Paypal thanks to "die.unendlichkeit@gmx.de"        #
// #    or other thanks and comments are highly appreciated    #
// #														   #
// #############################################################
// 

#region Updates
// Version 12 - 12.07.2011
// Version 13 - 13.07.2011
// Version 14 - 13.06.2012
// - Split version PriceActionSwing & PriceActionSwingPro
// Version 15 - 24.09.2012
// - Added naked swing visualization
// - Improved time output visualization
// - Improved duration output visualization
// Version 16 - 10.10.2012
// - Fixed volume bug - volume bigger than Int32
// Version 17 - 05.06.2014
// - Changed the code structure
// - Added swing tick and swing percentage calculation
// - Added swing calculation based on close values
// - Added seperate swing price output
// - Added developing GomCD output since the last swing
// - Improved risk management items
// - Added risk management tab
// - Added different alerts
// Version 18 - 15.06.2014
// - Fixed minimum swing size to < 1 for percentage settings
#endregion

#region Using declarations
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using PriceActionSwing.Base;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Xml.Serialization;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// PriceActionSwingPro calculates swings and visualize them in different ways
    /// and display several information about them. Features: 
    /// ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic
    /// | Divergence
    /// </summary>
    [Description("PriceActionSwingPro calculates swings and visualize them in different ways and display several information about them. Features: ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic | Divergence.")]
    public class PriceActionSwingPro : Indicator
    {
        #region Variables
        //#########################################################################################
        #region Parameters
        //=========================================================================================
        /// <summary>
        /// Represents the double top/-bottom strength for the swings.
        /// </summary>
        private int dtbStrength = 20;
        /// <summary>
        /// Represents the swing size for the swings. E.g. 1 = small and 5 = bigger swings.
        /// </summary>
        private double swingSize = 7;
        /// <summary>
        /// Represents the swing type for the swings.
        /// </summary>
        private SwingStyle swingType = SwingStyle.Standard;
        /// <summary>
        /// Indicates if high and low prices are used for the swing calculations or close values.
        /// </summary>
        private bool useCloseValues = false;
        //=========================================================================================
        #endregion

        #region Features
        //=========================================================================================
        /// <summary>
        /// Indicates if and for which direction AB=CD patterns are computed.
        /// </summary>
        private AbcPatternMode abcPattern = AbcPatternMode.Long_Short;
        /// <summary>
        /// Indicates the target in percent of the AB range for the AB=CD pattern for the risk 
        /// management.
        /// </summary>
        private double abcTarget = 100.0;
        private DivergenceMode divergenceMode = DivergenceMode.False;
        private DivergenceDirection divergenceDirection = DivergenceDirection.Long_Short;
        private int param1 = 10;
        private int param2 = 24;
        private int param3 = 9;
        /// <summary>
        /// Indicates if and where a swing statistic is shown.
        /// </summary>
        private StatisticPositionStyle statisticPosition = StatisticPositionStyle.False;
        /// <summary>
        /// Indicates the number of swings which are used for the current swing statistic.
        /// </summary>
        private int statisticLength = 5;
        /// <summary>
        /// Indicates if and where the risk management tools are shown.
        /// </summary>
        private RiskManagementStyle riskManagementPosition = RiskManagementStyle.Tab;
        /// <summary>
        /// Represents the stop loss in ticks for long entries to calculate the risk parameters.
        /// </summary>
        private int riskLongStopInTicks = 50;
        /// <summary>
        /// Represents the take profit in ticks for long entries to calculate the risk parameters.
        /// </summary>
        private int riskLongTargetInTicks = 100;
        /// <summary>
        /// Represents the stop loss in ticks for short entries to calculate the risk parameters.
        /// </summary>
        private int riskShortStopInTicks = 50;
        /// <summary>
        /// Represents the take profit in ticks for short entries to calculate the risk parameters.
        /// </summary>
        private int riskShortTargetInTicks = 100;
        /// <summary>
        /// Represents the account size from which the quantity for a trade is calculated based
        /// on the account risk.
        /// </summary>
        private double accountSize = 100000.00;
        /// <summary>
        /// Represents the percentage of the account size which is risked for each trade.
        /// </summary>
        private double accountRisk = 0.5;
        /// <summary>
        /// Indicates if a swing extension is drawn on the chart.
        /// </summary>
        private bool addSwingExtension = false;
        /// <summary>
        /// Indicates if a swing retracement is drawn on the chart for the current swing.
        /// </summary>
        private bool addSwingRetracementFast = false;
        /// <summary>
        /// Indicates if a swing retracement is drawn on the chart for the last swing.
        /// </summary>
        private bool addSwingRetracementSlow = false;
        //=========================================================================================
        #endregion

        #region Display
        //=========================================================================================
        /// <summary>
        /// Represents the swing length visualization type for the swings.
        /// </summary>
        private SwingLengthStyle swingLengthType = SwingLengthStyle.Ticks;
        /// <summary>
        /// Represents the swing duration visualization type for the swings.
        /// </summary>
        private SwingDurationStyle swingDurationType = SwingDurationStyle.False;
        /// <summary>
        /// Indicates if the swing price is shown for the swings.
        /// </summary>
        private bool showSwingPrice = false;
        /// <summary>
        /// Indicates if the swing label is shown for the swings.
        /// </summary>
        private bool showSwingLabel = false;
        /// <summary>
        /// Indicates if the swing percentage in relation to the last swing is shown for the 
        /// swings.
        /// </summary>
        private bool showSwingPercent = false;
        /// <summary>
        /// Represents the swing time visualization type for the swings. 
        /// </summary>
        private SwingTimeStyle swingTimeType = SwingTimeStyle.False;
        /// <summary>
        /// Indicates if the swing volume is shown for the swings.
        /// </summary>
        private SwingVolumeStyle swingVolumeType = SwingVolumeStyle.False;
        //=========================================================================================
        #endregion

        #region Visualize swings
        //=========================================================================================
        /// <summary>
        /// Represents the swing visualization type for the swings.
        /// </summary>
        private VisualizationStyle visualizationType = VisualizationStyle.Dots_ZigZag;
        /// <summary>
        /// Indicates if the change from down to up or up to down swings is indicated on the chart 
        /// for the swings.
        /// </summary>
        private bool showSwingSwitch = true;
        private int swingSwitchOffsetInTicks = 10;
        private Color swingSwitchDownColor = Color.Red;
        private Color swingSwitchUpColor = Color.Blue;
        /// <summary>
        /// Indicates if AutoScale is used for the swings. 
        /// </summary>
        private bool useAutoScale = true;
        /// <summary>
        /// Represents the color of the zig-zag up lines for the swings.
        /// </summary>
        private Color zigZagColorUp = Color.LimeGreen;
        /// <summary>
        /// Represents the color of the zig-zag down lines for the swings.
        /// </summary>
        private Color zigZagColorDn = Color.OrangeRed;
        /// <summary>
        /// Represents the line style of the zig-zag lines for the swings.
        /// </summary>
        private DashStyle zigZagStyle = DashStyle.Solid;
        /// <summary>
        /// Represents the line width of the zig-zag lines for the swings.
        /// </summary>
        private int zigZagWidth = 2;
        /// <summary>
        /// Represents the color of the swing value output for higher highs for the swings.
        /// </summary>
        private Color textColorHigherHigh = Color.White;
        /// <summary>
        /// Represents the color of the swing value output for lower highs for the swings.
        /// </summary>
        private Color textColorLowerHigh = Color.Black;
        /// <summary>
        /// Represents the color of the swing value output for double tops for the swings.
        /// </summary>
        private Color textColorDoubleTop = Color.Gold;
        /// <summary>
        /// Represents the color of the swing value output for higher lows for the swings.
        /// </summary>
        private Color textColorHigherLow = Color.White;
        /// <summary>
        /// Represents the color of the swing value output for lower lows swings for the swings.
        /// </summary>
        private Color textColorLowerLow = Color.Black;
        /// <summary>
        /// Represents the color of the swing value output for double bottoms for the swings.
        /// </summary>
        private Color textColorDoubleBottom = Color.Gold;
        /// <summary>
        /// Represents the text font for the swing value output for the swings.
        /// </summary>
        private Font textFont = new Font("Courier", 9, FontStyle.Regular);
        /// <summary>
        /// Represents the text offset in pixel for the swing length for the swings.
        /// </summary>
        private int textOffsetLength = 15;
        /// <summary>
        /// Represents the text offset in pixel for the retracement value for the swings.
        /// </summary>
        private int textOffsetPercent = 90;
        /// <summary>
        /// Represents the text offset in pixel for the swing price for the swings.
        /// </summary>
        private int textOffsetPrice = 45;
        /// <summary>
        /// Represents the text offset in pixel for the swing labels for the swings.
        /// </summary>
        private int textOffsetLabel = 60;
        /// <summary>
        /// Represents the text offset in pixel for the swing time for the swings.
        /// </summary>
        private int textOffsetTime = 75;
        /// <summary>
        /// Represents the text offset in pixel for the swing volume for the swings.
        /// </summary>
        private int textOffsetVolume = 30;

        /// <summary>
        /// Indicates if the Gann swings are updated if the last swing high/low is broken. 
        /// </summary>
        private bool useBreakouts = true;
        /// <summary>
        /// Indicates if inside bars are ignored for the Gann swing calculation. If set to 
        /// true it is possible that between consecutive up/down bars are inside bars.
        /// </summary>
        private bool ignoreInsideBars = true;
        /// <summary>
        /// Represents the number of decimal places for the instrument
        /// </summary>
        private int decimalPlaces;
        //=========================================================================================
        #endregion

        #region Naked swings
        //=========================================================================================
        private SortedList<double, int> nakedSwingHighsList = new SortedList<double, int>();
        private SortedList<double, int> nakedSwingLowsList = new SortedList<double, int>();
        private bool showNakedSwings = false;
        private bool showHistoricalNakedSwings = false;
        private int nakedSwingCounter = 0;
        private Color nakedSwingHighColor = Color.Red;
        private Color nakedSwingLowColor = Color.Blue;
        private DashStyle nakedSwingDashStyle = DashStyle.Solid;
        private int nakedSwingLineWidth = 1;
        //=========================================================================================
        #endregion

        #region ABC pattern
        //=========================================================================================
        private bool abcLabel = true;
        private DashStyle abcLineStyle = DashStyle.Solid;
        private DashStyle abcLineStyleRatio = DashStyle.Dash;
        private int abcLineWidth = 4;
        private int abcLineWidthRatio = 2;
        private Font abcTextFont = new Font("Courier", 11, FontStyle.Bold);
        private int abcTextOffsetLabel = 50;
        private Color abcTextColorDn = Color.Red;
        private Color abcTextColorUp = Color.Green;
        private Color abcZigZagColorDn = Color.Red;
        private Color abcZigZagColorUp = Color.Green;
        private double abcMaxRetracement = 92.0;
        private double abcMinRetracement = 61.0;
        private DashStyle entryLineStyle = DashStyle.Solid;
        private int entryLineWidth = 4;
        private Color entryLineColorDn = Color.Red;
        private Color entryLineColorUp = Color.Green;
        private bool showEntryArrows = true;
        private bool showEntryLine = true;
        private bool showHistoricalEntryLine = true;
        private int yTickOffset = 5;
        //=========================================================================================
        private DataSeries abcSignals;
        private DataSeries entryLong;
        private DataSeries entryShort;
        private DataSeries entryLevelLine;
        //=========================================================================================
        private int abcEntryTag = 0;
        private bool abcLongChanceInProgress = false;
        private bool abcShortChanceInProgress = false;
        private double retracementEntryValue = 38.0;
        private double entryLevel = 0.0;
        private int entryLineStartBar = 0;
        private int aBar = 0;
        private int bBar = 0;
        private int cBar = 0;
        private int entryBar = 0;
        private int patternCounter = 0;
        private int tmpCounter = 0;
        private int drawTag = 0;
        //=========================================================================================
        #endregion

        #region Alerts
        //=========================================================================================
        private int alertTag = 0;
        private bool alertAbc = false;
        private bool alertAbcEntry = false;
        private NinjaTrader.Cbi.Priority alertAbcPriority = NinjaTrader.Cbi.Priority.Medium;
        private NinjaTrader.Cbi.Priority alertAbcEntryPriority = NinjaTrader.Cbi.Priority.High;
        private string alertAbcLongSoundFileName = "AbcLong.wav";
        private string alertAbcLongEntrySoundFileName = "AbcLongEntry.wav";
        private string alertAbcShortSoundFileName = "AbcShort.wav";
        private string alertAbcShortEntrySoundFileName = "AbcShortEntry.wav";
        //=========================================================================================
        private bool alertDoubleBottom = false;
        private NinjaTrader.Cbi.Priority alertDoubleBottomPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertDoubleBottomMessage = "Double Bottom";
        private string alertDoubleBottomSoundFileName = "DoubleBottom.wav";
        private int alertDoubleBottomRearmSeconds = 0;
        private Color alertDoubleBottomBackColor = Color.Blue;
        private Color alertDoubleBottomForeColor = Color.White;
        //=========================================================================================
        private bool alertDoubleTop = false;
        private NinjaTrader.Cbi.Priority alertDoubleTopPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertDoubleTopMessage = "Double Top";
        private string alertDoubleTopSoundFileName = "DoubleTop.wav";
        private int alertDoubleTopRearmSeconds = 0;
        private Color alertDoubleTopBackColor = Color.Red;
        private Color alertDoubleTopForeColor = Color.White;
        //=========================================================================================
        private bool alertHigherLow = false;
        private NinjaTrader.Cbi.Priority alertHigherLowPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertHigherLowMessage = "Higher Low";
        private string alertHigherLowSoundFileName = "HigherLow.wav";
        private int alertHigherLowRearmSeconds = 0;
        private Color alertHigherLowBackColor = Color.Blue;
        private Color alertHigherLowForeColor = Color.White;
        //=========================================================================================
        private bool alertLowerHigh = false;
        private NinjaTrader.Cbi.Priority alertLowerHighPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertLowerHighMessage = "Lower High";
        private string alertLowerHighSoundFileName = "LowerHigh.wav";
        private int alertLowerHighRearmSeconds = 0;
        private Color alertLowerHighBackColor = Color.Red;
        private Color alertLowerHighForeColor = Color.White;
        //=========================================================================================
        private bool alertSwingChange = false;
        private NinjaTrader.Cbi.Priority alertSwingChangePriority =
            NinjaTrader.Cbi.Priority.Low;
        private string alertSwingChangeMessage = "Swing change";
        private string alertSwingChangeSoundFileName = "SwingChange.wav";
        private int alertSwingChangeRearmSeconds = 0;
        private Color alertSwingChangeBackColor = Color.Black;
        private Color alertSwingChangeForeColor = Color.White;
        //=========================================================================================
        private bool alertDivergenceRegularHigh = false;
        private NinjaTrader.Cbi.Priority alertDivergenceRegularHighPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertDivergenceRegularHighMessage = "High regular divergence";
        private string alertDivergenceRegularHighSoundFileName = "DivergenceRegularHigh.wav";
        private int alertDivergenceRegularHighRearmSeconds = 0;
        private Color alertDivergenceRegularHighBackColor = Color.Red;
        private Color alertDivergenceRegularHighForeColor = Color.White;
        //=========================================================================================
        private bool alertDivergenceHiddenHigh = false;
        private NinjaTrader.Cbi.Priority alertDivergenceHiddenHighPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertDivergenceHiddenHighMessage = "High hidden divergence";
        private string alertDivergenceHiddenHighSoundFileName = "DivergenceHiddenHigh.wav";
        private int alertDivergenceHiddenHighRearmSeconds = 0;
        private Color alertDivergenceHiddenHighBackColor = Color.Red;
        private Color alertDivergenceHiddenHighForeColor = Color.White;
        //=========================================================================================
        private bool alertDivergenceRegularLow = false;
        private NinjaTrader.Cbi.Priority alertDivergenceRegularLowPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertDivergenceRegularLowMessage = "Low regular divergence";
        private string alertDivergenceRegularLowSoundFileName = "DivergenceRegularLow.wav";
        private int alertDivergenceRegularLowRearmSeconds = 0;
        private Color alertDivergenceRegularLowBackColor = Color.Blue;
        private Color alertDivergenceRegularLowForeColor = Color.White;
        //=========================================================================================
        private bool alertDivergenceHiddenLow = false;
        private NinjaTrader.Cbi.Priority alertDivergenceHiddenLowPriority =
            NinjaTrader.Cbi.Priority.Medium;
        private string alertDivergenceHiddenLowMessage = "Low hidden divergence";
        private string alertDivergenceHiddenLowSoundFileName = "DivergenceHiddenLow.wav";
        private int alertDivergenceHiddenLowRearmSeconds = 0;
        private Color alertDivergenceHiddenLowBackColor = Color.Blue;
        private Color alertDivergenceHiddenLowForeColor = Color.White;
        //=========================================================================================
        #endregion

        #region Divergence
        //=========================================================================================
        private bool showDivergenceRegular = true;
        private bool showDivergenceHidden = true;
        private DataSeries divergenceDataHigh;
        private DataSeries divergenceDataLow;

        private double divLastSwing = 0.0;
        private double divLastOscValue = 0.0;
        private double divCurSwing = 0.0;
        private double divCurOscValue = 0.0;

        private int drawTagDivUp = 0;
        private int drawTagDivDn = 0;
        private DashStyle divDnLineStyle = DashStyle.Dot;
        private int divDnLineWidth = 2;
        private Color divDnLineColor = Color.Red;
        private DashStyle divUpLineStyle = DashStyle.Dot;
        private int divUpLineWidth = 2;
        private Color divUpLineColor = Color.Green;

        private Stochastics stochastics;
        private MACD macd;
        private DataSeries divSignal;
        private bool divHiddenShortActive = false;
        private bool divRegularShortActive = false;
        private bool divHiddenLongActive = false;
        private bool divRegularLongActive = false;
        //=========================================================================================
        #endregion

        #region GomCD
        // Uncomment the next lines to use Gomi's Cumulative Delta ================================ 
        //private DataSeries gomCdAllDeltaClose;
        //public enum ShowGomCD
        //{
        //    AbsoluteCD,
        //    AbsoluteCD_And_SwingCD,
        //    False,
        //    SwingCD,
        //    SwingCD_And_AbsoluteCD,
        //}
        //private ShowGomCD showGomCD = ShowGomCD.SwingCD_And_AbsoluteCD;
        //private int textOffsetGomCd = 60;
        //private GomCD gomCdAll;
        //private GomFilterModeType gomCdVolumeFilterSizeMode = GomFilterModeType.None;
        //private GomCDCalculationModeType gomCdCalcMode = GomCDCalculationModeType.BidAsk;
        //private int gomCdVolumeFilterSize = 1;
        //private int gomCdAllSwingLowValue = 0;
        //private int gomCdAllSwingHighValue = 0;
        //private int curHighGomCdAll = 0;
        //private int curLowGomCdAll = 0;
        //private int lastHighGomCdAll = 0;
        //private int lastLowGomCdAll = 0;
        ///// <summary>
        ///// Indicates if the developing cumulative delta since the last fix swing is printed in 
        ///// the right bottem corner.
        ///// </summary>
        //private bool gomCDCurrent = true;
        // Uncomment till here ====================================================================
        #endregion

        #region Forms
        // Statistic forms ========================================================================
        private System.Windows.Forms.Panel panel = null;
        private System.Windows.Forms.Label label = null;
        private System.Windows.Forms.TabControl mainTabControl = null;
        private System.Windows.Forms.TabPage tabABC = null;
        private System.Windows.Forms.TabPage tabSwingLength = null;
        private System.Windows.Forms.TabPage tabSwingRelation = null;
        private System.Windows.Forms.DataGridView lengthList = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colLengthDirection = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colLengthSwingCount = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colLengthLength = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colLengthLastLength = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colLengthDuration = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colLengthLastDuration = null;
        private System.Windows.Forms.DataGridView relationList = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colRelationSwing = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colRelationSwingCount = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colRelationHigherHigh = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colRelationLowerHigh = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colRelationHigherLow = null;
        private System.Windows.Forms.DataGridViewTextBoxColumn colRelationLowerLow = null;
        private System.Windows.Forms.Splitter splitter = null;
        // Necessary toolstrip items forms ========================================================
        System.Windows.Forms.ToolStrip toolStrip = null;
        System.Windows.Forms.ToolStripButton toolStripButton = null;
        System.Windows.Forms.ToolStripSeparator toolStripSeparator = null;
        // Risk calculation forms toolstrip =======================================================
        System.Windows.Forms.ToolStripSeparator toolStripSeparator1 = null;
        System.Windows.Forms.ToolStripSeparator toolStripSeparator2 = null;
        System.Windows.Forms.ToolStripSeparator toolStripSeparator3 = null;
        System.Windows.Forms.ToolStripSeparator toolStripSeparator4 = null;
        System.Windows.Forms.ToolStripLabel toolStripLabelQty = null;
        System.Windows.Forms.ToolStripLabel toolStripLabelRiskReward = null;
        System.Windows.Forms.ToolStripLabel toolStripLabelLoss = null;
        System.Windows.Forms.ToolStripLabel toolStripLabelProfit = null;
        System.Windows.Forms.ToolStripLabel toolStripLabelEntry = null;
        System.Windows.Forms.ToolStripButton toolStripButtonLong = null;
        System.Windows.Forms.ToolStripButton toolStripButtonShort = null;
        System.Windows.Forms.ToolStripLabel toolStripLabelStop = null;
        System.Windows.Forms.ToolStripLabel toolStripLabelTarget = null;
        System.Windows.Forms.NumericUpDown toolStripNumericUpDownEntry;
        System.Windows.Forms.NumericUpDown toolStripNumericUpDownStop;
        System.Windows.Forms.NumericUpDown toolStripNumericUpDownTarget;
        System.Windows.Forms.ToolStripControlHost toolStripControlHostEntry = null;
        System.Windows.Forms.ToolStripControlHost toolStripControlHostStop = null;
        System.Windows.Forms.ToolStripControlHost toolStripControlHostTarget = null;
        // Risk calculation forms tab =============================================================
        private System.Windows.Forms.TabPage tabRiskManagement = null;
        private System.Windows.Forms.Label labelQuantity = null;
        private System.Windows.Forms.Label labelRiskReward = null;
        private System.Windows.Forms.Label labelLoss = null;
        private System.Windows.Forms.Label labelWin = null;
        private System.Windows.Forms.Label labelLossValue = null;
        private System.Windows.Forms.Label labelWinValue = null;
        private System.Windows.Forms.Label labelEntry = null;
        private System.Windows.Forms.Label labelStopLoss = null;
        private System.Windows.Forms.Label labelTakeProfit = null;
        private System.Windows.Forms.Button buttonEntryLong = null;
        private System.Windows.Forms.Button buttonEntryShort = null;
        private System.Windows.Forms.NumericUpDown numericUpDownEntry = null;
        private System.Windows.Forms.NumericUpDown numericUpDownStopLoss = null;
        private System.Windows.Forms.NumericUpDown numericUpDownTakeProfit = null;
        //=========================================================================================
        #endregion

        #region Statistic
        //===================================================================
        private double overallAvgDnLength = 0;
        private double overallAvgUpLength = 0;
        private double overallUpLength = 0;
        private double overallDnLength = 0;
        private double overallAvgDnDuration = 0;
        private double overallAvgUpDuration = 0;
        private double overallUpDuration = 0;
        private double overallDnDuration = 0;

        private double avgUpLength = 0;
        private double avgDnLength = 0;
        private double upLength = 0;
        private double dnLength = 0;
        private double avgUpDuration = 0;
        private double avgDnDuration = 0;
        private double upDuration = 0;
        private double dnDuration = 0;

        // Variables for the swing to swing relation statistic ================
        private int hhCount = 0;
        private int hhCountHH = 0;
        private double hhCountHHPercent = 0;
        private int hhCountLH = 0;
        private double hhCountLHPercent = 0;
        private int hhCountHL = 0;
        private double hhCountHLPercent = 0;
        private int hhCountLL = 0;
        private double hhCountLLPercent = 0;

        private int lhCount = 0;
        private int lhCountHH = 0;
        private double lhCountHHPercent = 0;
        private int lhCountLH = 0;
        private double lhCountLHPercent = 0;
        private int lhCountHL = 0;
        private double lhCountHLPercent = 0;
        private int lhCountLL = 0;
        private double lhCountLLPercent = 0;

        private int dtCount = 0;
        private int dtCountHH = 0;
        private double dtCountHHPercent = 0;
        private int dtCountLH = 0;
        private double dtCountLHPercent = 0;
        private int dtCountHL = 0;
        private double dtCountHLPercent = 0;
        private int dtCountLL = 0;
        private double dtCountLLPercent = 0;

        private int llCount = 0;
        private int llCountHH = 0;
        private double llCountHHPercent = 0;
        private int llCountLH = 0;
        private double llCountLHPercent = 0;
        private int llCountHL = 0;
        private double llCountHLPercent = 0;
        private int llCountLL = 0;
        private double llCountLLPercent = 0;

        private int hlCount = 0;
        private int hlCountHH = 0;
        private double hlCountHHPercent = 0;
        private int hlCountLH = 0;
        private double hlCountLHPercent = 0;
        private int hlCountHL = 0;
        private double hlCountHLPercent = 0;
        private int hlCountLL = 0;
        private double hlCountLLPercent = 0;

        private int dbCount = 0;
        private int dbCountHH = 0;
        private double dbCountHHPercent = 0;
        private int dbCountLH = 0;
        private double dbCountLHPercent = 0;
        private int dbCountHL = 0;
        private double dbCountHLPercent = 0;
        private int dbCountLL = 0;
        private double dbCountLLPercent = 0;
        //===================================================================
        #endregion

        #region Class objects and DataSeries
        //=========================================================================================
        /// <summary>
        /// Represents the properties for the swing.
        /// </summary>
        private SwingProperties swingProperties;
        /// <summary>
        /// Represents the values for the current swing.
        /// </summary>
        private SwingCurrent swingCurrent = new SwingCurrent();
        /// <summary>
        /// Represents the swing high values.
        /// </summary>
        private Swings swingHigh = new Swings();
        /// <summary>
        /// Represents the swing low values.
        /// </summary>
        private Swings swingLow = new Swings();
        /// <summary>
        /// Indicates if the swing direction changed form down to up swing for the swings.
        /// </summary>
        private BoolSeries upFlip;
        /// <summary>
        /// Indicates if the swing direction changed form up to down swing for the swings.
        /// </summary>
        private BoolSeries dnFlip;
        /// <summary>
        /// Represents a list of all up swings for the swings.
        /// </summary>
        private List<SwingStruct> swingHighs = new List<SwingStruct>();
        /// <summary>
        /// Represents a list of all down swings for the swings.
        /// </summary>
        private List<SwingStruct> swingLows = new List<SwingStruct>();
        //=========================================================================================
        #endregion
        //#########################################################################################
        #endregion

        #region Initialize()
        //#########################################################################################
        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy 
        /// method is called.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Gold, 3), PlotStyle.Dot, "DoubleBottom"));
            Add(new Plot(new Pen(Color.Red, 3), PlotStyle.Dot, "LowerLow"));
            Add(new Plot(new Pen(Color.Green, 3), PlotStyle.Dot, "HigherLow"));

            Add(new Plot(new Pen(Color.Gold, 3), PlotStyle.Dot, "DoubleTop"));
            Add(new Plot(new Pen(Color.Red, 3), PlotStyle.Dot, "LowerHigh"));
            Add(new Plot(new Pen(Color.Green, 3), PlotStyle.Dot, "HigherHigh"));

            Add(new Plot(new Pen(Color.Blue, 1), PlotStyle.Square, "GannSwing"));

            AutoScale = true;
            useAutoScale = AutoScale;
            BarsRequired = 1;
            CalculateOnBarClose = false;
            Overlay = true;
            PriceTypeSupported = false;

            dnFlip = new BoolSeries(this);
            upFlip = new BoolSeries(this);

            abcSignals = new DataSeries(this);
            entryLong = new DataSeries(this);
            entryShort = new DataSeries(this);
            entryLevelLine = new DataSeries(this);

            divergenceDataHigh = new DataSeries(this);
            divergenceDataLow = new DataSeries(this);
            divSignal = new DataSeries(this);

            #region GomCD
            // Uncomment the next lines to use Gomi's Cumulative Delta stuff ======================
            //gomCdAllDeltaClose = new DataSeries(this);
            // Uncomment till here ================================================================
            #endregion
        }
        //#########################################################################################
        #endregion

        #region OnStartUp()
        //#########################################################################################
        /// <summary>
        /// This method is used to initialize any variables or resources. This method is called 
        /// only once immediately prior to the start of the script, but after the Initialize() 
        /// method.
        /// </summary>
        protected override void OnStartUp()
        {
            // Calculate decimal places
            decimal increment = Convert.ToDecimal(Instrument.MasterInstrument.TickSize);
            int incrementLength = increment.ToString().Length;
            decimalPlaces = 0;
            if (incrementLength == 1)
                decimalPlaces = 0;
            else if (incrementLength > 2)
                decimalPlaces = incrementLength - 2;

            if (swingType != SwingStyle.Percent)
            {
                swingSize = Convert.ToInt32(Math.Round(swingSize, 0,
                    MidpointRounding.AwayFromZero));
                if (swingSize <= 0)
                    swingSize = 1;
            }

            swingProperties = new SwingProperties(swingType, swingSize, dtbStrength,
                swingLengthType, swingDurationType, showSwingPrice, showSwingLabel,
                showSwingPercent, swingTimeType, swingVolumeType, visualizationType,
                useBreakouts, ignoreInsideBars, useAutoScale,
                zigZagColorUp, zigZagColorDn, zigZagStyle, zigZagWidth, textColorHigherHigh,
                textColorLowerHigh, textColorDoubleTop, textColorHigherLow,
                textColorLowerLow, textColorDoubleBottom, textFont, textOffsetLength,
                textOffsetPercent, textOffsetPrice, textOffsetLabel, textOffsetTime,
                textOffsetVolume, useCloseValues);

            #region GomCD
            // Uncomment the next lines to use Gomi's Cumulative Delta stuff ======================
            //gomCdAll = GomCD(gomCdCalcMode, GomCDChartType.CumulativeChart,
            //        "Binary", Gom.FileModeType.OnePerDay, gomCdVolumeFilterSizeMode,
            //        gomCdVolumeFilterSize, false);
            // Uncomment till here ================================================================
            #endregion

            #region Divergence
            //=====================================================================================
            if (divergenceMode != DivergenceMode.False)
            {
                switch (divergenceMode)
                {
                    case DivergenceMode.Custom:
                        // Add custom divergence indicator here
                        break;
                    case DivergenceMode.MACD:
                        macd = MACD(param1, param2, param3);
                        break;
                    case DivergenceMode.Stochastics:
                        stochastics = Stochastics(param1, param2, param3);
                        break;
                }
            }
            //=====================================================================================
            #endregion

            #region Controls
            //=====================================================================================
            if (ChartControl == null)
            {
                statisticPosition = StatisticPositionStyle.False;
                riskManagementPosition = RiskManagementStyle.False;
            }

            if (statisticPosition != StatisticPositionStyle.False
                || riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                toolStrip = (ToolStrip)ChartControl.Controls["tsrTool"];
            }

            #region Statistic panel
            //-------------------------------------------------------------------------------------
            if (statisticPosition != StatisticPositionStyle.False)
            {
                DockStyle dockStyle = DockStyle.Bottom;
                switch (statisticPosition)
                {
                    case StatisticPositionStyle.Bottom:
                        dockStyle = DockStyle.Bottom;
                        break;
                    case StatisticPositionStyle.Top:
                        dockStyle = DockStyle.Top;
                        break;
                    default:
                        dockStyle = DockStyle.Bottom;
                        break;
                }

                #region Create new control objects
                //.................................................................................
                System.Windows.Forms.DataGridViewCellStyle dataGridViewCellStyle1 = 
                    new System.Windows.Forms.DataGridViewCellStyle();
                this.panel = new System.Windows.Forms.Panel();
                this.label = new System.Windows.Forms.Label();
                this.mainTabControl = new System.Windows.Forms.TabControl();
                this.tabABC = new System.Windows.Forms.TabPage();
                this.tabSwingLength = new System.Windows.Forms.TabPage();
                this.tabSwingRelation = new System.Windows.Forms.TabPage();
                this.lengthList = new System.Windows.Forms.DataGridView();
                this.colLengthDirection = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colLengthSwingCount = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colLengthLength = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colLengthLastLength = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colLengthDuration = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colLengthLastDuration = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.relationList = new System.Windows.Forms.DataGridView();
                this.colRelationSwing = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colRelationSwingCount = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colRelationHigherHigh = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colRelationLowerHigh = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colRelationHigherLow = new System.Windows.Forms.DataGridViewTextBoxColumn();
                this.colRelationLowerLow = new System.Windows.Forms.DataGridViewTextBoxColumn();
                //.................................................................................
                #endregion

                #region Tab Swing Length
                //.................................................................................
                // 
                // colLengthDirection
                // 
                this.colLengthDirection.HeaderText = "Direction";
                this.colLengthDirection.Name = "colLengthDirection";
                this.colLengthDirection.ReadOnly = true;
                this.colLengthDirection.Width = 120;
                // 
                // colLengthSwingCount
                // 
                this.colLengthSwingCount.HeaderText = "Swing Count";
                this.colLengthSwingCount.Name = "colLengthSwingCount";
                this.colLengthSwingCount.ReadOnly = true;
                this.colLengthSwingCount.Width = 120;
                // 
                // colLengthLength
                // 
                this.colLengthLength.HeaderText = "Length";
                this.colLengthLength.Name = "colLengthLength";
                this.colLengthLength.ReadOnly = true;
                this.colLengthLength.Width = 120;
                // 
                // colLengthLastLength
                // 
                this.colLengthLastLength.HeaderText = "Last Length";
                this.colLengthLastLength.Name = "colLengthLastLength";
                this.colLengthLastLength.ReadOnly = true;
                this.colLengthLastLength.Width = 120;
                // 
                // colLengthDuration
                // 
                this.colLengthDuration.HeaderText = "Duration";
                this.colLengthDuration.Name = "colLengthDuration";
                this.colLengthDuration.ReadOnly = true;
                this.colLengthDuration.Width = 120;
                // 
                // colLengthLastDuration
                // 
                this.colLengthLastDuration.HeaderText = "Last Duration";
                this.colLengthLastDuration.Name = "colLengthLastDuration";
                this.colLengthLastDuration.ReadOnly = true;
                this.colLengthLastDuration.Width = 120;
                // 
                // lengthList
                // 
                this.lengthList.AllowUserToAddRows = false;
                this.lengthList.AllowUserToDeleteRows = false;
                this.lengthList.AllowUserToResizeColumns = false;
                this.lengthList.AllowUserToResizeRows = false;
                this.lengthList.BackgroundColor = System.Drawing.SystemColors.Control;
                this.lengthList.ColumnHeadersHeightSizeMode = 
                    System.Windows.Forms.DataGridViewColumnHeadersHeightSizeMode.AutoSize;
                this.lengthList.Columns.AddRange(new System.Windows.Forms.DataGridViewColumn[] {
				this.colLengthDirection,
				this.colLengthSwingCount,
				this.colLengthLength,
				this.colLengthLastLength,
				this.colLengthDuration,
				this.colLengthLastDuration});
                dataGridViewCellStyle1.Alignment = 
                    System.Windows.Forms.DataGridViewContentAlignment.MiddleRight;
                dataGridViewCellStyle1.BackColor = System.Drawing.SystemColors.Window;
                dataGridViewCellStyle1.Font = new System.Drawing.Font("Microsoft Sans Serif", 
                    7.8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, 
                    ((byte)(0)));
                dataGridViewCellStyle1.ForeColor = System.Drawing.SystemColors.ControlText;
                dataGridViewCellStyle1.SelectionBackColor = 
                    System.Drawing.SystemColors.Highlight;
                dataGridViewCellStyle1.SelectionForeColor = 
                    System.Drawing.SystemColors.HighlightText;
                dataGridViewCellStyle1.WrapMode = 
                    System.Windows.Forms.DataGridViewTriState.False;
                this.lengthList.DefaultCellStyle = dataGridViewCellStyle1;
                this.lengthList.Dock = System.Windows.Forms.DockStyle.Fill;
                this.lengthList.Location = new System.Drawing.Point(3, 3);
                this.lengthList.Name = "lengthList";
                this.lengthList.ReadOnly = true;
                this.lengthList.Rows.Add("Up");
                this.lengthList.Rows.Add("Down");
                this.lengthList.RowHeadersVisible = false;
                this.lengthList.RowTemplate.Height = 24;
                this.lengthList.Size = new System.Drawing.Size(920, 305);
                this.lengthList.TabIndex = 0;
                // 
                // tabSwingLength
                // 
                this.tabSwingLength.Controls.Add(this.lengthList);
                this.tabSwingLength.Location = new System.Drawing.Point(4, 25);
                this.tabSwingLength.Name = "tabSwingLength";
                this.tabSwingLength.Padding = new System.Windows.Forms.Padding(3);
                this.tabSwingLength.TabIndex = 1;
                this.tabSwingLength.Text = "Swing Length";
                this.tabSwingLength.UseVisualStyleBackColor = true;
                //.................................................................................
                #endregion

                #region Tab Swing Relation
                //.................................................................................
                // 
                // colRelationSwing
                // 
                this.colRelationSwing.HeaderText = "Swing";
                this.colRelationSwing.Name = "colRelationSwing";
                this.colRelationSwing.ReadOnly = true;
                this.colRelationSwing.Width = 120;
                // 
                // colRelationSwingCount
                // 
                this.colRelationSwingCount.HeaderText = "Swing Count";
                this.colRelationSwingCount.Name = "colRelationSwingCount";
                this.colRelationSwingCount.ReadOnly = true;
                this.colRelationSwingCount.Width = 120;
                // 
                // colRelationHigherHigh
                // 
                this.colRelationHigherHigh.HeaderText = "Higher High";
                this.colRelationHigherHigh.Name = "colRelationHigherHigh";
                this.colRelationHigherHigh.ReadOnly = true;
                this.colRelationHigherHigh.Width = 120;
                // 
                // colRelationLowerHigh
                // 
                this.colRelationLowerHigh.HeaderText = "Lower High";
                this.colRelationLowerHigh.Name = "colRelationLowerHigh";
                this.colRelationLowerHigh.ReadOnly = true;
                this.colRelationLowerHigh.Width = 120;
                // 
                // colRelationHigherLow
                // 
                this.colRelationHigherLow.HeaderText = "Higher Low";
                this.colRelationHigherLow.Name = "colRelationHigherLow";
                this.colRelationHigherLow.ReadOnly = true;
                this.colRelationHigherLow.Width = 120;
                // 
                // colRelationLowerLow
                // 
                this.colRelationLowerLow.HeaderText = "Lower Low";
                this.colRelationLowerLow.Name = "colRelationLowerLow";
                this.colRelationLowerLow.ReadOnly = true;
                this.colRelationLowerLow.Width = 120;
                // 
                // relationList
                // 
                this.relationList.AllowUserToAddRows = false;
                this.relationList.AllowUserToDeleteRows = false;
                this.relationList.AllowUserToResizeColumns = false;
                this.relationList.AllowUserToResizeRows = false;
                this.relationList.BackgroundColor = System.Drawing.SystemColors.Control;
                this.relationList.ColumnHeadersHeightSizeMode = 
                    System.Windows.Forms.DataGridViewColumnHeadersHeightSizeMode.AutoSize;
                this.relationList.Columns.AddRange(new System.Windows.Forms.DataGridViewColumn[] {
				this.colRelationSwing,
				this.colRelationSwingCount,
				this.colRelationHigherHigh,
				this.colRelationLowerHigh,
				this.colRelationHigherLow,
				this.colRelationLowerLow});
                this.relationList.DefaultCellStyle = dataGridViewCellStyle1;
                this.relationList.Dock = System.Windows.Forms.DockStyle.Fill;
                this.relationList.Location = new System.Drawing.Point(3, 3);
                this.relationList.Name = "relationList";
                this.relationList.ReadOnly = true;
                this.relationList.Rows.Add("Higher High");
                this.relationList.Rows.Add("Lower High");
                this.relationList.Rows.Add("Double Top");
                this.relationList.Rows.Add("Lower Low");
                this.relationList.Rows.Add("Higher Low");
                this.relationList.Rows.Add("Double Bottom");
                this.relationList.RowHeadersVisible = false;
                this.relationList.RowTemplate.Height = 24;
                this.relationList.Size = new System.Drawing.Size(986, 240);
                this.relationList.TabIndex = 1;
                // 
                // tabSwingRelation
                // 
                this.tabSwingRelation.Controls.Add(this.relationList);
                this.tabSwingRelation.Location = new System.Drawing.Point(4, 25);
                this.tabSwingRelation.Name = "tabSwingRelation";
                this.tabSwingRelation.Padding = new System.Windows.Forms.Padding(3);
                this.tabSwingRelation.TabIndex = 2;
                this.tabSwingRelation.Text = "Swing Relation";
                this.tabSwingRelation.UseVisualStyleBackColor = true;
                //.................................................................................
                #endregion

                #region Add button to the tool strip and create main controls
                //.................................................................................
                // 
                // toolStripSeparator
                // 
                toolStripSeparator = new System.Windows.Forms.ToolStripSeparator();
                toolStripSeparator.Name = "separator";
                // 
                // Add controls to the toolstrip
                // 
                toolStrip.Items.Add(toolStripSeparator);

                toolStripButton = new System.Windows.Forms.ToolStripButton("Hide Extras");
                toolStripButton.Name = "button";
                toolStripButton.Text = "Show Statistic";
                toolStripButton.Click += ButtonClickShowStatistic;
                toolStripButton.Enabled = true;
                toolStripButton.ForeColor = Color.Black;
                toolStrip.Items.Add(toolStripButton);

                this.splitter = new System.Windows.Forms.Splitter();
                this.splitter.Name = "splitter";
                this.splitter.BackColor = Color.LightGray;
                this.splitter.Dock = dockStyle;
                this.splitter.Hide();
                this.splitter.Width = 3;
                ChartControl.Controls.Add(this.splitter);
                //.................................................................................
                this.panel = new System.Windows.Forms.Panel();
                this.mainTabControl = new System.Windows.Forms.TabControl();
                // 
                // mainTabControl
                // 
                this.mainTabControl.Controls.Add(this.tabSwingLength);
                this.mainTabControl.Controls.Add(this.tabSwingRelation);
                this.mainTabControl.Dock = System.Windows.Forms.DockStyle.Fill;
                this.mainTabControl.Location = new System.Drawing.Point(0, 0);
                this.mainTabControl.Multiline = true;
                this.mainTabControl.Name = "mainTabControl";
                this.mainTabControl.Padding = new System.Drawing.Point(3, 3);
                this.mainTabControl.SelectedIndex = 3;
                this.mainTabControl.TabIndex = 0;
                // 
                // panel
                // 
                this.panel.BackColor = System.Drawing.Color.White;
                this.panel.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
                //this.panel.Controls.Add(this.label);
                this.panel.Controls.Add(this.mainTabControl);
                this.panel.Dock = dockStyle;
                this.panel.Location = new System.Drawing.Point(0, 0);
                this.panel.MinimumSize = new System.Drawing.Size(150, 0);
                this.panel.Name = "panel";
                this.panel.Hide();
                if (dockStyle == DockStyle.Bottom || dockStyle == DockStyle.Top)
                    this.panel.Size = new System.Drawing.Size(ChartControl.Width, 210);
                else
                    this.panel.Size = new System.Drawing.Size(250, ChartControl.Height);
                this.panel.TabIndex = 0;
                ChartControl.Controls.Add(this.panel);
                //.................................................................................
                #endregion
            }
            //-------------------------------------------------------------------------------------
            #endregion

            #region Risk management
            //-------------------------------------------------------------------------------------
            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                #region Create new control objects
                //.................................................................................
                toolStripSeparator1 = new System.Windows.Forms.ToolStripSeparator();
                toolStripSeparator2 = new System.Windows.Forms.ToolStripSeparator();
                toolStripSeparator3 = new System.Windows.Forms.ToolStripSeparator();
                toolStripSeparator4 = new System.Windows.Forms.ToolStripSeparator();
                toolStripLabelQty = new System.Windows.Forms.ToolStripLabel("toolStripLabelQty");
                toolStripLabelRiskReward = 
                    new System.Windows.Forms.ToolStripLabel("toolStripLabelRiskReward");
                toolStripLabelLoss = 
                    new System.Windows.Forms.ToolStripLabel("toolStripLabelLoss");
                toolStripLabelProfit = 
                    new System.Windows.Forms.ToolStripLabel("toolStripLabelProfit");
                toolStripLabelEntry =
                    new System.Windows.Forms.ToolStripLabel("toolStripLabelEntry");
                toolStripButtonLong =
                    new System.Windows.Forms.ToolStripButton("toolStripLabelLong");
                toolStripButtonShort =
                    new System.Windows.Forms.ToolStripButton("toolStripLabelShort");
                toolStripLabelStop = 
                    new System.Windows.Forms.ToolStripLabel("toolStripLabelStop");
                toolStripLabelTarget = 
                    new System.Windows.Forms.ToolStripLabel("toolStripLabelTarget");
                toolStripNumericUpDownEntry = new System.Windows.Forms.NumericUpDown();
                toolStripNumericUpDownStop = new System.Windows.Forms.NumericUpDown();
                toolStripNumericUpDownTarget = new System.Windows.Forms.NumericUpDown();
                //.................................................................................
                #endregion

                #region Add button to the tool strip and create main controls
                //.................................................................................
                // 
                // toolStripSeparator
                // 
                toolStripSeparator1.Name = "toolStripSeparator1";
                toolStripSeparator2.Name = "toolStripSeparator2";
                toolStripSeparator3.Name = "toolStripSeparator3";
                toolStripSeparator4.Name = "toolStripSeparator4";
                // 
                // toolStripLabelQty
                // 
                toolStripLabelQty.AutoSize = false;
                toolStripLabelQty.ForeColor = System.Drawing.Color.Black;
                toolStripLabelQty.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, 
                    System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                toolStripLabelQty.Name = "toolStripLabelQty";
                toolStripLabelQty.Size = new System.Drawing.Size(126, 22);
                toolStripLabelQty.Text = "Qty: ";
                toolStripLabelQty.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
                // 
                // toolStripLabelRiskReward
                // 
                toolStripLabelRiskReward.AutoSize = false;
                toolStripLabelRiskReward.ForeColor = System.Drawing.Color.Black;
                toolStripLabelRiskReward.Name = "toolStripLabelRiskReward";
                toolStripLabelRiskReward.Size = new System.Drawing.Size(78, 22);
                toolStripLabelRiskReward.Text = "R/R: ";
                toolStripLabelRiskReward.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
                // 
                // toolStripLabelLoss
                // 
                toolStripLabelLoss.AutoSize = false;
                toolStripLabelLoss.ForeColor = System.Drawing.Color.Red;
                toolStripLabelLoss.Name = "toolStripLabelLoss";
                toolStripLabelLoss.Size = new System.Drawing.Size(96, 22);
                toolStripLabelLoss.Text = "Loss: ";
                toolStripLabelLoss.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
                // 
                // toolStripLabelProfit
                // 
                toolStripLabelProfit.AutoSize = false;
                toolStripLabelProfit.ForeColor = System.Drawing.Color.Green;
                toolStripLabelProfit.Name = "toolStripLabelProfit";
                toolStripLabelProfit.Size = new System.Drawing.Size(96, 22);
                toolStripLabelProfit.Text = "Win: ";
                toolStripLabelProfit.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
                // 
                // toolStripButtonLong
                // 
                toolStripButtonLong.AutoSize = true;
                toolStripButtonLong.AutoToolTip = false;
                toolStripButtonLong.Enabled = true;
                toolStripButtonLong.ForeColor = System.Drawing.Color.Black;
                toolStripButtonLong.Name = "toolStripButtonLong";
                toolStripButtonLong.Text = "L";
                toolStripButtonLong.Click += new EventHandler(CalculateQuantityEnterLong);
                // 
                // toolStripButtonShort
                // 
                toolStripButtonShort.AutoSize = true;
                toolStripButtonShort.AutoToolTip = false;
                toolStripButtonShort.Enabled = true;
                toolStripButtonShort.ForeColor = System.Drawing.Color.Black;
                toolStripButtonShort.Name = "toolStripButtonShort";
                toolStripButtonShort.Text = "S";
                toolStripButtonShort.Click += new EventHandler(CalculateQuantityEnterShort);
                // 
                // toolStripLabelEntry
                // 
                toolStripLabelEntry.AutoSize = true;
                toolStripLabelEntry.ForeColor = System.Drawing.Color.Black;
                toolStripLabelEntry.Name = "toolStripLabelEntry";
                toolStripLabelEntry.Text = "Entry: ";
                // 
                // toolStripLabelStop
                // 
                toolStripLabelStop.AutoSize = true;
                toolStripLabelStop.ForeColor = System.Drawing.Color.Red;
                toolStripLabelStop.Name = "toolStripLabelStop";
                toolStripLabelStop.Text = " SL: ";
                // 
                // toolStripLabelTarget
                // 
                toolStripLabelTarget.AutoSize = true;
                toolStripLabelTarget.ForeColor = System.Drawing.Color.Green;
                toolStripLabelTarget.Name = "toolStripLabelTarget";
                toolStripLabelTarget.Text = " PT: ";
                // 
                // numericUpDownEntry
                // 
                toolStripNumericUpDownEntry.DecimalPlaces = decimalPlaces;
                toolStripNumericUpDownEntry.Increment = increment;
                toolStripNumericUpDownEntry.Maximum = new decimal(new int[] {
            999999,
            0,
            0,
            0});
                toolStripNumericUpDownEntry.Name = "toolStripNumericUpDownEntry";
                toolStripNumericUpDownEntry.Size = new System.Drawing.Size(90, 22);
                toolStripNumericUpDownEntry.TabIndex = 0;
                toolStripNumericUpDownEntry.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
                toolStripNumericUpDownEntry.ThousandsSeparator = true;
                toolStripNumericUpDownEntry.UpDownAlign = System.Windows.Forms.LeftRightAlignment.Left;
                toolStripNumericUpDownEntry.Value = new decimal(new int[] {
            1000,
            0,
            0,
            0});
                toolStripNumericUpDownEntry.ValueChanged += 
                    new System.EventHandler(NumericUpDownValueChanged);
                // 
                // numericUpDownStop
                // 
                toolStripNumericUpDownStop.DecimalPlaces = decimalPlaces;
                toolStripNumericUpDownStop.Increment = increment;
                toolStripNumericUpDownStop.Maximum = new decimal(new int[] {
            999999,
            0,
            0,
            0});
                toolStripNumericUpDownStop.Name = "toolStripNumericUpDownStop";
                toolStripNumericUpDownStop.Size = new System.Drawing.Size(90, 22);
                toolStripNumericUpDownStop.TabIndex = 10;
                toolStripNumericUpDownStop.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
                toolStripNumericUpDownStop.ThousandsSeparator = true;
                toolStripNumericUpDownStop.UpDownAlign = System.Windows.Forms.LeftRightAlignment.Left;
                toolStripNumericUpDownStop.Value = new decimal(new int[] {
            995,
            0,
            0,
            0});
                toolStripNumericUpDownStop.ValueChanged +=
                    new System.EventHandler(NumericUpDownValueChanged);
                // 
                // numericUpDownTarget
                // 
                toolStripNumericUpDownTarget.DecimalPlaces = decimalPlaces;
                toolStripNumericUpDownTarget.Increment = increment;
                toolStripNumericUpDownTarget.Maximum = new decimal(new int[] {
            999999,
            0,
            0,
            0});
                toolStripNumericUpDownTarget.Name = "toolStripNumericUpDownTarget";
                toolStripNumericUpDownTarget.Size = new System.Drawing.Size(90, 22);
                toolStripNumericUpDownTarget.TabIndex = 9;
                toolStripNumericUpDownTarget.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
                toolStripNumericUpDownTarget.ThousandsSeparator = true;
                toolStripNumericUpDownTarget.UpDownAlign = System.Windows.Forms.LeftRightAlignment.Left;
                toolStripNumericUpDownTarget.Value = new decimal(new int[] {
            1010,
            0,
            0,
            0});
                toolStripNumericUpDownTarget.ValueChanged +=
                    new System.EventHandler(NumericUpDownValueChanged);
                // 
                // Add controls to the toolstrip
                // 
                toolStripControlHostEntry = 
                    new ToolStripControlHost(toolStripNumericUpDownEntry, "toolStripControlHostEntry");
                toolStripControlHostEntry.AutoSize = false;
                toolStripControlHostStop = 
                    new ToolStripControlHost(toolStripNumericUpDownStop, "toolStripControlHostStop");
                toolStripControlHostStop.AutoSize = false;
                toolStripControlHostTarget = 
                    new ToolStripControlHost(toolStripNumericUpDownTarget, "toolStripControlHostTarget");
                toolStripControlHostTarget.AutoSize = false;
                toolStrip.Items.Add(toolStripSeparator1);
                toolStrip.Items.Add(toolStripLabelQty);
                toolStrip.Items.Add(toolStripSeparator2);
                toolStrip.Items.Add(toolStripLabelRiskReward);
                toolStrip.Items.Add(toolStripLabelLoss);
                toolStrip.Items.Add(toolStripLabelProfit);
                toolStrip.Items.Add(toolStripSeparator3);
                toolStrip.Items.Add(toolStripButtonLong);
                toolStrip.Items.Add(toolStripButtonShort);
                toolStrip.Items.Add(toolStripSeparator4);
                toolStrip.Items.Add(toolStripLabelEntry);
                toolStrip.Items.Add(toolStripControlHostEntry);
                toolStrip.Items.Add(toolStripLabelStop);
                toolStrip.Items.Add(toolStripControlHostStop);
                toolStrip.Items.Add(toolStripLabelTarget);
                toolStrip.Items.Add(toolStripControlHostTarget);
                //.................................................................................
                #endregion
            }
            else if (riskManagementPosition == RiskManagementStyle.Tab)
            {
                #region Tab
                //.................................................................................
                if (statisticPosition == StatisticPositionStyle.False)
                {
                    this.splitter = new System.Windows.Forms.Splitter();
                    this.panel = new System.Windows.Forms.Panel();
                    this.mainTabControl = new System.Windows.Forms.TabControl();
                    this.panel = new System.Windows.Forms.Panel();
                    // 
                    // splitter
                    // 
                    this.splitter.Name = "splitter";
                    this.splitter.BackColor = Color.LightGray;
                    this.splitter.Dock = DockStyle.Bottom;
                    this.splitter.Hide();
                    this.splitter.Width = 3;
                    ChartControl.Controls.Add(this.splitter);
                    // 
                    // mainTabControl
                    // 
                    this.mainTabControl.Dock = System.Windows.Forms.DockStyle.Fill;
                    this.mainTabControl.Location = new System.Drawing.Point(0, 0);
                    this.mainTabControl.Multiline = true;
                    this.mainTabControl.Name = "mainTabControl";
                    this.mainTabControl.Padding = new System.Drawing.Point(3, 3);
                    this.mainTabControl.SelectedIndex = 3;
                    this.mainTabControl.TabIndex = 0;
                    this.panel.Controls.Add(this.mainTabControl);
                    // 
                    // panel
                    // 
                    this.panel.BackColor = System.Drawing.Color.White;
                    this.panel.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
                    this.panel.Dock = DockStyle.Bottom;
                    this.panel.Location = new System.Drawing.Point(0, 0);
                    this.panel.MinimumSize = new System.Drawing.Size(125, 0);
                    this.panel.Name = "panel";
                    this.panel.Show();
                    this.panel.Size = new System.Drawing.Size(ChartControl.Width, 125);
                    this.panel.TabIndex = 0;
                    ChartControl.Controls.Add(this.panel);
                }
                else
                {
                    panel.Show();
                    splitter.Show();
                    toolStripButton.Text = "Hide Statistic";
                }

                this.tabRiskManagement = new System.Windows.Forms.TabPage();
                this.labelQuantity = new System.Windows.Forms.Label();
                this.labelRiskReward = new System.Windows.Forms.Label();
                this.labelLoss = new System.Windows.Forms.Label();
                this.labelWin = new System.Windows.Forms.Label();
                this.labelLossValue = new System.Windows.Forms.Label();
                this.labelWinValue = new System.Windows.Forms.Label();
                this.labelEntry = new System.Windows.Forms.Label();
                this.labelStopLoss = new System.Windows.Forms.Label();
                this.labelTakeProfit = new System.Windows.Forms.Label();
                this.buttonEntryLong = new System.Windows.Forms.Button();
                this.buttonEntryShort = new System.Windows.Forms.Button();
                this.numericUpDownEntry = new System.Windows.Forms.NumericUpDown();
                this.numericUpDownStopLoss = new System.Windows.Forms.NumericUpDown();
                this.numericUpDownTakeProfit = new System.Windows.Forms.NumericUpDown();
                
                // Set a margin
                int margin = 5;
                // Get the chart with
                int chartWidth = ChartControl.Width - 20;
                int sizeX1 = 70;
                int sizeX2 = 90;
                int sizeX3 = 126;
                int sizeX4 = 42;
                int sizeX5 = 70;
                int positionX5 = chartWidth - sizeX5;
                int positionX4 = positionX5 - sizeX4;
                int positionX3 = positionX4 - sizeX3 - margin;
                int positionX2 = positionX3 - sizeX2 - margin;
                int positionX1 = positionX2 - sizeX1 - margin;
                //
                // tabSwingLength
                // 
                this.tabRiskManagement.Location = new System.Drawing.Point(4, 25);
                this.tabRiskManagement.Name = "tabRiskManagement";
                this.tabRiskManagement.Padding = new System.Windows.Forms.Padding(3);
                if (statisticPosition == StatisticPositionStyle.False)
                    this.tabRiskManagement.TabIndex = 1;
                else
                    this.tabRiskManagement.TabIndex = 3;
                this.tabRiskManagement.Text = "Risk Management";
                this.tabRiskManagement.UseVisualStyleBackColor = true;
                this.mainTabControl.Controls.Add(this.tabRiskManagement);
                // 
                // labelEntry
                // 
                this.labelEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelEntry.AutoSize = true;
                this.labelEntry.ForeColor = System.Drawing.Color.Black;
                this.labelEntry.Location = new System.Drawing.Point(positionX1, 7);
                this.labelEntry.Name = "labelEntry";
                this.labelEntry.Text = "Entry:";
                this.tabRiskManagement.Controls.Add(this.labelEntry);
                // 
                // labelStopLoss
                // 
                this.labelStopLoss.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelStopLoss.AutoSize = true;
                this.labelStopLoss.ForeColor = System.Drawing.Color.Red;
                this.labelStopLoss.Location = new System.Drawing.Point(positionX1, 37);
                this.labelStopLoss.Name = "labelStopLoss";
                this.labelStopLoss.Text = "Stop Loss:";
                this.tabRiskManagement.Controls.Add(this.labelStopLoss);
                // 
                // labelTakeProfit
                // 
                this.labelTakeProfit.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelTakeProfit.AutoSize = true;
                this.labelTakeProfit.ForeColor = System.Drawing.Color.Green;
                this.labelTakeProfit.Location = new System.Drawing.Point(positionX1, 67);
                this.labelTakeProfit.Name = "labelTakeProfit";
                this.labelTakeProfit.Text = "Take profit:";
                this.tabRiskManagement.Controls.Add(this.labelTakeProfit);
                // 
                // numericUpDownEntry
                // 
                this.numericUpDownEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.numericUpDownEntry.AutoSize = false;
                this.numericUpDownEntry.Location = new System.Drawing.Point(positionX2, 5);
                this.numericUpDownEntry.DecimalPlaces = decimalPlaces;
                this.numericUpDownEntry.Increment = increment;
                this.numericUpDownEntry.Maximum = new decimal(new int[] {
            999999,
            0,
            0,
            0});
                this.numericUpDownEntry.Name = "numericUpDownEntry";
                this.numericUpDownEntry.Size = new System.Drawing.Size(90, 22);
                this.numericUpDownEntry.TabIndex = 3;
                this.numericUpDownEntry.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
                this.numericUpDownEntry.ThousandsSeparator = true;
                this.numericUpDownEntry.UpDownAlign = System.Windows.Forms.LeftRightAlignment.Left;
                this.numericUpDownEntry.Value = new decimal(new int[] {
            1000,
            0,
            0,
            0});
                this.numericUpDownEntry.ValueChanged +=
                    new System.EventHandler(NumericUpDownValueChanged);
                this.tabRiskManagement.Controls.Add(this.numericUpDownEntry);
                // 
                // numericUpDownStopLoss
                // 
                this.numericUpDownStopLoss.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.numericUpDownStopLoss.AutoSize = false;
                this.numericUpDownStopLoss.Location = new System.Drawing.Point(positionX2, 35);
                this.numericUpDownStopLoss.DecimalPlaces = decimalPlaces;
                this.numericUpDownStopLoss.Increment = increment;
                this.numericUpDownStopLoss.Maximum = new decimal(new int[] {
            999999,
            0,
            0,
            0});
                this.numericUpDownStopLoss.Name = "numericUpDownStopLoss";
                this.numericUpDownStopLoss.Size = new System.Drawing.Size(90, 22);
                this.numericUpDownStopLoss.TabIndex = 3;
                this.numericUpDownStopLoss.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
                this.numericUpDownStopLoss.ThousandsSeparator = true;
                this.numericUpDownStopLoss.UpDownAlign = System.Windows.Forms.LeftRightAlignment.Left;
                this.numericUpDownStopLoss.Value = new decimal(new int[] {
            995,
            0,
            0,
            0});
                this.numericUpDownStopLoss.ValueChanged +=
                    new System.EventHandler(NumericUpDownValueChanged);
                this.tabRiskManagement.Controls.Add(this.numericUpDownStopLoss);
                // 
                // numericUpDownEntry
                // 
                this.numericUpDownTakeProfit.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.numericUpDownTakeProfit.AutoSize = false;
                this.numericUpDownTakeProfit.Location = new System.Drawing.Point(positionX2, 65);
                this.numericUpDownTakeProfit.DecimalPlaces = decimalPlaces;
                this.numericUpDownTakeProfit.Increment = increment;
                this.numericUpDownTakeProfit.Maximum = new decimal(new int[] {
            999999,
            0,
            0,
            0});
                this.numericUpDownTakeProfit.Name = "numericUpDownTakeProfit";
                this.numericUpDownTakeProfit.Size = new System.Drawing.Size(90, 22);
                this.numericUpDownTakeProfit.TabIndex = 3;
                this.numericUpDownTakeProfit.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
                this.numericUpDownTakeProfit.ThousandsSeparator = true;
                this.numericUpDownTakeProfit.UpDownAlign = System.Windows.Forms.LeftRightAlignment.Left;
                this.numericUpDownTakeProfit.Value = new decimal(new int[] {
            1010,
            0,
            0,
            0});
                this.numericUpDownTakeProfit.ValueChanged +=
                    new System.EventHandler(NumericUpDownValueChanged);
                this.tabRiskManagement.Controls.Add(this.numericUpDownTakeProfit);
                // 
                // buttonEntryLong
                // 
                this.buttonEntryLong.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.buttonEntryLong.AutoSize = false;
                this.buttonEntryLong.Location = new System.Drawing.Point(positionX3, 3);
                this.buttonEntryLong.Name = "buttonEntryLong";
                this.buttonEntryLong.Size = new System.Drawing.Size(126, 27);
                this.buttonEntryLong.TabIndex = 1;
                this.buttonEntryLong.Text = "Get long values";
                this.buttonEntryLong.UseVisualStyleBackColor = true;
                this.buttonEntryLong.Click += new System.EventHandler(CalculateQuantityEnterLong);
                this.tabRiskManagement.Controls.Add(this.buttonEntryLong);
                // 
                // buttonEntryShort
                // 
                this.buttonEntryShort.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.buttonEntryShort.AutoSize = false;
                this.buttonEntryShort.Location = new System.Drawing.Point(positionX3, 33);
                this.buttonEntryShort.Name = "buttonEntryShort";
                this.buttonEntryShort.Size = new System.Drawing.Size(126, 27);
                this.buttonEntryShort.TabIndex = 2;
                this.buttonEntryShort.Text = "Get short values";
                this.buttonEntryShort.UseVisualStyleBackColor = true;
                this.buttonEntryShort.Click += new System.EventHandler(CalculateQuantityEnterShort);
                this.tabRiskManagement.Controls.Add(this.buttonEntryShort);
                // 
                // labelQuantity
                // 
                this.labelQuantity.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelQuantity.AutoSize = false;
                this.labelQuantity.ForeColor = System.Drawing.Color.Black;
                this.labelQuantity.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F,
                    System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                this.labelQuantity.Location = new System.Drawing.Point(positionX3, 63);
                this.labelQuantity.Name = "labelQuantity";
                this.labelQuantity.Size = new System.Drawing.Size(126, 27);
                this.labelQuantity.Text = "Qty: ";
                this.labelQuantity.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
                this.tabRiskManagement.Controls.Add(this.labelQuantity);
                // 
                // labelRiskReward
                // 
                
                this.labelRiskReward.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelRiskReward.AutoSize = true;
                this.labelRiskReward.ForeColor = System.Drawing.Color.Black;
                this.labelRiskReward.Location = new System.Drawing.Point(positionX4, 7);
                this.labelRiskReward.Name = "labelRiskReward";
                this.labelRiskReward.Size = new System.Drawing.Size(100, 27);
                this.labelRiskReward.Text = "R/R: ";
                this.tabRiskManagement.Controls.Add(this.labelRiskReward);
                // 
                // labelLoss
                // 
                this.labelLoss.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelLoss.AutoSize = true;
                this.labelLoss.ForeColor = System.Drawing.Color.Black;
                this.labelLoss.Location = new System.Drawing.Point(positionX4, 37);
                this.labelLoss.Name = "labelLoss";
                this.labelLoss.Size = new System.Drawing.Size(sizeX4, 27);
                this.labelLoss.Text = "Loss: ";
                this.tabRiskManagement.Controls.Add(this.labelLoss);
                // 
                // labelWin
                // 
                this.labelWin.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelWin.AutoSize = true;
                this.labelWin.ForeColor = System.Drawing.Color.Black;
                this.labelWin.Location = new System.Drawing.Point(positionX4, 67);
                this.labelWin.Name = "labelWin";
                this.labelWin.Size = new System.Drawing.Size(sizeX4, 27);
                this.labelWin.Text = "Win: ";
                this.tabRiskManagement.Controls.Add(this.labelWin);
                // 
                // labelLossValue
                // 
                this.labelLossValue.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelLossValue.ForeColor = System.Drawing.Color.Red;
                this.labelLossValue.Location = new System.Drawing.Point(positionX5, 37);
                this.labelLossValue.Name = "labelLossValue";
                this.labelLossValue.Size = new System.Drawing.Size(sizeX5, 27);
                this.labelLossValue.TextAlign = System.Drawing.ContentAlignment.TopRight;
                this.tabRiskManagement.Controls.Add(this.labelLossValue);
                // 
                // labelWinValue
                // 
                this.labelWinValue.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.labelWinValue.ForeColor = System.Drawing.Color.Green;
                this.labelWinValue.Location = new System.Drawing.Point(positionX5, 67);
                this.labelWinValue.Name = "labelWinValue";
                this.labelWinValue.Size = new System.Drawing.Size(sizeX5, 27);
                this.labelWinValue.TextAlign = System.Drawing.ContentAlignment.TopRight;
                this.tabRiskManagement.Controls.Add(this.labelWinValue);
                //.................................................................................
                #endregion
            }
            //-------------------------------------------------------------------------------------
            #endregion
            //=====================================================================================
            #endregion
        }
        //#########################################################################################
        #endregion

        #region OnBarUpdate()
        //#########################################################################################
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            #region Initialize varibles
            //=====================================================================================
            if (FirstTickOfBar)
            {
                entryLevelLine.Set(0);
                abcSignals.Set(0);

                if (divHiddenShortActive == true)
                    divSignal.Set(-2);
                else if (divRegularShortActive == true)
                    divSignal.Set(-1);
                else if (divHiddenLongActive == true)
                    divSignal.Set(2);
                else if (divRegularLongActive == true)
                    divSignal.Set(1);
                else
                    divSignal.Set(0);

                if (swingLow.CurPrice == 0.0 || swingHigh.CurPrice == 0.0)
                {
                    entryLong.Set(0);
                    entryShort.Set(0);
                }
                else
                {
                    entryLong.Set(entryLong[1]);
                    entryShort.Set(entryShort[1]);
                }

                if (CurrentBar == 1)
                {
                    #region GomCD
                    // Uncomment the next lines to use Gomi's Cumulative Delta ====================
                    //gomCdAllDeltaClose = gomCdAll.DeltaClose;
                    // Uncomment till here ========================================================
                    #endregion

                    #region Divergence
                    switch (divergenceMode)
                    {
                        case DivergenceMode.Custom:
                            // Add custom divergence indicator here
                            break;
                        case DivergenceMode.GomCD:
                            #region GomCD
                            // Uncomment the next lines to use Gomi's Cumulative Delta ====================
                            //divergenceDataHigh = gomCdAll.DeltaHigh;
                            //divergenceDataLow = gomCdAll.DeltaHigh;
                            // Uncomment till here ========================================================
                            #endregion
                            break;
                        case DivergenceMode.MACD:
                            divergenceDataHigh = macd.Diff;
                            divergenceDataLow = macd.Diff;
                            break;
                        case DivergenceMode.Stochastics:
                            divergenceDataHigh = stochastics.K;
                            divergenceDataLow = stochastics.K;
                            break;
                    }
                    #endregion
                }
            }

            // Checks to ensure there are enough bars before beginning
            if (CurrentBars[BarsInProgress] <= BarsRequired
                || CurrentBars[BarsInProgress] < swingSize)
                return;
            //=====================================================================================
            #endregion

            #region Swing calculation
            //=====================================================================================
            InitializeSwingCalculation(swingHigh, swingLow, swingCurrent, upFlip, swingHighs, 
                dnFlip, swingLows);

            switch (swingType)
            {
                case SwingStyle.Standard:
                    CalculateSwingStandard(swingHigh, swingLow, swingCurrent, swingProperties,
                        upFlip, swingHighs, dnFlip, swingLows, decimalPlaces, useCloseValues,
                        DoubleBottom, LowerLow, HigherLow, DoubleTop, LowerHigh, HigherHigh,
                        GannSwing);
                    break;
                case SwingStyle.Gann:
                    CalculateSwingGann(swingHigh, swingLow, swingCurrent, swingProperties, upFlip,
                        swingHighs, dnFlip, swingLows, decimalPlaces, DoubleBottom, LowerLow,
                        HigherLow, DoubleTop, LowerHigh, HigherHigh, GannSwing);
                    break;
                case SwingStyle.Ticks:
                    CalculateSwingTicks(swingHigh, swingLow, swingCurrent, swingProperties,
                        upFlip, swingHighs, dnFlip, swingLows, decimalPlaces, useCloseValues,
                        DoubleBottom, LowerLow, HigherLow, DoubleTop, LowerHigh, HigherHigh,
                        GannSwing);
                    break;
                case SwingStyle.Percent:
                    CalculateSwingPercent(swingHigh, swingLow, swingCurrent, swingProperties,
                        upFlip, swingHighs, dnFlip, swingLows, decimalPlaces, useCloseValues,
                        DoubleBottom, LowerLow, HigherLow, DoubleTop, LowerHigh, HigherHigh,
                        GannSwing);
                    break;
            }
            //=====================================================================================
            #endregion

            #region Fibonacci tools
            //=====================================================================================
            if (swingHigh.New || swingLow.New || swingHigh.Update || swingLow.Update)
            {
                #region Fibonacci extensions
                //---------------------------------------------------------------------------------
                if (addSwingExtension)
                {
                    if (swingHigh.LastPrice == 0.0 || swingLow.LastPrice == 0.0)
                        return;

                    if (swingLows[swingLows.Count - 1].relation == 1 
                        && swingCurrent.SwingSlope == -1)
                    {
                        int anchor1BarsAgo = CurrentBar - swingLow.LastBar;
                        int anchor2BarsAgo = CurrentBar - swingHigh.CurBar;
                        int anchor3BarsAgo = CurrentBar - swingLow.CurBar;
                        double anchor1Y = swingLow.LastPrice;
                        double anchor2Y = swingHigh.CurPrice;
                        double anchor3Y = swingLow.CurPrice;
                        DrawFibonacciExtensions("FibExtUp", false, anchor1BarsAgo, anchor1Y,
                            anchor2BarsAgo, anchor2Y, anchor3BarsAgo, anchor3Y);
                    }
                    else if (swingLows[swingLows.Count - 1].relation == 1 
                        && swingCurrent.SwingSlope == 1)
                    {
                        int anchor1BarsAgo = CurrentBar - swingLow.LastBar;
                        int anchor2BarsAgo = CurrentBar - swingHigh.LastBar;
                        int anchor3BarsAgo = CurrentBar - swingLow.CurBar;
                        double anchor1Y = swingLow.LastPrice;
                        double anchor2Y = swingHigh.LastPrice;
                        double anchor3Y = swingLow.CurPrice;
                        DrawFibonacciExtensions("FibExtUp", false, anchor1BarsAgo, anchor1Y,
                            anchor2BarsAgo, anchor2Y, anchor3BarsAgo, anchor3Y);
                    }
                    else
                        RemoveDrawObject("FibExtUp");

                    if (swingHighs[swingHighs.Count - 1].relation == -1 
                        && swingCurrent.SwingSlope == 1)
                    {
                        int anchor1BarsAgo = CurrentBar - swingHigh.LastBar;
                        int anchor2BarsAgo = CurrentBar - swingLow.CurBar;
                        int anchor3BarsAgo = CurrentBar - swingHigh.CurBar;
                        double anchor1Y = swingHigh.LastPrice;
                        double anchor2Y = swingLow.CurPrice;
                        double anchor3Y = swingHigh.CurPrice;
                        DrawFibonacciExtensions("FibExtDn", false, anchor1BarsAgo, anchor1Y,
                            anchor2BarsAgo, anchor2Y, anchor3BarsAgo, anchor3Y);
                    }
                    else if (swingHighs[swingHighs.Count - 1].relation == -1 
                        && swingCurrent.SwingSlope == -1)
                    {
                        int anchor1BarsAgo = CurrentBar - swingHigh.LastBar;
                        int anchor2BarsAgo = CurrentBar - swingLow.LastBar;
                        int anchor3BarsAgo = CurrentBar - swingHigh.CurBar;
                        double anchor1Y = swingHigh.LastPrice;
                        double anchor2Y = swingLow.LastPrice;
                        double anchor3Y = swingHigh.CurPrice;
                        DrawFibonacciExtensions("FibExtDn", false, anchor1BarsAgo, anchor1Y,
                            anchor2BarsAgo, anchor2Y, anchor3BarsAgo, anchor3Y);
                    }
                    else
                        RemoveDrawObject("FibExtDn");
                }
                //---------------------------------------------------------------------------------
                #endregion

                #region Fibonacci retracements
                //---------------------------------------------------------------------------------
                if (addSwingRetracementFast)
                {
                    int anchor1BarsAgo = 0;
                    int anchor2BarsAgo = 0;
                    double anchor1Y = 0.0;
                    double anchor2Y = 0.0;

                    if (swingCurrent.SwingSlope == 1)
                    {
                        anchor1BarsAgo = CurrentBar - swingLow.CurBar;
                        anchor1Y = swingLow.CurPrice;
                        anchor2BarsAgo = CurrentBar - swingHigh.CurBar;
                        anchor2Y = swingHigh.CurPrice;
                    }
                    else
                    {
                        anchor1BarsAgo = CurrentBar - swingHigh.CurBar;
                        anchor1Y = swingHigh.CurPrice;
                        anchor2BarsAgo = CurrentBar - swingLow.CurBar;
                        anchor2Y = swingLow.CurPrice;
                    }
                    DrawFibonacciRetracements("FastFibRet", AutoScale,
                        anchor1BarsAgo, anchor1Y, anchor2BarsAgo, anchor2Y);
                }

                if (addSwingRetracementSlow)
                {
                    if (swingHigh.LastPrice == 0.0 || swingLow.LastPrice == 0.0) return;

                    int anchor1BarsAgo = 0;
                    int anchor2BarsAgo = 0;
                    double anchor1Y = 0.0;
                    double anchor2Y = 0.0;

                    if (swingCurrent.SwingSlope == 1)
                    {
                        anchor1BarsAgo = CurrentBar - swingHigh.LastBar;
                        anchor1Y = swingHigh.LastPrice;
                        anchor2BarsAgo = CurrentBar - swingLow.CurBar;
                        anchor2Y = swingLow.CurPrice;
                    }
                    else
                    {
                        anchor1BarsAgo = CurrentBar - swingLow.LastBar;
                        anchor1Y = swingLow.LastPrice;
                        anchor2BarsAgo = CurrentBar - swingHigh.CurBar;
                        anchor2Y = swingHigh.CurPrice;
                    }

                    if ((swingCurrent.SwingSlope == 1 && swingHigh.CurPrice < swingHigh.LastPrice) 
                        || (swingCurrent.SwingSlope == -1 
                        && swingLow.CurPrice > swingLow.LastPrice))
                        DrawFibonacciRetracements("SlowFibRet", AutoScale, 
                            anchor1BarsAgo, anchor1Y, anchor2BarsAgo, anchor2Y);
                    else
                        RemoveDrawObject("SlowFibRet");
                }
                //---------------------------------------------------------------------------------
                #endregion
            }
            //=====================================================================================
            #endregion

            #region Swing statistic
            //=====================================================================================
            if (statisticPosition != StatisticPositionStyle.False)
            {
                if (swingLow.New == true && swingLow.Update == false)
                    UpStatistic();
                if (swingHigh.New == true && swingHigh.Update == false)
                    DownStatistic();
            }
            //=====================================================================================
            #endregion

            #region ABC pattern
            //=====================================================================================
            #region ABC long pattern
            //-------------------------------------------------------------------------------------
            if (abcPattern == AbcPatternMode.Long_Short || abcPattern == AbcPatternMode.Long)
            {
                if ((swingLow.Update || swingLow.New) && !abcLongChanceInProgress
                        && swingLow.LastRelation == -1 && swingLow.CurRelation == 1
                        && swingLow.CurPercent > abcMinRetracement && swingLow.CurPercent < abcMaxRetracement)
                {
                    drawTag = swingHigh.Counter;
                    abcLongChanceInProgress = true;
                    entryLineStartBar = CurrentBar;
                    patternCounter++;
                    tmpCounter = swingLow.Counter;
                    abcSignals.Set(1);
                    aBar = CurrentBar - swingLow.LastBar;
                    bBar = CurrentBar - swingHigh.CurBar;
                    cBar = CurrentBar - swingLow.CurBar;

                    DrawLine("ABLineUp" + drawTag, AutoScale, aBar, swingLow.LastPrice,
                        bBar, swingHigh.CurPrice, abcZigZagColorUp, abcLineStyle, abcLineWidth);
                    DrawLine("BCLineUp" + drawTag, AutoScale, bBar, swingHigh.CurPrice,
                        cBar, swingLow.CurPrice, abcZigZagColorUp, abcLineStyle, abcLineWidth);
                    DrawLine("ACLineUp" + drawTag, AutoScale, aBar, swingLow.LastPrice,
                        cBar, swingLow.CurPrice, abcZigZagColorUp, abcLineStyleRatio, abcLineWidthRatio);
                    if (abcLabel)
                    {
                        DrawText("AUp" + drawTag, AutoScale, "A", aBar, swingLow.LastPrice,
                            -abcTextOffsetLabel, abcTextColorUp, abcTextFont,
                            StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                        DrawText("BUp" + drawTag, AutoScale, "B", bBar, swingHigh.CurPrice,
                            abcTextOffsetLabel, abcTextColorUp, abcTextFont,
                            StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                        DrawText("CUp" + drawTag, AutoScale, "C", cBar, swingLow.CurPrice,
                            -abcTextOffsetLabel, abcTextColorUp, abcTextFont,
                            StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                    }

                    entryLevel = swingLow.CurPrice + Instrument.MasterInstrument.Round2TickSize((Math.Abs(swingLow.CurLength) * TickSize)
                        / 100 * retracementEntryValue);
                    entryLevelLine.Set(entryLevel);
                    if (showEntryLine)
                        DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[0], 0, entryLevelLine[0],
                            entryLineColorUp, entryLineStyle, entryLineWidth);
                    if (alertAbc)
                        Alert("Alert_Abc_Long" + alertTag++.ToString(), alertAbcPriority, "ABC Long" + " ("
                            + Bars.Period.ToString() + ")", alertAbcLongSoundFileName, 0, Color.White, Color.Blue);
                    entryLong.Set(entryLevel);
                    if (riskManagementPosition != RiskManagementStyle.False)
                    {
                        double entryRM = entryLong[0];
                        double stopRM = swingLow.CurPrice - 1 * TickSize;
                        double target100RM = 0.0;
                        if (swingCurrent.SwingSlope == 1)
                            target100RM = swingLow.CurPrice + swingHigh.LastLength * TickSize * abcTarget / 100;
                        else
                            target100RM = swingLow.CurPrice + swingHigh.CurLength * TickSize * abcTarget / 100;
                        CalculateLongRisk(entryRM, stopRM, target100RM);
                    }
                }

                if (abcLongChanceInProgress)
                {
                    if (swingLow.CurPercent > abcMaxRetracement && tmpCounter == swingLow.Counter)
                    {
                        abcLongChanceInProgress = false;
                        RemoveDrawObject("ABLineUp" + drawTag.ToString());
                        RemoveDrawObject("BCLineUp" + drawTag.ToString());
                        RemoveDrawObject("ACLineUp" + drawTag.ToString());
                        RemoveDrawObject("AUp" + drawTag.ToString());
                        RemoveDrawObject("BUp" + drawTag.ToString());
                        RemoveDrawObject("CUp" + drawTag.ToString());
                        // Remove entryLevelLine (maybe remove more objects as drawn (COBC))
                        if (!showHistoricalEntryLine)
                        {
                            for (int index = 0; index < CurrentBar - entryLineStartBar + 1; index++)
                            {
                                RemoveDrawObject("EntryLine" + (CurrentBar - index).ToString());
                            }
                            entryLineStartBar = 0;
                        }
                    }
                    else if (dnFlip[0] && tmpCounter != swingLow.Counter)
                    {
                        abcLongChanceInProgress = false;
                        entryLineStartBar = 0;
                    }
                    else
                    {
                        if (swingLow.Update && tmpCounter == swingLow.Counter)
                        {
                            aBar = CurrentBar - swingLow.LastBar;
                            bBar = CurrentBar - swingHigh.CurBar;
                            cBar = CurrentBar - swingLow.CurBar;

                            DrawLine("BCLineUp" + drawTag, AutoScale, bBar, swingHigh.CurPrice,
                                cBar, swingLow.CurPrice, abcZigZagColorUp, abcLineStyle, abcLineWidth);
                            DrawLine("ACLineUp" + drawTag, AutoScale, aBar, swingLow.LastPrice,
                                cBar, swingLow.CurPrice, abcZigZagColorUp, abcLineStyleRatio, abcLineWidthRatio);
                            if (abcLabel)
                            {
                                DrawText("CUp" + drawTag, AutoScale, "C", cBar, swingLow.CurPrice,
                                    -abcTextOffsetLabel, abcTextColorUp, abcTextFont,
                                    StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                            }

                            entryLevel = swingLow.CurPrice + Instrument.MasterInstrument.Round2TickSize((Math.Abs(swingLow.CurLength) * TickSize)
                                / 100 * retracementEntryValue);
                            entryLevelLine.Set(entryLevel);

                            if (showEntryLine)
                            {
                                if (entryLevelLine[1] == 0)
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[0], 0, entryLevelLine[0],
                                        entryLineColorUp, entryLineStyle, entryLineWidth);
                                else
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[1], 0, entryLevelLine[0],
                                        entryLineColorUp, entryLineStyle, entryLineWidth);
                            }
                            entryLong.Set(entryLevel);
                            if (riskManagementPosition != RiskManagementStyle.False)
                            {
                                double entryRM = entryLong[0];
                                double stopRM = swingLow.CurPrice - 1 * TickSize;
                                double target100RM = 0.0;
                                if (swingCurrent.SwingSlope == 1)
                                    target100RM = swingLow.CurPrice + swingHigh.LastLength * TickSize * abcTarget / 100;
                                else
                                    target100RM = swingLow.CurPrice + swingHigh.CurLength * TickSize * abcTarget / 100;
                                CalculateLongRisk(entryRM, stopRM, target100RM);
                            }
                        }
                        else if (FirstTickOfBar)
                        {
                            entryLevelLine.Set(entryLevel);

                            if (showEntryLine)
                            {
                                if (entryLevelLine[1] == 0)
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[0], 0, entryLevelLine[0],
                                        entryLineColorUp, entryLineStyle, entryLineWidth);
                                else
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[1], 0, entryLevelLine[0],
                                        entryLineColorUp, entryLineStyle, entryLineWidth);
                            }
                        }
                        abcSignals.Set(1);

                        bool abcLong = false;
                        if (CalculateOnBarClose || Historical)
                        {
                            if (Close[0] > entryLevel)
                            {
                                entryLong.Set(Close[0]);
                                abcLong = true;
                            }
                        }
                        else
                        {
                            if (FirstTickOfBar && Open[0] > entryLevel)
                            {
                                entryLong.Set(Open[0]);
                                abcLong = true;
                            }
                        }

                        if (abcLong)
                        {
                            if (showEntryArrows)
                                DrawArrowUp("AbcUp" + abcEntryTag++.ToString(), AutoScale, 0,
                                    Low[0] - yTickOffset * TickSize, abcTextColorUp);
                            abcLongChanceInProgress = false;
                            entryLineStartBar = 0;
                            entryBar = CurrentBar;
                            abcSignals.Set(2);
                            if (riskManagementPosition != RiskManagementStyle.False)
                            {
                                double entryRM = entryLong[0];
                                double stopRM = swingLow.CurPrice - 1 * TickSize;
                                double target100RM = 0.0;
                                if (swingCurrent.SwingSlope == 1)
                                    target100RM = swingLow.CurPrice + swingHigh.LastLength * TickSize * abcTarget / 100;
                                else
                                    target100RM = swingLow.CurPrice + swingHigh.CurLength * TickSize * abcTarget / 100;
                                CalculateLongRisk(entryRM, stopRM, target100RM);
                            }
                            if (alertAbcEntry)
                                Alert("Alert_Abc_Long_Entry" + alertTag++.ToString(), alertAbcEntryPriority, "ABC Long Entry" + " ("
                                    + Bars.Period.ToString() + ")", alertAbcLongEntrySoundFileName, 0, Color.Blue, Color.White);
                        }
                    }
                }
            }
            //-------------------------------------------------------------------------------------
            #endregion

            #region ABC short pattern
            //-------------------------------------------------------------------------------------
            if (abcPattern == AbcPatternMode.Long_Short || abcPattern == AbcPatternMode.Short)
            {
                if ((swingHigh.Update || swingHigh.New) && !abcShortChanceInProgress
                        && swingHigh.LastRelation == 1 && swingHigh.CurRelation == -1
                        && swingHigh.CurPercent > abcMinRetracement && swingHigh.CurPercent < abcMaxRetracement)
                {
                    drawTag = swingLow.Counter;
                    abcShortChanceInProgress = true;
                    entryLineStartBar = CurrentBar;
                    patternCounter++;
                    tmpCounter = swingHigh.Counter;
                    abcSignals.Set(-1);

                    aBar = CurrentBar - swingHigh.LastBar;
                    bBar = CurrentBar - swingLow.CurBar;
                    cBar = CurrentBar - swingHigh.CurBar;

                    DrawLine("ABLineDn" + drawTag, AutoScale, aBar, swingHigh.LastPrice,
                        bBar, swingLow.CurPrice, abcZigZagColorDn, abcLineStyle, abcLineWidth);
                    DrawLine("BCLineDn" + drawTag, AutoScale, bBar, swingLow.CurPrice,
                        cBar, swingHigh.CurPrice, abcZigZagColorDn, abcLineStyle, abcLineWidth);
                    DrawLine("ACLineDn" + drawTag, AutoScale, aBar, swingHigh.LastPrice,
                        cBar, swingHigh.CurPrice, abcZigZagColorDn, abcLineStyleRatio, abcLineWidthRatio);
                    if (abcLabel)
                    {
                        DrawText("ADn" + drawTag, AutoScale, "A", aBar, swingHigh.LastPrice,
                            abcTextOffsetLabel, abcTextColorDn, abcTextFont,
                            StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                        DrawText("BDn" + drawTag, AutoScale, "B", bBar, swingLow.CurPrice,
                            -abcTextOffsetLabel, abcTextColorDn, abcTextFont,
                            StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                        DrawText("CDn" + drawTag, AutoScale, "C", cBar, swingHigh.CurPrice,
                            abcTextOffsetLabel, abcTextColorDn, abcTextFont,
                            StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                    }

                    entryLevel = swingHigh.CurPrice - Instrument.MasterInstrument.Round2TickSize((swingHigh.CurLength * TickSize)
                        / 100 * retracementEntryValue);
                    entryLevelLine.Set(entryLevel);
                    if (showEntryLine)
                        DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[0], 0, entryLevelLine[0],
                            entryLineColorDn, entryLineStyle, entryLineWidth);
                    if (alertAbc)
                        Alert("Alert_Abc_Short" + alertTag++.ToString(), alertAbcPriority, "ABC Short" + " ("
                            + Bars.Period.ToString() + ")", alertAbcShortSoundFileName, 0, Color.White, Color.Red);
                    entryShort.Set(entryLevel);
                    if (riskManagementPosition != RiskManagementStyle.False)
                    {
                        double entryRM = entryShort[0];
                        double stopRM = swingHigh.CurPrice + 1 * TickSize;
                        double target100RM = 0.0;
                        if (swingCurrent.SwingSlope == -1)
                            target100RM = swingHigh.CurPrice + swingLow.LastLength * TickSize * abcTarget / 100;
                        else
                            target100RM = swingHigh.CurPrice + swingLow.CurLength * TickSize * abcTarget / 100;
                        CalculateShortRisk(entryRM, stopRM, target100RM);
                    }
                }

                if (abcShortChanceInProgress)
                {
                    if (swingHigh.CurPercent > abcMaxRetracement && tmpCounter == swingHigh.Counter)
                    {
                        abcShortChanceInProgress = false;
                        RemoveDrawObject("ABLineDn" + drawTag.ToString());
                        RemoveDrawObject("BCLineDn" + drawTag.ToString());
                        RemoveDrawObject("ACLineDn" + drawTag.ToString());
                        RemoveDrawObject("ADn" + drawTag.ToString());
                        RemoveDrawObject("BDn" + drawTag.ToString());
                        RemoveDrawObject("CDn" + drawTag.ToString());
                        // Remove entryLevelLine (maybe remove more objects as drawn (COBC))
                        if (!showHistoricalEntryLine)
                        {
                            for (int index = 0; index < CurrentBar - entryLineStartBar + 1; index++)
                            {
                                RemoveDrawObject("EntryLine" + (CurrentBar - index).ToString());
                            }
                            entryLineStartBar = 0;
                        }
                    }
                    else if (upFlip[0] && tmpCounter != swingHigh.Counter)
                    {
                        abcShortChanceInProgress = false;
                        entryLineStartBar = 0;
                    }
                    else
                    {
                        if (swingHigh.Update && tmpCounter == swingHigh.Counter)
                        {
                            aBar = CurrentBar - swingHigh.LastBar;
                            bBar = CurrentBar - swingLow.CurBar;
                            cBar = CurrentBar - swingHigh.CurBar;

                            DrawLine("BCLineDn" + drawTag, AutoScale, bBar, swingLow.CurPrice,
                                cBar, swingHigh.CurPrice, abcZigZagColorDn, abcLineStyle, abcLineWidth);
                            DrawLine("ACLineDn" + drawTag, AutoScale, aBar, swingHigh.LastPrice,
                                cBar, swingHigh.CurPrice, abcZigZagColorDn, abcLineStyleRatio, abcLineWidthRatio);
                            if (abcLabel)
                            {
                                DrawText("CDn" + drawTag, AutoScale, "C", cBar, swingHigh.CurPrice,
                                    abcTextOffsetLabel, abcTextColorDn, abcTextFont,
                                    StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
                            }

                            entryLevel = swingHigh.CurPrice - Instrument.MasterInstrument.Round2TickSize((swingHigh.CurLength * TickSize)
                                / 100 * retracementEntryValue);
                            entryLevelLine.Set(entryLevel);

                            if (showEntryLine)
                            {
                                if (entryLevelLine[1] == 0)
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[0], 0, entryLevelLine[0],
                                        entryLineColorDn, entryLineStyle, entryLineWidth);
                                else
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[1], 0, entryLevelLine[0],
                                        entryLineColorDn, entryLineStyle, entryLineWidth);
                            }
                            entryShort.Set(entryLevel);
                            if (riskManagementPosition != RiskManagementStyle.False)
                            {
                                double entryRM = entryShort[0];
                                double stopRM = swingHigh.CurPrice + 1 * TickSize;
                                double target100RM = 0.0;
                                if (swingCurrent.SwingSlope == -1)
                                    target100RM = swingHigh.CurPrice + swingLow.LastLength * TickSize * abcTarget / 100;
                                else
                                    target100RM = swingHigh.CurPrice + swingLow.CurLength * TickSize * abcTarget / 100;
                                CalculateShortRisk(entryRM, stopRM, target100RM);
                            }
                        }
                        else if (FirstTickOfBar)
                        {
                            entryLevelLine.Set(entryLevel);

                            if (showEntryLine)
                            {
                                if (entryLevelLine[1] == 0)
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[0], 0, entryLevelLine[0],
                                        entryLineColorDn, entryLineStyle, entryLineWidth);
                                else
                                    DrawLine("EntryLine" + CurrentBar.ToString(), AutoScale, 1, entryLevelLine[1], 0, entryLevelLine[0],
                                        entryLineColorDn, entryLineStyle, entryLineWidth);
                            }
                        }
                        abcSignals.Set(-1);

                        bool abcShort = false;
                        if (CalculateOnBarClose || Historical)
                        {
                            if (Close[0] < entryLevel)
                            {
                                entryShort.Set(Close[0]);
                                abcShort = true;
                            }
                        }
                        else
                        {
                            if (FirstTickOfBar && Open[0] < entryLevel)
                            {
                                entryShort.Set(Open[0]);
                                abcShort = true;
                            }
                        }

                        if (abcShort)
                        {
                            if (showEntryArrows)
                                DrawArrowDown("AbcDn" + abcEntryTag++.ToString(), AutoScale, 0,
                                    High[0] + yTickOffset * TickSize, abcTextColorDn);
                            abcShortChanceInProgress = false;
                            entryLineStartBar = 0;
                            entryBar = CurrentBar;
                            abcSignals.Set(-2);
                            if (riskManagementPosition != RiskManagementStyle.False)
                            {
                                double entryRM = entryShort[0];
                                double stopRM = swingHigh.CurPrice + 1 * TickSize;
                                double target100RM = 0.0;
                                if (swingCurrent.SwingSlope == -1)
                                    target100RM = swingHigh.CurPrice + swingLow.LastLength * TickSize * abcTarget / 100;
                                else
                                    target100RM = swingHigh.CurPrice + swingLow.CurLength * TickSize * abcTarget / 100;
                                CalculateShortRisk(entryRM, stopRM, target100RM);
                            }
                            if (alertAbcEntry)
                                Alert("Alert_Abc_Short_Entry" + alertTag++.ToString(), alertAbcEntryPriority, "ABC Short Entry" + " ("
                                    + Bars.Period.ToString() + ")", alertAbcShortEntrySoundFileName, 0, Color.Red, Color.Black);
                        }
                    }
                }
            }
            //-------------------------------------------------------------------------------------
            #endregion
            //=====================================================================================
            #endregion

            #region Divergence
            //=====================================================================================
            if (divergenceMode != DivergenceMode.False)
            {
                if (divergenceDirection != DivergenceDirection.Short)
                {
                    if (swingHigh.New == true && swingHigh.Update == false)
                    {
                        drawTagDivDn++;
                        divLastSwing = swingHigh.LastPrice;
                        divLastOscValue = Math.Max(divergenceDataHigh[CurrentBar - 
                            swingHigh.LastBar + 1], Math.Max(divergenceDataHigh[CurrentBar - 
                            swingHigh.LastBar], divergenceDataHigh[CurrentBar - 
                            swingHigh.LastBar - 1]));
                    }

                    if (swingHigh.New == true || swingHigh.Update == true)
                    {
                        divCurSwing = swingHigh.CurPrice;
                        divCurOscValue = Math.Max(divergenceDataHigh[CurrentBar - 
                            swingHigh.CurBar], divergenceDataHigh[CurrentBar - 
                            swingHigh.CurBar + 1]);

                        if (showDivergenceHidden == true)
                        {
                            if (divLastSwing > divCurSwing && divLastOscValue < divCurOscValue)
                            {
                                DrawLine("DivHidSignalDn" + drawTagDivDn, AutoScale,
                                    CurrentBar - swingHigh.LastBar, swingHigh.LastPrice, 
                                    CurrentBar - swingHigh.CurBar, swingHigh.CurPrice, 
                                    divDnLineColor, divDnLineStyle, divDnLineWidth);
                                int textDivBarAgo = Convert.ToInt32(CurrentBar -
                                    (swingHigh.LastBar + swingHigh.CurBar) / 2);
                                double textDivPrice = Instrument.MasterInstrument.Round2TickSize(
                                    (swingHigh.LastPrice + swingHigh.CurPrice) / 2);
                                DrawText("DivHidTextDn" + drawTagDivDn, AutoScale, "hDiv",
                                    textDivBarAgo, textDivPrice, 10, divDnLineColor, textFont,
                                    StringAlignment.Center, Color.Transparent, Color.Transparent,
                                    0);
                                divHiddenShortActive = true;
                                divSignal.Set(-2);
                            }
                            else
                            {
                                RemoveDrawObject("DivHidSignalDn" + drawTagDivDn);
                                RemoveDrawObject("DivHidTextDn" + drawTagDivDn);
                                divHiddenShortActive = false;
                            }
                        }

                        if (showDivergenceRegular == true)
                        {
                            if (divLastSwing < divCurSwing && divLastOscValue > divCurOscValue)
                            {
                                DrawLine("DivSignalDn" + drawTagDivDn, AutoScale,
                                    CurrentBar - swingHigh.LastBar, swingHigh.LastPrice, 
                                    CurrentBar - swingHigh.CurBar, swingHigh.CurPrice, 
                                    divDnLineColor, divDnLineStyle, divDnLineWidth);
                                int textDivBarAgo = Convert.ToInt32(CurrentBar -
                                    (swingHigh.LastBar + swingHigh.CurBar) / 2);
                                double textDivPrice = Instrument.MasterInstrument.Round2TickSize(
                                    (swingHigh.LastPrice + swingHigh.CurPrice) / 2);
                                DrawText("DivRegTextDn" + drawTagDivDn, AutoScale, "rDiv",
                                    textDivBarAgo, textDivPrice, 10, divDnLineColor, textFont,
                                    StringAlignment.Center, Color.Transparent, Color.Transparent, 
                                    0);
                                divRegularShortActive = true;
                                divSignal.Set(-1);
                            }
                            else
                            {
                                RemoveDrawObject("DivSignalDn" + drawTagDivDn);
                                RemoveDrawObject("DivRegTextDn" + drawTagDivDn);
                                divRegularShortActive = false;
                            }
                        }
                    }
                }

                if (divergenceDirection != DivergenceDirection.Long)
                {
                    if (swingLow.New == true && swingLow.Update == false)
                    {
                        drawTagDivUp++;
                        divLastSwing = swingLow.LastPrice;
                        divLastOscValue = Math.Min(divergenceDataLow[CurrentBar - 
                            swingLow.LastBar + 1], Math.Min(divergenceDataLow[CurrentBar -
                            swingLow.LastBar], divergenceDataLow[CurrentBar - 
                            swingLow.LastBar - 1]));
                    }

                    if (swingLow.New == true || swingLow.Update == true)
                    {
                        divCurSwing = swingLow.CurPrice;
                        divCurOscValue = Math.Min(divergenceDataLow[CurrentBar - swingLow.CurBar],
                            divergenceDataLow[CurrentBar - swingLow.CurBar + 1]);

                        if (showDivergenceHidden == true)
                        {
                            if (divLastSwing < divCurSwing && divLastOscValue > divCurOscValue)
                            {
                                DrawLine("DivHidSignalup" + drawTagDivUp, AutoScale,
                                    CurrentBar - swingLow.LastBar, swingLow.LastPrice, 
                                    CurrentBar - swingLow.CurBar, swingLow.CurPrice, 
                                    divUpLineColor, divUpLineStyle, divUpLineWidth);
                                int textDivBarAgo = Convert.ToInt32(CurrentBar -
                                    (swingLow.LastBar + swingLow.CurBar) / 2);
                                double textDivPrice = Instrument.MasterInstrument.Round2TickSize(
                                    (swingLow.LastPrice + swingLow.CurPrice) / 2);
                                DrawText("DivHidTextUp" + drawTagDivUp, AutoScale, "hDiv",
                                    textDivBarAgo, textDivPrice, -10, divUpLineColor, textFont,
                                    StringAlignment.Center, Color.Transparent, Color.Transparent, 
                                    0);
                                divHiddenLongActive = true;
                                divSignal.Set(2);
                            }
                            else
                            {
                                RemoveDrawObject("DivHidSignalup" + drawTagDivUp);
                                RemoveDrawObject("DivHidTextUp" + drawTagDivUp);
                                divHiddenLongActive = false;
                            }
                        }

                        if (showDivergenceRegular == true)
                        {
                            if (divLastSwing > divCurSwing && divLastOscValue < divCurOscValue)
                            {
                                DrawLine("DivSignalUp" + drawTagDivUp, AutoScale,
                                    CurrentBar - swingLow.LastBar, swingLow.LastPrice, 
                                    CurrentBar - swingLow.CurBar, swingLow.CurPrice, 
                                    divUpLineColor, divUpLineStyle, divUpLineWidth);
                                int textDivBarAgo = Convert.ToInt32(CurrentBar -
                                    (swingLow.LastBar + swingLow.CurBar) / 2);
                                double textDivPrice = Instrument.MasterInstrument.Round2TickSize(
                                    (swingLow.LastPrice + swingLow.CurPrice) / 2);
                                DrawText("DivRegTextUp" + drawTagDivUp, AutoScale, "rDiv",
                                    textDivBarAgo, textDivPrice, -10, divUpLineColor, textFont,
                                    StringAlignment.Center, Color.Transparent, Color.Transparent, 
                                    0);
                                divRegularLongActive = true;
                                divSignal.Set(1);
                            }
                            else
                            {
                                RemoveDrawObject("DivSignalUp" + drawTagDivUp);
                                RemoveDrawObject("DivRegTextUp" + drawTagDivUp);
                                divRegularLongActive = false;
                            }
                        }
                    }
                }

                if (dnFlip[0] == true)
                {
                    if (divRegularShortActive == true)
                    {
                        divRegularShortActive = false;
                        divSignal.Set(-3);
                    }
                    else if (divHiddenShortActive == true)
                    {
                        divHiddenShortActive = false;
                        divSignal.Set(-4);
                    }
                }
                else if (upFlip[0] == true)
                {
                    if (divRegularLongActive == true)
                    {
                        divRegularLongActive = false;
                        divSignal.Set(3);
                    }
                    else if (divHiddenLongActive == true)
                    {
                        divHiddenLongActive = false;
                        divSignal.Set(4);
                    }
                }
            }
            //=====================================================================================
            #endregion

            #region Naked swing
            //=====================================================================================
            if (showNakedSwings == true)
            {
                if (swingLow.New == true && swingLow.Update == false)
                {
                    nakedSwingHighsList.Add(swingHigh.CurPrice, swingHigh.CurBar);
                    DrawRay("NakedSwingHigh" + swingHigh.CurPrice.ToString(), false,
                        CurrentBar - swingHigh.CurBar, swingHigh.CurPrice,
                        CurrentBar - swingHigh.CurBar - 1, swingHigh.CurPrice,
                        nakedSwingHighColor, nakedSwingDashStyle,
                        nakedSwingLineWidth);
                }
                if ((swingLow.New == true || swingLow.Update == true)
                    && nakedSwingLowsList.Count > 0)
                {
                    while (nakedSwingLowsList.Count > 0
                        && nakedSwingLowsList.Keys[nakedSwingLowsList.Count
                        - 1] >= swingLow.CurPrice)
                    {
                        int counter = nakedSwingLowsList.Count - 1;
                        double nakedSwingLowPrice = nakedSwingLowsList.Keys[counter];
                        RemoveDrawObject("NakedSwingLow" + nakedSwingLowPrice.ToString());
                        if (showHistoricalNakedSwings == true)
                        {
                            DrawLine("NakedSwingLow" + nakedSwingCounter++, false,
                                CurrentBar - nakedSwingLowsList.Values[counter],
                                nakedSwingLowPrice, CurrentBar - swingLow.CurBar,
                                nakedSwingLowPrice, nakedSwingLowColor,
                                nakedSwingDashStyle, nakedSwingLineWidth);
                        }
                        nakedSwingLowsList.RemoveAt(counter);
                    }
                }
                if (swingHigh.New == true && swingHigh.Update == false)
                {
                    nakedSwingLowsList.Add(swingLow.CurPrice, swingLow.CurBar);
                    DrawRay("NakedSwingLow" + swingLow.CurPrice.ToString(), false,
                        CurrentBar - swingLow.CurBar, swingLow.CurPrice,
                        CurrentBar - swingLow.CurBar - 1, swingLow.CurPrice,
                        nakedSwingLowColor, nakedSwingDashStyle,
                        nakedSwingLineWidth);
                }
                if ((swingHigh.New == true || swingHigh.Update == true)
                    && nakedSwingHighsList.Count > 0)
                {
                    while (nakedSwingHighsList.Count > 0
                        && nakedSwingHighsList.Keys[0] <= swingHigh.CurPrice)
                    {
                        double nakedSwingHighPrice = nakedSwingHighsList.Keys[0];
                        RemoveDrawObject("NakedSwingHigh" + nakedSwingHighPrice.ToString());
                        if (showHistoricalNakedSwings == true)
                        {
                            DrawLine("NakedSwingHigh" + nakedSwingCounter++, false,
                                CurrentBar - nakedSwingHighsList.Values[0],
                                nakedSwingHighPrice, CurrentBar - swingHigh.CurBar,
                                nakedSwingHighPrice, nakedSwingHighColor,
                                nakedSwingDashStyle, nakedSwingLineWidth);
                        }
                        nakedSwingHighsList.RemoveAt(0);
                    }
                }
            }
            //=====================================================================================
            #endregion

            #region Swing switch
            //=====================================================================================
            if (showSwingSwitch == true)
            {
                if (swingLow.New == true && swingLow.Update == false)
                    DrawTriangleDown("DnSwingStart", false, 0,
                        Highs[BarsInProgress][0] + swingSwitchOffsetInTicks * TickSize, 
                        swingSwitchDownColor);

                if (swingHigh.New == true && swingHigh.Update == false)
                    DrawTriangleUp("UpSwingStart", false, 0,
                        Lows[BarsInProgress][0] - swingSwitchOffsetInTicks * TickSize, 
                        swingSwitchUpColor);
            }
            //=====================================================================================
            #endregion

            #region Alerts
            //=====================================================================================
            // Double bottom
            if (alertDoubleBottom == true && swingHigh.New == true && swingHigh.Update == false
                && swingLow.CurRelation == 0)
            {
                Alert("alertDoubleBottom" + alertTag++, alertDoubleBottomPriority,
                    alertDoubleBottomMessage + " (" + Bars.Period.ToString() + ")",
                    alertDoubleBottomSoundFileName, alertDoubleBottomRearmSeconds,
                    alertDoubleBottomBackColor, alertDoubleBottomForeColor);
            }
            //=====================================================================================
            // Double Top
            if (alertDoubleTop == true && swingLow.New == true && swingLow.Update == false
                && swingHigh.CurRelation == 0)
            {
                Alert("alertDoubleTop" + alertTag++, alertDoubleTopPriority,
                    alertDoubleTopMessage + " (" + Bars.Period.ToString() + ")",
                    alertDoubleTopSoundFileName, alertDoubleTopRearmSeconds,
                    alertDoubleTopBackColor, alertDoubleTopForeColor);
            }
            //=====================================================================================
            // Swing change
            if (alertSwingChange == true && ((swingHigh.New == true && swingHigh.Update == false)
                || (swingLow.New == true && swingLow.Update == false)))
            {
                Alert("alertSwingChange" + alertTag++, alertSwingChangePriority,
                    alertSwingChangeMessage + " (" + Bars.Period.ToString() + ")",
                    alertSwingChangeSoundFileName, alertSwingChangeRearmSeconds,
                    alertSwingChangeBackColor, alertSwingChangeForeColor);
            }
            //=====================================================================================
            // Divergence regular high
            if (alertDivergenceRegularHigh == true && swingLow.New == true
                && swingLow.Update == false && divSignal[0] == -3)
            {
                Alert("alertDivergenceRegularHigh" + alertTag++,
                    alertDivergenceRegularHighPriority,
                    alertDivergenceRegularHighMessage + " (" + Bars.Period.ToString() + ")",
                    alertDivergenceRegularHighSoundFileName,
                    alertDivergenceRegularHighRearmSeconds, alertDivergenceRegularHighBackColor,
                    alertDivergenceRegularHighForeColor);
            }
            //=====================================================================================
            // Divergence hidden high
            if (alertDivergenceHiddenHigh == true && swingLow.New == true
                && swingLow.Update == false && divSignal[0] == -4)
            {
                Alert("alertDivergenceHiddenHigh" + alertTag++,
                    alertDivergenceHiddenHighPriority,
                    alertDivergenceHiddenHighMessage + " (" + Bars.Period.ToString() + ")",
                    alertDivergenceHiddenHighSoundFileName,
                    alertDivergenceHiddenHighRearmSeconds, alertDivergenceHiddenHighBackColor,
                    alertDivergenceHiddenHighForeColor);
            }
            //=====================================================================================
            // Divergence regular low
            if (alertDivergenceRegularLow == true && swingHigh.New == true
                && swingHigh.Update == false && divSignal[0] == 3)
            {
                Alert("alertDivergenceRegularLow" + alertTag++,
                    alertDivergenceRegularLowPriority,
                    alertDivergenceRegularLowMessage + " (" + Bars.Period.ToString() + ")",
                    alertDivergenceRegularLowSoundFileName,
                    alertDivergenceRegularLowRearmSeconds, alertDivergenceRegularLowBackColor,
                    alertDivergenceRegularLowForeColor);
            }
            //=====================================================================================
            // Divergence hidden low
            if (alertDivergenceHiddenLow == true && swingHigh.New == true
                && swingHigh.Update == false && divSignal[0] == 4)
            {
                Alert("alertDivergenceHiddenLow" + alertTag++,
                    alertDivergenceHiddenLowPriority,
                    alertDivergenceHiddenLowMessage + " (" + Bars.Period.ToString() + ")",
                    alertDivergenceHiddenLowSoundFileName,
                    alertDivergenceHiddenLowRearmSeconds, alertDivergenceHiddenLowBackColor,
                    alertDivergenceHiddenLowForeColor);
            }
            //=====================================================================================
            // Higher low
            if (alertHigherLow == true && swingHigh.New == true
                && swingHigh.Update == false && swingLow.CurRelation == 1)
            {
                Alert("alertHigherLow" + alertTag++,
                    alertHigherLowPriority,
                    alertHigherLowMessage + " (" + Bars.Period.ToString() + ")",
                    alertHigherLowSoundFileName,
                    alertHigherLowRearmSeconds, alertHigherLowBackColor,
                    alertHigherLowForeColor);
            }
            //=====================================================================================
            // Lower high
            if (alertLowerHigh == true && swingLow.New == true
                && swingLow.Update == false && swingHigh.CurRelation == -1)
            {
                Alert("alertLowerHigh" + alertTag++,
                    alertLowerHighPriority,
                    alertLowerHighMessage + " (" + Bars.Period.ToString() + ")",
                    alertLowerHighSoundFileName,
                    alertLowerHighRearmSeconds, alertLowerHighBackColor,
                    alertLowerHighForeColor);
            }
            //=====================================================================================
            #endregion

            #region GomCD
            // Uncomment the next lines to use Gomi's Cumulative Delta ============================
            //if (showGomCD != ShowGomCD.False)
            //{
            //    if (swingHigh.New == true || swingHigh.Update == true)
            //    {
            //        lastLowGomCdAll = curLowGomCdAll;
            //        gomCdAllSwingLowValue = Convert.ToInt32(gomCdAllDeltaClose[0]);
            //        curLowGomCdAll = gomCdAllSwingLowValue - gomCdAllSwingHighValue;

            //        string gomCdOutput = "";
            //        switch (showGomCD)
            //        {
            //            case ShowGomCD.AbsoluteCD:
            //                gomCdOutput = TruncIntToStr(gomCdAllSwingLowValue);
            //                break;
            //            case ShowGomCD.AbsoluteCD_And_SwingCD:
            //                gomCdOutput = TruncIntToStr(gomCdAllSwingLowValue) + "  /" +
            //                    TruncIntToStr(curLowGomCdAll);
            //                break;
            //            case ShowGomCD.SwingCD:
            //                gomCdOutput = TruncIntToStr(curLowGomCdAll);
            //                break;
            //            case ShowGomCD.SwingCD_And_AbsoluteCD:
            //                gomCdOutput = TruncIntToStr(curLowGomCdAll) + " / " +
            //                    TruncIntToStr(gomCdAllSwingLowValue);
            //                break;
            //        }
            //        Color textColor = Color.Transparent;
            //        switch (swingLow.CurRelation)
            //        {
            //            case 1:
            //                textColor = swingProperties.TextColorHigherLow;
            //                break;
            //            case -1:
            //                textColor = swingProperties.TextColorLowerLow;
            //                break;
            //            case 0:
            //                textColor = swingProperties.TextColorDoubleBottom;
            //                break;
            //        }
            //        DrawText("DnGomCd" + swingLow.Counter, AutoScale, gomCdOutput,
            //            CurrentBar - swingLow.CurBar, swingLow.CurPrice, -textOffsetGomCd,
            //            textColor, textFont, StringAlignment.Center, Color.Transparent,
            //            Color.Transparent, 0);
            //    }

            //    if (swingLow.New == true || swingLow.Update == true)
            //    {
            //        lastHighGomCdAll = curHighGomCdAll;
            //        gomCdAllSwingHighValue = Convert.ToInt32(gomCdAllDeltaClose[0]);
            //        curHighGomCdAll = gomCdAllSwingHighValue - gomCdAllSwingLowValue;

            //        string gomCdOutput = "";
            //        switch (showGomCD)
            //        {
            //            case ShowGomCD.AbsoluteCD:
            //                gomCdOutput = TruncIntToStr(gomCdAllSwingHighValue);
            //                break;
            //            case ShowGomCD.AbsoluteCD_And_SwingCD:
            //                gomCdOutput = TruncIntToStr(gomCdAllSwingHighValue) + " / " +
            //                    TruncIntToStr(curHighGomCdAll);
            //                break;
            //            case ShowGomCD.SwingCD:
            //                gomCdOutput = TruncIntToStr(curHighGomCdAll);
            //                break;
            //            case ShowGomCD.SwingCD_And_AbsoluteCD:
            //                gomCdOutput = TruncIntToStr(curHighGomCdAll) + " / " +
            //                    TruncIntToStr(gomCdAllSwingHighValue);
            //                break;
            //        }
            //        Color textColor = Color.Transparent;
            //        switch (swingHigh.CurRelation)
            //        {
            //            case 1:
            //                textColor = swingProperties.TextColorHigherHigh;
            //                break;
            //            case -1:
            //                textColor = swingProperties.TextColorLowerHigh;
            //                break;
            //            case 0:
            //                textColor = swingProperties.TextColorDoubleTop;
            //                break;
            //        }
            //        DrawText("UpGomCd" + swingHigh.Counter, AutoScale, gomCdOutput,
            //            CurrentBar - swingHigh.CurBar, swingHigh.CurPrice, textOffsetGomCd,
            //            textColor, textFont, StringAlignment.Center, Color.Transparent,
            //            Color.Transparent, 0);
            //    }
            //}
            //if (gomCDCurrent)
            //{
            //    if (swingCurrent.SwingSlope == 1)
            //        DrawTextFixed("PasDelta", 
            //            "CD: " + (gomCdAllDeltaClose[0] - gomCdAllSwingLowValue),
            //            TextPosition.BottomRight);
            //    else
            //        DrawTextFixed("PasDelta",
            //            "CD: " + (gomCdAllDeltaClose[0] - gomCdAllSwingHighValue),
            //            TextPosition.BottomRight);
            //}
            // Uncomment till here ================================================================
            #endregion
        }
        //#########################################################################################
        #endregion

        #region Properties
        //#########################################################################################
        #region Plots
        /// <summary>
        /// Represents the ABC signals. -2 = short entry | -1 = possible short entry 
        /// | 0 = no signal | 1 = possible long entry | 2 = long entry
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public DataSeries AbcSignals
        {
            get { return abcSignals; }
        }
        /// <summary>
        /// Represents the divergence signals. 
        /// -4 = hidden divergence short entry | 
        /// -3 = regular divergence short entry | 
        /// -2 = hidden divergence short signal |
        /// -1 = regular divergence short signal |
        /// 0 = no signal | 
        /// 1 = regular divergence long signal |
        /// 2 = hidden divergence long signal |
        /// 3 = regular divergence long entry |
        /// 4 = hidden divergence long entry |
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public DataSeries DivergenceSignals
        {
            get { return divSignal; }
        }
        /// <summary>
        /// Indicates if the swing direction changed form down to up swing.
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public BoolSeries UpFlip
        {
            get { return upFlip; }
        }
        /// <summary>
        /// Indicates if the swing direction changed form up to down swing.
        /// </summary>
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        public BoolSeries DnFlip
        {
            get { return dnFlip; }
        }
        // Plots ==================================================================================
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DoubleBottom
        {
            get { return Values[0]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LowerLow
        {
            get { return Values[1]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries HigherLow
        {
            get { return Values[2]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DoubleTop
        {
            get { return Values[3]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LowerHigh
        {
            get { return Values[4]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries HigherHigh
        {
            get { return Values[5]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries GannSwing
        {
            get { return Values[6]; }
        }
        //=========================================================================================
        #endregion

        #region Parameters
        //=========================================================================================
        /// <summary>
        /// Represents the swing type for the swings.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Represents the swing type for the swings.")]
        [Gui.Design.DisplayName("01. Swing type")]
        public SwingStyle SwingType
        {
            get { return swingType; }
            set { swingType = value; }
        }
        /// <summary>
        /// Represents the swing size. e.g. 1 = small swings and 5 = bigger swings.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Represents the swing size. e.g. 1 = small swings and 5 = bigger swings.")]
        [Gui.Design.DisplayName("02. Swing size")]
        public double SwingSize
        {
            get { return swingSize; }
            set { swingSize = Math.Max(Double.MinValue, value); }
        }
        /// <summary>
        /// Represents the double top/-bottom strength.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Represents the double top/-bottom strength. Increase the value to get more DB/DT.")]
        [Gui.Design.DisplayName("03. Double top/-bottom strength")]
        public int DtbStrength
        {
            get { return dtbStrength; }
            set { dtbStrength = Math.Max(1, value); }
        }
        /// <summary>
        /// Indicates if high and low prices are used for the swing calculations or close values.
        /// </summary>
        [GridCategory("Parameters")]
        [Description("Indicates if high and low prices are used for the swing calculations or close values.")]
        [Gui.Design.DisplayName("04. Use close values")]
        public bool UseCloseValues
        {
            get { return useCloseValues; }
            set { useCloseValues = value; }
        }
        //=========================================================================================
        #endregion

        #region Swing features
        //=========================================================================================
        [Category("01. Features")]
        [Description("Indicates if and for which direction AB=CD patterns are computed.")]
        [Gui.Design.DisplayName("01. ABC Pattern")]
        public AbcPatternMode AbcPattern
        {
            get { return abcPattern; }
            set { abcPattern = value; }
        }
        [Category("01. Features")]
        [Description("Indicates the target in percent of the AB range for the AB=CD pattern for the risk management.")]
        [Gui.Design.DisplayName("02. ABC target in percent")]
        public double AbcTarget
        {
            get { return abcTarget; }
            set { abcTarget = Math.Max(1.0, value); }
        }
        [Category("01. Features")]
        [Description("Represents the indicator for the divergence calculations.")]
        [Gui.Design.DisplayName("11. Divergence indicator")]
        public DivergenceMode DivergenceIndicatorMode
        {
            get { return divergenceMode; }
            set { divergenceMode = value; }
        }
        [Category("01. Features")]
        [Description("Represents the direction the divergences are calculated for.")]
        [Gui.Design.DisplayName("12. Divergence long and short")]
        public DivergenceDirection DivergenceDirectionMode
        {
            get { return divergenceDirection; }
            set { divergenceDirection = value; }
        }
        [Category("01. Features")]
        [Description("Represents the first parameter for the indicator choosen in 'Divergence indicator'.")]
        [Gui.Design.DisplayName("13. Divergence Parameter 1")]
        public int Param1
        {
            get { return param1; }
            set { param1 = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Represents the first parameter for the indicator choosen in 'Divergence indicator'.")]
        [Gui.Design.DisplayName("14. Divergence Parameter 2")]
        public int Param2
        {
            get { return param2; }
            set { param2 = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Represents the first parameter for the indicator choosen in 'Divergence indicator'.")]
        [Gui.Design.DisplayName("15. Divergence Parameter 3")]
        public int Param3
        {
            get { return param3; }
            set { param3 = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Indicates if naked swing lines are shown.")]
        [Gui.Design.DisplayName("21. Naked swing lines")]
        public bool ShowNakedSwings
        {
            get { return showNakedSwings; }
            set { showNakedSwings = value; }
        }
        [Category("01. Features")]
        [Description("Indicates if and where a swing statistic is shown.")]
        [Gui.Design.DisplayName("31. Statistic")]
        public StatisticPositionStyle StatisticPosition
        {
            get { return statisticPosition; }
            set { statisticPosition = value; }
        }
        [Category("01. Features")]
        [Description("Indicates the number of swings which are used for the current swing statistic.")]
        [Gui.Design.DisplayName("32. Statistic number of swings")]
        public int StatisticLength
        {
            get { return statisticLength; }
            set { statisticLength = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Indicates if the risk management tools are shown. If ABC pattern is on, it calculates the risk reward ratio and the trade quantity.")]
        [Gui.Design.DisplayName("41. Risk management")]
        public RiskManagementStyle RiskManagementPosition
        {
            get { return riskManagementPosition; }
            set { riskManagementPosition = value; }
        }
        [Category("01. Features")]
        [Description("Indicates the number of ticks for a long stop. Used to get the current values when the risk management buttons are clicked.")]
        [Gui.Design.DisplayName("42. RM long stop in ticks")]
        public int RiskLongStopInTicks
        {
            get { return riskLongStopInTicks; }
            set { riskLongStopInTicks = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Indicates the number of ticks for a long target. Used to get the current values when the risk management buttons are clicked.")]
        [Gui.Design.DisplayName("43. RM long target in ticks")]
        public int RiskLongTargetInTicks
        {
            get { return riskLongTargetInTicks; }
            set { riskLongTargetInTicks = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Indicates the number of ticks for a short stop. Used to get the current values when the risk management buttons are clicked.")]
        [Gui.Design.DisplayName("44. RM short stop in ticks")]
        public int RiskShortStopInTicks
        {
            get { return riskShortStopInTicks; }
            set { riskShortStopInTicks = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Indicates the number of ticks for a short target. Used to get the current values when the risk management buttons are clicked.")]
        [Gui.Design.DisplayName("45. RM short target in ticks")]
        public int RiskShortTargetInTicks
        {
            get { return riskShortTargetInTicks; }
            set { riskShortTargetInTicks = Math.Max(1, value); }
        }
        [Category("01. Features")]
        [Description("Represents the account size from which the quantity for a trade is calculated based on the account risk.")]
        [Gui.Design.DisplayName("46. Account size")]
        public double AccountSize
        {
            get { return accountSize; }
            set { accountSize = Math.Max(1.0, value); }
        }
        [Category("01. Features")]
        [Description("Represents the percentage of the account size which is risked for each trade.")]
        [Gui.Design.DisplayName("47. Account risk per trade")]
        public double AccountRisk
        {
            get { return accountRisk; }
            set { accountRisk = Math.Max(0.001, value); }
        }
        [Category("01. Features")]
        [Description("Indicates if a swing extension is drawn on the chart.")]
        [Gui.Design.DisplayName("51. Swing extensions")]
        public bool AddSwingExtension
        {
            get { return addSwingExtension; }
            set { addSwingExtension = value; }
        }
        [Category("01. Features")]
        [Description("Indicates if a swing retracement is drawn on the chart for the current swing.")]
        [Gui.Design.DisplayName("52. Swing retracement (current)")]
        public bool AddSwingRetracementFast
        {
            get { return addSwingRetracementFast; }
            set { addSwingRetracementFast = value; }
        }
        [Category("01. Features")]
        [Description("Indicates if a swing retracement is drawn on the chart for the last swing.")]
        [Gui.Design.DisplayName("53. Swing retracement (last)")]
        public bool AddSwingRetracementSlow
        {
            get { return addSwingRetracementSlow; }
            set { addSwingRetracementSlow = value; }
        }
        //=========================================================================================
        #endregion

        #region Swings values
        //=========================================================================================
        [Category("02. Display")]
        [Description("Represents the swing length visualization type for the swings.")]
        [Gui.Design.DisplayName("01. Length")]
        public SwingLengthStyle SwingLengthType
        {
            get { return swingLengthType; }
            set { swingLengthType = value; }
        }
        [Category("02. Display")]
        [Description("Represents the swing duration visualization type for the swings.")]
        [Gui.Design.DisplayName("02. Duration")]
        public SwingDurationStyle SwingDurationType
        {
            get { return swingDurationType; }
            set { swingDurationType = value; }
        }
        [Category("02. Display")]
        [Description("Indicates if the swing price is shown for the swings.")]
        [Gui.Design.DisplayName("03. Price")]
        public bool ShowSwingPrice
        {
            get { return showSwingPrice; }
            set { showSwingPrice = value; }
        }
        [Category("02. Display")]
        [Description("Indicates if the swing label is shown (HH, HL, LL, LH, DB, DT).")]
        [Gui.Design.DisplayName("04. Labels")]
        public bool ShowSwingLabel
        {
            get { return showSwingLabel; }
            set { showSwingLabel = value; }
        }
        [Category("02. Display")]
        [Description("Indicates if the swing percentage in relation to the last swing is shown.")]
        [Gui.Design.DisplayName("05. Percentage")]
        public bool ShowSwingPercent
        {
            get { return showSwingPercent; }
            set { showSwingPercent = value; }
        }
        [Category("02. Display")]
        [Description("Represents the swing time visualization type for the swings. ")]
        [Gui.Design.DisplayName("06. Time")]
        public SwingTimeStyle SwingTimeType
        {
            get { return swingTimeType; }
            set { swingTimeType = value; }
        }
        [Category("02. Display")]
        [Description("Represents the swing volume visualization type for the swings. ")]
        [Gui.Design.DisplayName("07. Volume")]
        public SwingVolumeStyle SwingVolumeType
        {
            get { return swingVolumeType; }
            set { swingVolumeType = value; }
        }
        //=========================================================================================
        #endregion

        #region Visualize swings
        //=========================================================================================
        [Category("03. Visualization")]
        [Description("Represents the swing visualization type for the swings.")]
        [Gui.Design.DisplayName("01. Visualization type")]
        public VisualizationStyle VisualizationType
        {
            get { return visualizationType; }
            set { visualizationType = value; }
        }
        [Category("03. Visualization")]
        [Description("Indicates if the change from down to up or up to down swings is indicated on the chart for the swings.")]
        [Gui.Design.DisplayName("02. Swing switch")]
        public bool ShowSwingSwitch
        {
            get { return showSwingSwitch; }
            set { showSwingSwitch = value; }
        }
        [Category("03. Visualization")]
        [Description("Represents the offset in ticks for the swing switch triangles.")]
        [Gui.Design.DisplayName("03. Swing switch offset in ticks")]
        public int SwingSwitchOffsetInTicks
        {
            get { return swingSwitchOffsetInTicks; }
            set { swingSwitchOffsetInTicks = value; }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing switch down triangle.")]
        [Gui.Design.DisplayName("04. Swing switch down color")]
        public Color SwingSwitchDownColor
        {
            get { return swingSwitchDownColor; }
            set { swingSwitchDownColor = value; }
        }
        [Browsable(false)]
        public string SwingSwitchDownColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(swingSwitchDownColor); }
            set { swingSwitchDownColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing switch up triangle.")]
        [Gui.Design.DisplayName("05. Swing switch up color")]
        public Color SwingSwitchUpColor
        {
            get { return swingSwitchUpColor; }
            set { swingSwitchUpColor = value; }
        }
        [Browsable(false)]
        public string SwingSwitchUpColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(swingSwitchUpColor); }
            set { swingSwitchUpColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [XmlIgnore()]
        [Category("03. Visualization")]
        [Description("Represents the text font for the swing value output.")]
        [Gui.Design.DisplayName("06. Text font")]
        public Font TextFont
        {
            get { return textFont; }
            set { textFont = value; }
        }
        [Browsable(false)]
        public string TextFontSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableFont.ToString(textFont); }
            set { textFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing value output for higher highs.")]
        [Gui.Design.DisplayName("07. Text color higher high")]
        public Color TextColorHigherHigh
        {
            get { return textColorHigherHigh; }
            set { textColorHigherHigh = value; }
        }
        [Browsable(false)]
        public string TextColorHigherHighSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(textColorHigherHigh); }
            set { textColorHigherHigh = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing value output for lower highs.")]
        [Gui.Design.DisplayName("08. Text color lower high")]
        public Color TextColorLowerHigh
        {
            get { return textColorLowerHigh; }
            set { textColorLowerHigh = value; }
        }
        [Browsable(false)]
        public string TextColorLowerHighSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(textColorLowerHigh); }
            set { textColorLowerHigh = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing value output for double tops.")]
        [Gui.Design.DisplayName("09. Text color double top")]
        public Color TextColorDoubleTop
        {
            get { return textColorDoubleTop; }
            set { textColorDoubleTop = value; }
        }
        [Browsable(false)]
        public string TextColorDoubleTopSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(textColorDoubleTop); }
            set { textColorDoubleTop = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing value output for higher lows.")]
        [Gui.Design.DisplayName("10. Text color higher high")]
        public Color TextColorHigherLow
        {
            get { return textColorHigherLow; }
            set { textColorHigherLow = value; }
        }
        [Browsable(false)]
        public string TextColorHigherLowSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(textColorHigherLow); }
            set { textColorHigherLow = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing value output for lower lows.")]
        [Gui.Design.DisplayName("11. Text color lower high")]
        public Color TextColorLowerLow
        {
            get { return textColorLowerLow; }
            set { textColorLowerLow = value; }
        }
        [Browsable(false)]
        public string TextColorLowerLowSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(textColorLowerLow); }
            set { textColorLowerLow = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the swing value output for double bottoms.")]
        [Gui.Design.DisplayName("12. Text color double bottem")]
        public Color TextColorDoubleBottom
        {
            get { return textColorDoubleBottom; }
            set { textColorDoubleBottom = value; }
        }
        [Browsable(false)]
        public string TextColorDoubleBottomSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(textColorDoubleBottom); }
            set { textColorDoubleBottom = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the text offset in pixel for the swing length/duration.")]
        [Gui.Design.DisplayName("13. Text offset length / duration")]
        public int TextOffsetLength
        {
            get { return textOffsetLength; }
            set { textOffsetLength = Math.Max(1, value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the text offset in pixel for the swing volume.")]
        [Gui.Design.DisplayName("14. Text offset volume")]
        public int TextOffsetVolume
        {
            get { return textOffsetVolume; }
            set { textOffsetVolume = Math.Max(1, value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the text offset in pixel for the swing price for the swings.")]
        [Gui.Design.DisplayName("15. Text offset price")]
        public int TextOffsetPrice
        {
            get { return textOffsetPrice; }
            set { textOffsetPrice = Math.Max(1, value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the text offset in pixel for the swing labels.")]
        [Gui.Design.DisplayName("13. Text offset swing labels")]
        public int TextOffsetLabel
        {
            get { return textOffsetLabel; }
            set { textOffsetLabel = Math.Max(1, value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the text offset in pixel for the time value.")]
        [Gui.Design.DisplayName("16. Text offset time")]
        public int TextOffsetTime
        {
            get { return textOffsetTime; }
            set { textOffsetTime = Math.Max(1, value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the text offset in pixel for the retracement value.")]
        [Gui.Design.DisplayName("17. Text offset percent")]
        public int TextOffsetPercent
        {
            get { return textOffsetPercent; }
            set { textOffsetPercent = Math.Max(1, value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the zig-zag up lines.")]
        [Gui.Design.DisplayName("18. Zig-Zag color up")]
        public Color ZigZagColorUp
        {
            get { return zigZagColorUp; }
            set { zigZagColorUp = value; }
        }
        [Browsable(false)]
        public string ZigZagColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(zigZagColorUp); }
            set { zigZagColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the color of the zig-zag down lines.")]
        [Gui.Design.DisplayName("19. Zig-Zag color down")]
        public Color ZigZagColorDn
        {
            get { return zigZagColorDn; }
            set { zigZagColorDn = value; }
        }
        [Browsable(false)]
        public string ZigZagColorDnSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(zigZagColorDn); }
            set { zigZagColorDn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("03. Visualization")]
        [Description("Represents the line style of the zig-zag lines.")]
        [Gui.Design.DisplayName("20. Zig-Zag style")]
        public DashStyle ZigZagStyle
        {
            get { return zigZagStyle; }
            set { zigZagStyle = value; }
        }
        [Category("03. Visualization")]
        [Description("Represents the line width of the zig-zag lines.")]
        [Gui.Design.DisplayName("21. Zig-Zag width")]
        public int ZigZagWidth
        {
            get { return zigZagWidth; }
            set { zigZagWidth = Math.Max(1, value); }
        }
        //=====================================================================
        #endregion

        #region ABC visualization
        //=====================================================================
        [Category("04. ABC Visualization")]
        [Description("Represents the line style for pattern lines.")]
        [Gui.Design.DisplayName("01. Line style")]
        public DashStyle AbcLineStyle
        {
            get { return abcLineStyle; }
            set { abcLineStyle = value; }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the line style for pattern ratio lines.")]
        [Gui.Design.DisplayName("02. Line style ratio")]
        public DashStyle AbcLineStyleRatio
        {
            get { return abcLineStyleRatio; }
            set { abcLineStyleRatio = value; }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the line width for pattern lines.")]
        [Gui.Design.DisplayName("03. Line width")]
        public int AbcLineWidth
        {
            get { return abcLineWidth; }
            set { abcLineWidth = Math.Max(1, value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the line width for pattern ratio lines.")]
        [Gui.Design.DisplayName("04. Line width ratio")]
        public int AbcLineWidthRatio
        {
            get { return abcLineWidthRatio; }
            set { abcLineWidthRatio = Math.Max(1, value); }
        }
        [XmlIgnore()]
        [Category("04. ABC Visualization")]
        [Description("Represents the text font for the displayed swing information.")]
        [Gui.Design.DisplayName("05. Text font")]
        public Font AbcTextFont
        {
            get { return abcTextFont; }
            set { abcTextFont = value; }
        }
        [Browsable(false)]
        public string AbcTextFontSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableFont.ToString(abcTextFont); }
            set { abcTextFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the offset value in pixels from within the text box area that display the swing label.")]
        [Gui.Design.DisplayName("06. Text offset label")]
        public int AbcTextOffsetLabel
        {
            get { return abcTextOffsetLabel; }
            set { abcTextOffsetLabel = Math.Max(1, value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the text color for down patterns.")]
        [Gui.Design.DisplayName("07. Text color down")]
        public Color AbcTextColorDn
        {
            get { return abcTextColorDn; }
            set { abcTextColorDn = value; }
        }
        [Browsable(false)]
        public string AbcTextColorDnSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(abcTextColorDn); }
            set { abcTextColorDn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the text color for up patterns.")]
        [Gui.Design.DisplayName("08. Text color up")]
        public Color AbcTextColorUp
        {
            get { return abcTextColorUp; }
            set { abcTextColorUp = value; }
        }
        [Browsable(false)]
        public string AbcTextColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(abcTextColorUp); }
            set { abcTextColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the line color for down patterns.")]
        [Gui.Design.DisplayName("09. Line color down")]
        public Color AbcZigZagColorDn
        {
            get { return abcZigZagColorDn; }
            set { abcZigZagColorDn = value; }
        }
        [Browsable(false)]
        public string AbcZigZagColorDnSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(abcZigZagColorDn); }
            set { abcZigZagColorDn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the line color for up patterns.")]
        [Gui.Design.DisplayName("10. Line color up")]
        public Color AbcZigZagColorUp
        {
            get { return abcZigZagColorUp; }
            set { abcZigZagColorUp = value; }
        }
        [Browsable(false)]
        public string AbcZigZagColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(abcZigZagColorUp); }
            set { abcZigZagColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the maximum value in percent for a retracement in relation to the last swing. The price must retrace between this two values, otherwise the pattern is not valid.")]
        [Gui.Design.DisplayName("11. Retracement maximum (percent)")]
        public double AbcMaxRetracement
        {
            get { return abcMaxRetracement; }
            set { abcMaxRetracement = Math.Max(1, Math.Min(99, value)); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the minimum value in percent for a retracement in relation to the last swing. The price must retrace between this two values, otherwise the pattern is not valid.")]
        [Gui.Design.DisplayName("12. Retracement minimum (percent)")]
        public double AbcMinRetracement
        {
            get { return abcMinRetracement; }
            set { abcMinRetracement = Math.Max(1, Math.Min(99, value)); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the entry line color for down patterns.")]
        [Gui.Design.DisplayName("13. Entry line color down")]
        public Color EntryLineColorDn
        {
            get { return entryLineColorDn; }
            set { entryLineColorDn = value; }
        }
        [Browsable(false)]
        public string EntryLineColorDnSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(entryLineColorDn); }
            set { entryLineColorDn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the entry line color for up patterns.")]
        [Gui.Design.DisplayName("14. Entry line color up")]
        public Color EntryLineColorUp
        {
            get { return entryLineColorUp; }
            set { entryLineColorUp = value; }
        }
        [Browsable(false)]
        public string EntryLineColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(entryLineColorUp); }
            set { entryLineColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the line style for the entry lines.")]
        [Gui.Design.DisplayName("15. Entry line style")]
        public DashStyle EntryLineStyle
        {
            get { return entryLineStyle; }
            set { entryLineStyle = value; }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the line width for pattern lines.")]
        [Gui.Design.DisplayName("16. Entry line width")]
        public int EntryLineWidth
        {
            get { return entryLineWidth; }
            set { entryLineWidth = Math.Max(1, value); }
        }
        [Category("04. ABC Visualization")]
        [Description("If bar close above/below the entry retracement an entry is triggered.")]
        [Gui.Design.DisplayName("17. Entry retracement")]
        public double RetracementEntryValue
        {
            get { return retracementEntryValue; }
            set { retracementEntryValue = Math.Max(1.0, Math.Min(99.0, value)); }
        }
        [Category("04. ABC Visualization")]
        [Description("Indicates if entry arrows are displayed.")]
        [Gui.Design.DisplayName("18. Entry arrows")]
        public bool ShowEntryArrows
        {
            get { return showEntryArrows; }
            set { showEntryArrows = value; }
        }
        [Category("04. ABC Visualization")]
        [Description("Indicates if historical entry lines are displayed.")]
        [Gui.Design.DisplayName("19. Entry line historical")]
        public bool ShowHistoricalEntryLine
        {
            get { return showHistoricalEntryLine; }
            set { showHistoricalEntryLine = value; }
        }
        [Category("04. ABC Visualization")]
        [Description("Represents the offset value in ticks from the high/low that triggered the entry.")]
        [Gui.Design.DisplayName("20. Entry arrows offset (ticks)")]
        public int YTickOffset
        {
            get { return yTickOffset; }
            set { yTickOffset = Math.Max(0, value); }
        }
        //=====================================================================
        #endregion

        #region Divergence Visualization
        //=========================================================================================
        [Category("05. Divergence Visualization")]
        [Description("Indicates if regalur divergence is shown.")]
        [Gui.Design.DisplayName("01. Show regular divergence")]
        public bool ShowDivergenceRegular
        {
            get { return showDivergenceRegular; }
            set { showDivergenceRegular = value; }
        }
        [Category("05. Divergence Visualization")]
        [Description("Indicates if hidden divergence is shown.")]
        [Gui.Design.DisplayName("02. Show hidden divergence")]
        public bool ShowDivergenceHidden
        {
            get { return showDivergenceHidden; }
            set { showDivergenceHidden = value; }
        }
        [Category("05. Divergence Visualization")]
        [Description("Represents the line style for down divergence.")]
        [Gui.Design.DisplayName("03. Down line style")]
        public DashStyle DivDnLineStyle
        {
            get { return divDnLineStyle; }
            set { divDnLineStyle = value; }
        }
        [Category("05. Divergence Visualization")]
        [Description("Represents the line style for hidden divergence.")]
        [Gui.Design.DisplayName("04. Up line style")]
        public DashStyle DivUpLineStyle
        {
            get { return divUpLineStyle; }
            set { divUpLineStyle = value; }
        }
        [Category("05. Divergence Visualization")]
        [Description("Represents the line width for regular divergence.")]
        [Gui.Design.DisplayName("05. Down line width")]
        public int DivDnLineWidth
        {
            get { return divDnLineWidth; }
            set { divDnLineWidth = Math.Max(1, value); }
        }
        [Category("05. Divergence Visualization")]
        [Description("Represents the line width for hidden divergence.")]
        [Gui.Design.DisplayName("06. Up line width")]
        public int DivUpLineWidth
        {
            get { return divUpLineWidth; }
            set { divUpLineWidth = Math.Max(1, value); }
        }
        [Category("05. Divergence Visualization")]
        [Description("Represents the text color for regular divergence.")]
        [Gui.Design.DisplayName("07. Down text color")]
        public Color DivDnLineColor
        {
            get { return divDnLineColor; }
            set { divDnLineColor = value; }
        }
        [Browsable(false)]
        public string DivDnLineColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(divDnLineColor); }
            set { divDnLineColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("05. Divergence Visualization")]
        [Description("Represents the text color for hidden divergence.")]
        [Gui.Design.DisplayName("08. Up text color")]
        public Color DivUpLineColor
        {
            get { return divUpLineColor; }
            set { divUpLineColor = value; }
        }
        [Browsable(false)]
        public string DivUpLineColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(divUpLineColor); }
            set { divUpLineColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //=========================================================================================
        #endregion

        #region Naked swings visualization
        //=========================================================================================
        [Category("06. Naked swings")]
        [Description("Indicates if historical naked swing lines are shown.")]
        [Gui.Design.DisplayName("01. Show historical naked swing lines")]
        public bool ShowHistoricalNakedSwings
        {
            get { return showHistoricalNakedSwings; }
            set { showHistoricalNakedSwings = value; }
        }
        [Category("06. Naked swings")]
        [Description("Represents the color of the naked swing high lines.")]
        [Gui.Design.DisplayName("02. Naked swing high color")]
        public Color NakedSwingHighColor
        {
            get { return nakedSwingHighColor; }
            set { nakedSwingHighColor = value; }
        }
        [Browsable(false)]
        public string NakedSwingHighColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(nakedSwingHighColor); }
            set { nakedSwingHighColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("06. Naked swings")]
        [Description("Represents the color of the naked swing low lines.")]
        [Gui.Design.DisplayName("03. Naked swing low color")]
        public Color NakedSwingLowColor
        {
            get { return nakedSwingLowColor; }
            set { nakedSwingLowColor = value; }
        }
        [Browsable(false)]
        public string NakedSwingLowColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(nakedSwingLowColor); }
            set { nakedSwingLowColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("06. Naked swings")]
        [Description("Represents the line style of the naked swing lines.")]
        [Gui.Design.DisplayName("04. Naked swing line style")]
        public DashStyle NakedSwingDashStyle
        {
            get { return nakedSwingDashStyle; }
            set { nakedSwingDashStyle = value; }
        }
        [Category("06. Naked swings")]
        [Description("Represents the line width of the naked swing lines.")]
        [Gui.Design.DisplayName("05. Naked swing line width")]
        public int NakedSwingLineWidth
        {
            get { return nakedSwingLineWidth; }
            set { nakedSwingLineWidth = Math.Max(1, value); }
        }
        //=========================================================================================
        #endregion

        #region Gann Swings
        //=========================================================================================
        [Category("07. Gann Swings")]
        [Description("Indicates if inside bars are ignored. If set to true it is possible that between consecutive up/down bars are inside bars. Only used if calculationSize > 1.")]
        [Gui.Design.DisplayName("01. Ignore inside Bars")]
        public bool IgnoreInsideBars
        {
            get { return ignoreInsideBars; }
            set { ignoreInsideBars = value; }
        }
        [Category("07. Gann Swings")]
        [Description("Indicates if the swings are updated if the last swing high/low is broken. Only used if calculationSize > 1.")]
        [Gui.Design.DisplayName("02. Use breakouts")]
        public bool UseBreakouts
        {
            get { return useBreakouts; }
            set { useBreakouts = value; }
        }
        //=========================================================================================
        #endregion

        #region Alerts
        //=========================================================================================
        #region ABC
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("01. Alert ABC")]
        public bool AlertAbc
        {
            get { return alertAbc; }
            set { alertAbc = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("02. Alert ABC entry")]
        public bool AlertAbcEntry
        {
            get { return alertAbcEntry; }
            set { alertAbcEntry = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("03. Priority")]
        public NinjaTrader.Cbi.Priority AlertAbcPriority
        {
            get { return alertAbcPriority; }
            set { alertAbcPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("04. Entry priority")]
        public NinjaTrader.Cbi.Priority AlertAbcEntryPriority
        {
            get { return alertAbcEntryPriority; }
            set { alertAbcEntryPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("05. Long sound file name")]
        public string AlertAbcLongSoundFileName
        {
            get { return alertAbcLongSoundFileName; }
            set { alertAbcLongSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("06. Long entry sound file name")]
        public string AlertAbcLongEntrySoundFileName
        {
            get { return alertAbcLongEntrySoundFileName; }
            set { alertAbcLongEntrySoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("07. Short sound file name")]
        public string AlertAbcShortSoundFileName
        {
            get { return alertAbcShortSoundFileName; }
            set { alertAbcShortSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("08. Short entry sound file name")]
        public string AlertAbcShortEntrySoundFileName
        {
            get { return alertAbcShortEntrySoundFileName; }
            set { alertAbcShortEntrySoundFileName = value; }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Double bottem
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("11. Alert double bottem")]
        public bool AlertDoubleBottom
        {
            get { return alertDoubleBottom; }
            set { alertDoubleBottom = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("12. Priority")]
        public NinjaTrader.Cbi.Priority AlertDoubleBottomPriority
        {
            get { return alertDoubleBottomPriority; }
            set { alertDoubleBottomPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("13. Message")]
        public string AlertDoubleBottomMessage
        {
            get { return alertDoubleBottomMessage; }
            set { alertDoubleBottomMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("14. Sound file name")]
        public string AlertDoubleBottomSoundFileName
        {
            get { return alertDoubleBottomSoundFileName; }
            set { alertDoubleBottomSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("15. Rearm seconds")]
        public int AlertDoubleBottomRearmSeconds
        {
            get { return alertDoubleBottomRearmSeconds; }
            set { alertDoubleBottomRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("16. Back color")]
        public Color AlertDoubleBottomBackColor
        {
            get { return alertDoubleBottomBackColor; }
            set { alertDoubleBottomBackColor = value; }
        }
        [Browsable(false)]
        public string AlertDoubleBottomBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDoubleBottomBackColor); }
            set { alertDoubleBottomBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("17. Fore color")]
        public Color AlertDoubleBottomForeColor
        {
            get { return alertDoubleBottomForeColor; }
            set { alertDoubleBottomForeColor = value; }
        }
        [Browsable(false)]
        public string AlertDoubleBottomForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDoubleBottomForeColor); }
            set { alertDoubleBottomForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Double top
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("21. Alert double top")]
        public bool AlertDoubleTop
        {
            get { return alertDoubleTop; }
            set { alertDoubleTop = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("22. Priority")]
        public NinjaTrader.Cbi.Priority AlertDoubleTopPriority
        {
            get { return alertDoubleTopPriority; }
            set { alertDoubleTopPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("23. Message")]
        public string AlertDoubleTopMessage
        {
            get { return alertDoubleTopMessage; }
            set { alertDoubleTopMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("24. Sound file name")]
        public string AlertDoubleTopSoundFileName
        {
            get { return alertDoubleTopSoundFileName; }
            set { alertDoubleTopSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("25. Rearm seconds")]
        public int AlertDoubleTopRearmSeconds
        {
            get { return alertDoubleTopRearmSeconds; }
            set { alertDoubleTopRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("26. Back color")]
        public Color AlertDoubleTopBackColor
        {
            get { return alertDoubleTopBackColor; }
            set { alertDoubleTopBackColor = value; }
        }
        [Browsable(false)]
        public string AlertDoubleTopBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDoubleTopBackColor); }
            set { alertDoubleTopBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("27. Fore color")]
        public Color AlertDoubleTopForeColor
        {
            get { return alertDoubleTopForeColor; }
            set { alertDoubleTopForeColor = value; }
        }
        [Browsable(false)]
        public string AlertDoubleTopForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDoubleTopForeColor); }
            set { alertDoubleTopForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Higher low
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("31. Alert higher low")]
        public bool AlertHigherLow
        {
            get { return alertHigherLow; }
            set { alertHigherLow = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("32. Priority")]
        public NinjaTrader.Cbi.Priority AlertHigherLowPriority
        {
            get { return alertHigherLowPriority; }
            set { alertHigherLowPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("33. Message")]
        public string AlertHigherLowMessage
        {
            get { return alertHigherLowMessage; }
            set { alertHigherLowMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("34. Sound file name")]
        public string AlertHigherLowSoundFileName
        {
            get { return alertHigherLowSoundFileName; }
            set { alertHigherLowSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("35. Rearm seconds")]
        public int AlertHigherLowRearmSeconds
        {
            get { return alertHigherLowRearmSeconds; }
            set { alertHigherLowRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("36. Back color")]
        public Color AlertHigherLowBackColor
        {
            get { return alertHigherLowBackColor; }
            set { alertHigherLowBackColor = value; }
        }
        [Browsable(false)]
        public string AlertHigherLowBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertHigherLowBackColor); }
            set { alertHigherLowBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("37. Fore color")]
        public Color AlertHigherLowForeColor
        {
            get { return alertHigherLowForeColor; }
            set { alertHigherLowForeColor = value; }
        }
        [Browsable(false)]
        public string AlertHigherLowForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertHigherLowForeColor); }
            set { alertHigherLowForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Lower high
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("41. Alert lower high")]
        public bool AlertLowerHigh
        {
            get { return alertLowerHigh; }
            set { alertLowerHigh = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("42. Priority")]
        public NinjaTrader.Cbi.Priority AlertLowerHighPriority
        {
            get { return alertLowerHighPriority; }
            set { alertLowerHighPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("43. Message")]
        public string AlertLowerHighMessage
        {
            get { return alertLowerHighMessage; }
            set { alertLowerHighMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("44. Sound file name")]
        public string AlertLowerHighSoundFileName
        {
            get { return alertLowerHighSoundFileName; }
            set { alertLowerHighSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("45. Rearm seconds")]
        public int AlertLowerHighRearmSeconds
        {
            get { return alertLowerHighRearmSeconds; }
            set { alertLowerHighRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("46. Back color")]
        public Color AlertLowerHighBackColor
        {
            get { return alertLowerHighBackColor; }
            set { alertLowerHighBackColor = value; }
        }
        [Browsable(false)]
        public string AlertLowerHighBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertLowerHighBackColor); }
            set { alertLowerHighBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("47. Fore color")]
        public Color AlertLowerHighForeColor
        {
            get { return alertLowerHighForeColor; }
            set { alertLowerHighForeColor = value; }
        }
        [Browsable(false)]
        public string AlertLowerHighForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertLowerHighForeColor); }
            set { alertLowerHighForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Swing change
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("51. Alert swing change")]
        public bool AlertSwingChange
        {
            get { return alertSwingChange; }
            set { alertSwingChange = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("52. Priority")]
        public NinjaTrader.Cbi.Priority AlertSwingChangePriority
        {
            get { return alertSwingChangePriority; }
            set { alertSwingChangePriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("53. Message")]
        public string AlertSwingChangeMessage
        {
            get { return alertSwingChangeMessage; }
            set { alertSwingChangeMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("54. Sound file name")]
        public string AlertSwingChangeSoundFileName
        {
            get { return alertSwingChangeSoundFileName; }
            set { alertSwingChangeSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("55. Rearm seconds")]
        public int AlertSwingChangeRearmSeconds
        {
            get { return alertSwingChangeRearmSeconds; }
            set { alertSwingChangeRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("56. Back color")]
        public Color AlertSwingChangeBackColor
        {
            get { return alertSwingChangeBackColor; }
            set { alertSwingChangeBackColor = value; }
        }
        [Browsable(false)]
        public string AlertSwingChangeBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertSwingChangeBackColor); }
            set { alertSwingChangeBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("57. Fore color")]
        public Color AlertSwingChangeForeColor
        {
            get { return alertSwingChangeForeColor; }
            set { alertSwingChangeForeColor = value; }
        }
        [Browsable(false)]
        public string AlertSwingChangeForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertSwingChangeForeColor); }
            set { alertSwingChangeForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Divergence regular high
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("61. Alert divergence regular high")]
        public bool AlertDivergenceRegularHigh
        {
            get { return alertDivergenceRegularHigh; }
            set { alertDivergenceRegularHigh = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("62. Priority")]
        public NinjaTrader.Cbi.Priority AlertDivergenceRegularHighPriority
        {
            get { return alertDivergenceRegularHighPriority; }
            set { alertDivergenceRegularHighPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("63. Message")]
        public string AlertDivergenceRegularHighMessage
        {
            get { return alertDivergenceRegularHighMessage; }
            set { alertDivergenceRegularHighMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("64. Sound file name")]
        public string AlertDivergenceRegularHighSoundFileName
        {
            get { return alertDivergenceRegularHighSoundFileName; }
            set { alertDivergenceRegularHighSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("65. Rearm seconds")]
        public int AlertDivergenceRegularHighRearmSeconds
        {
            get { return alertDivergenceRegularHighRearmSeconds; }
            set { alertDivergenceRegularHighRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("66. Back color")]
        public Color AlertDivergenceRegularHighBackColor
        {
            get { return alertDivergenceRegularHighBackColor; }
            set { alertDivergenceRegularHighBackColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceRegularHighBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceRegularHighBackColor); }
            set { alertDivergenceRegularHighBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("67. Fore color")]
        public Color AlertDivergenceRegularHighForeColor
        {
            get { return alertDivergenceRegularHighForeColor; }
            set { alertDivergenceRegularHighForeColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceRegularHighForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceRegularHighForeColor); }
            set { alertDivergenceRegularHighForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Divergence hidden high
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("71. Alert divergence hidden high")]
        public bool AlertDivergenceHiddenHigh
        {
            get { return alertDivergenceHiddenHigh; }
            set { alertDivergenceHiddenHigh = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("72. Priority")]
        public NinjaTrader.Cbi.Priority AlertDivergenceHiddenHighPriority
        {
            get { return alertDivergenceHiddenHighPriority; }
            set { alertDivergenceHiddenHighPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("73. Message")]
        public string AlertDivergenceHiddenHighMessage
        {
            get { return alertDivergenceHiddenHighMessage; }
            set { alertDivergenceHiddenHighMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("74. Sound file name")]
        public string AlertDivergenceHiddenHighSoundFileName
        {
            get { return alertDivergenceHiddenHighSoundFileName; }
            set { alertDivergenceHiddenHighSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("75. Rearm seconds")]
        public int AlertDivergenceHiddenHighRearmSeconds
        {
            get { return alertDivergenceHiddenHighRearmSeconds; }
            set { alertDivergenceHiddenHighRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("76. Back color")]
        public Color AlertDivergenceHiddenHighBackColor
        {
            get { return alertDivergenceHiddenHighBackColor; }
            set { alertDivergenceHiddenHighBackColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceHiddenHighBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceHiddenHighBackColor); }
            set { alertDivergenceHiddenHighBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("77. Fore color")]
        public Color AlertDivergenceHiddenHighForeColor
        {
            get { return alertDivergenceHiddenHighForeColor; }
            set { alertDivergenceHiddenHighForeColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceHiddenHighForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceHiddenHighForeColor); }
            set { alertDivergenceHiddenHighForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Divergence regular low
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("81. Alert divergence regular low")]
        public bool AlertDivergenceRegularLow
        {
            get { return alertDivergenceRegularLow; }
            set { alertDivergenceRegularLow = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("82. Priority")]
        public NinjaTrader.Cbi.Priority AlertDivergenceRegularLowPriority
        {
            get { return alertDivergenceRegularLowPriority; }
            set { alertDivergenceRegularLowPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("83. Message")]
        public string AlertDivergenceRegularLowMessage
        {
            get { return alertDivergenceRegularLowMessage; }
            set { alertDivergenceRegularLowMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("84. Sound file name")]
        public string AlertDivergenceRegularLowSoundFileName
        {
            get { return alertDivergenceRegularLowSoundFileName; }
            set { alertDivergenceRegularLowSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("85. Rearm seconds")]
        public int AlertDivergenceRegularLowRearmSeconds
        {
            get { return alertDivergenceRegularLowRearmSeconds; }
            set { alertDivergenceRegularLowRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("86. Back color")]
        public Color AlertDivergenceRegularLowBackColor
        {
            get { return alertDivergenceRegularLowBackColor; }
            set { alertDivergenceRegularLowBackColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceRegularLowBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceRegularLowBackColor); }
            set { alertDivergenceRegularLowBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("87. Fore color")]
        public Color AlertDivergenceRegularLowForeColor
        {
            get { return alertDivergenceRegularLowForeColor; }
            set { alertDivergenceRegularLowForeColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceRegularLowForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceRegularLowForeColor); }
            set { alertDivergenceRegularLowForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion

        #region Divergence hidden low
        //-----------------------------------------------------------------------------------------
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("91. Alert divergence hidden low")]
        public bool AlertDivergenceHiddenLow
        {
            get { return alertDivergenceHiddenLow; }
            set { alertDivergenceHiddenLow = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("92. Priority")]
        public NinjaTrader.Cbi.Priority AlertDivergenceHiddenLowPriority
        {
            get { return alertDivergenceHiddenLowPriority; }
            set { alertDivergenceHiddenLowPriority = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("93. Message")]
        public string AlertDivergenceHiddenLowMessage
        {
            get { return alertDivergenceHiddenLowMessage; }
            set { alertDivergenceHiddenLowMessage = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("94. Sound file name")]
        public string AlertDivergenceHiddenLowSoundFileName
        {
            get { return alertDivergenceHiddenLowSoundFileName; }
            set { alertDivergenceHiddenLowSoundFileName = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("95. Rearm seconds")]
        public int AlertDivergenceHiddenLowRearmSeconds
        {
            get { return alertDivergenceHiddenLowRearmSeconds; }
            set { alertDivergenceHiddenLowRearmSeconds = value; }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("96. Back color")]
        public Color AlertDivergenceHiddenLowBackColor
        {
            get { return alertDivergenceHiddenLowBackColor; }
            set { alertDivergenceHiddenLowBackColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceHiddenLowBackColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceHiddenLowBackColor); }
            set { alertDivergenceHiddenLowBackColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Category("08. Alerts")]
        [Gui.Design.DisplayName("97. Fore color")]
        public Color AlertDivergenceHiddenLowForeColor
        {
            get { return alertDivergenceHiddenLowForeColor; }
            set { alertDivergenceHiddenLowForeColor = value; }
        }
        [Browsable(false)]
        public string AlertDivergenceHiddenLowForeColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertDivergenceHiddenLowForeColor); }
            set { alertDivergenceHiddenLowForeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        //-----------------------------------------------------------------------------------------
        #endregion
        //=========================================================================================
        #endregion

        #region GomCD
        // Uncomment the next lines to use Gomi's Cumulative Delta ================================
        //[Category("09. GomCD")]
        //[Description("To use GomCD you have to uncomment all 'GomCD regions' in the source code.")]
        //[Gui.Design.DisplayName("01. GomCD mode")]
        //public ShowGomCD ShowGomCDMode
        //{
        //    get { return showGomCD; }
        //    set { showGomCD = value; }
        //}
        //[Category("09. GomCD")]
        //[Description("Represents the text offset in pixel for the GomCD values.")]
        //[Gui.Design.DisplayNameAttribute("02. Text offset")]
        //public int TextOffsetGomCd
        //{
        //    get { return textOffsetGomCd; }
        //    set { textOffsetGomCd = Math.Max(1, value); }
        //}
        //[Category("09. GomCD")]
        //[Description("Indicates if the developing cumulative delta since the last fix swing is printed in the right bottem corner.")]
        //[Gui.Design.DisplayNameAttribute("03. Show current CD")]
        //public bool GomCDCurrent
        //{
        //    get { return gomCDCurrent; }
        //    set { gomCDCurrent = value; }
        //}
        //[Category("09. GomCD")]
        //[Description("Indicates which calculation mode is used.")]
        //[Gui.Design.DisplayName("04. Calculation mode")]
        //public GomCDCalculationModeType GomCdCalcMode
        //{
        //    get { return gomCdCalcMode; }
        //    set { gomCdCalcMode = value; }
        //}
        //[Category("09. GomCD")]
        //[Description("Indicates which volume filter mode is used.")]
        //[Gui.Design.DisplayName("05. Volume filter mode")]
        //public GomFilterModeType GomFilterModeType
        //{
        //    get { return gomCdVolumeFilterSizeMode; }
        //    set { gomCdVolumeFilterSizeMode = value; }
        //}
        //[Category("09. GomCD")]
        //[Description("Represents the volume filter size.")]
        //[Gui.Design.DisplayName("06. Volume filter size")]
        //public int GomCdVolumeFilterSize
        //{
        //    get { return gomCdVolumeFilterSize; }
        //    set { gomCdVolumeFilterSize = Math.Max(1, value); }
        //}
        // Uncomment till here ====================================================================
        #endregion
        //#########################################################################################
        #endregion

        #region ButtonClickShowStatistic
        //#########################################################################################
        /// <summary>
        /// Show or hide the statistic.
        /// </summary>
        private void ButtonClickShowStatistic(object s, EventArgs e)
        {
            if (panel.Visible)
            {
                panel.Hide();
                splitter.Hide();
                toolStripButton.Text = "Show Statistic";

            }
            else
            {
                panel.Show();
                splitter.Show();
                toolStripButton.Text = "Hide Statistic";
            }
        }
        //#########################################################################################
        #endregion

        #region CalculateQuantityEnterLong
        //#########################################################################################
        /// <summary>
        /// Calculates the risk, if a the numericUpDown value changed .
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CalculateQuantityEnterLong(object sender, EventArgs e)
        {
            double entryPrice = GetCurrentAsk(BarsInProgress);
            double stopLossPrice = entryPrice - riskLongStopInTicks * TickSize;
            double takeProfitPrice = entryPrice + riskLongTargetInTicks * TickSize;

            double loss = Math.Round(((entryPrice - stopLossPrice) *
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            int quantity = (int)Math.Truncate(accountSize / 100.0 * accountRisk / loss);
            double win = Math.Round(((takeProfitPrice - entryPrice) *
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            double riskReward = Math.Round(win / loss, 2, MidpointRounding.AwayFromZero);

            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                this.toolStripNumericUpDownEntry.Value = (decimal)entryPrice;
                this.toolStripNumericUpDownStop.Value = (decimal)stopLossPrice;
                this.toolStripNumericUpDownTarget.Value = (decimal)takeProfitPrice;

                this.toolStripLabelQty.Text = "Qty: " + quantity;
                this.toolStripLabelQty.ForeColor = Color.Green;

                this.toolStripLabelRiskReward.Text = "R/R: " + riskReward;
                this.toolStripLabelLoss.Text = "Loss: " + loss;
                this.toolStripLabelProfit.Text = "Win: " + win;
                if (riskReward < 1.0)
                    this.toolStripLabelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.toolStripLabelRiskReward.ForeColor = Color.Black;
                else
                    this.toolStripLabelRiskReward.ForeColor = Color.Green;
            }
            else
            {
                this.numericUpDownEntry.Value = (decimal)entryPrice;
                this.numericUpDownStopLoss.Value = (decimal)stopLossPrice;
                this.numericUpDownTakeProfit.Value = (decimal)takeProfitPrice;
                this.labelQuantity.Text = "Qty: " + quantity;
                this.labelQuantity.ForeColor = Color.Green;

                this.labelRiskReward.Text = "R/R: " + riskReward;
                this.labelLossValue.Text = "-" + String.Format("{0:F" + 2 + "}", loss);
                this.labelWinValue.Text = String.Format("{0:F" + 2 + "}", win);
                if (riskReward < 1.0)
                    this.labelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.labelRiskReward.ForeColor = Color.Black;
                else
                    this.labelRiskReward.ForeColor = Color.Green;
            }
        }
        //#########################################################################################
        #endregion

        #region CalculateQuantityEnterShort
        //#########################################################################################
        /// <summary>
        /// Calculates the risk, if a the numericUpDown value changed .
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CalculateQuantityEnterShort(object sender, EventArgs e)
        {
            double entryPrice = GetCurrentBid(BarsInProgress);
            double stopLossPrice = entryPrice + riskShortStopInTicks * TickSize;
            double takeProfitPrice = entryPrice - riskShortTargetInTicks * TickSize;

            double loss = Math.Round(((stopLossPrice - entryPrice) *
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            int quantity = (int)Math.Truncate(accountSize / 100.0 * accountRisk / loss);
            double win = Math.Round(((entryPrice - takeProfitPrice) *
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            double riskReward = Math.Round(win / loss, 2, MidpointRounding.AwayFromZero);

            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                this.toolStripNumericUpDownEntry.Value = (decimal)entryPrice;
                this.toolStripNumericUpDownStop.Value = (decimal)stopLossPrice;
                this.toolStripNumericUpDownTarget.Value = (decimal)takeProfitPrice;

                this.toolStripLabelQty.Text = "Qty: " + quantity;
                this.toolStripLabelQty.ForeColor = Color.Red;

                this.toolStripLabelRiskReward.Text = "R/R: " + riskReward;
                this.toolStripLabelLoss.Text = "Loss: " + loss;
                this.toolStripLabelProfit.Text = "Win: " + win;
                if (riskReward < 1.0)
                    this.toolStripLabelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.toolStripLabelRiskReward.ForeColor = Color.Black;
                else
                    this.toolStripLabelRiskReward.ForeColor = Color.Green;
            }
            else
            {
                this.numericUpDownEntry.Value = (decimal)entryPrice;
                this.numericUpDownStopLoss.Value = (decimal)stopLossPrice;
                this.numericUpDownTakeProfit.Value = (decimal)takeProfitPrice;

                this.labelQuantity.Text = "Qty: " + quantity;
                this.labelQuantity.ForeColor = Color.Red;

                this.labelRiskReward.Text = "R/R: " + riskReward;
                this.labelLossValue.Text = "-" + String.Format("{0:F" + 2 + "}", loss);
                this.labelWinValue.Text = String.Format("{0:F" + 2 + "}", win);
                if (riskReward < 1.0)
                    this.labelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.labelRiskReward.ForeColor = Color.Black;
                else
                    this.labelRiskReward.ForeColor = Color.Green;
            }
        }
        //#########################################################################################
        #endregion

        #region NumericUpDownValueChanged
        //#########################################################################################
        /// <summary>
        /// Calculates the risk, if a numericUpDown value changes.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NumericUpDownValueChanged(object sender, EventArgs e)
        {
            double entryPrice = 0;
            double stopLossPrice = 0;
            double takeProfitPrice = 0;
            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                entryPrice = (double)toolStripNumericUpDownEntry.Value;
                stopLossPrice = (double)toolStripNumericUpDownStop.Value;
                takeProfitPrice = (double)toolStripNumericUpDownTarget.Value;
            }
            else if (riskManagementPosition == RiskManagementStyle.Tab)
            {
                entryPrice = (double)numericUpDownEntry.Value;
                stopLossPrice = (double)numericUpDownStopLoss.Value;
                takeProfitPrice = (double)numericUpDownTakeProfit.Value;
            }

            double loss = 0;
            int quantity = 0;
            double win = 0;
            double riskReward = 0;
            int direction = 0;

            if (stopLossPrice < entryPrice && entryPrice < takeProfitPrice)
            {
                loss = Math.Round(((entryPrice - stopLossPrice) * 
                    Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
                quantity = (int)Math.Truncate(accountSize / 100.0 * accountRisk / loss);
                win = Math.Round(((takeProfitPrice - entryPrice) *
                    Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
                riskReward = Math.Round(win / loss, 2, MidpointRounding.AwayFromZero);
                direction = 1;
            }
            else if (stopLossPrice > entryPrice && entryPrice > takeProfitPrice)
            {
                loss = Math.Round(((stopLossPrice - entryPrice) * 
                    Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
                quantity = (int)Math.Truncate(accountSize / 100.0 * accountRisk / loss);
                win = Math.Round(((entryPrice - takeProfitPrice) *
                    Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
                riskReward = Math.Round(win / loss, 2, MidpointRounding.AwayFromZero);
                direction = -1;
            }
            else
                return;
            
            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                this.toolStripLabelQty.Text = "Qty: " + quantity;
                if (direction == 1)
                    this.toolStripLabelQty.ForeColor = Color.Green;
                else
                    this.toolStripLabelQty.ForeColor = Color.Red;

                this.toolStripLabelRiskReward.Text = "R/R: " + riskReward;
                this.toolStripLabelLoss.Text = "Loss: " + loss;
                this.toolStripLabelProfit.Text = "Win: " + win;
                if (riskReward < 1.0)
                    this.toolStripLabelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.toolStripLabelRiskReward.ForeColor = Color.Black;
                else
                    toolStripLabelRiskReward.ForeColor = Color.Green;
            }
            else
            {
                this.labelQuantity.Text = "Qty: " + quantity;
                if (direction == 1)
                    this.labelQuantity.ForeColor = Color.Green;
                else
                    this.labelQuantity.ForeColor = Color.Red;

                this.labelRiskReward.Text = "R/R: " + riskReward;
                this.labelLossValue.Text = "-" + String.Format("{0:F" + 2 + "}", loss);
                this.labelWinValue.Text = String.Format("{0:F" + 2 + "}", win);
                if (riskReward < 1.0)
                    labelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.labelRiskReward.ForeColor = Color.Black;
                else
                    this.labelRiskReward.ForeColor = Color.Green;
            }
        }
        //#########################################################################################
        #endregion

        #region OnTermination()
        //#########################################################################################
        /// <summary>
        /// Remove the controls.
        /// </summary>
        protected override void OnTermination()
        {
            #region Remove statistic forms
            //=====================================================================================
            if (statisticPosition != StatisticPositionStyle.False)
            {
                // Remove control elements
                ChartControl.Controls.RemoveByKey("colRelationLowerLow");
                ChartControl.Controls.RemoveByKey("colRelationHigherLow");
                ChartControl.Controls.RemoveByKey("colRelationLowerHigh");
                ChartControl.Controls.RemoveByKey("colRelationHigherHigh");
                ChartControl.Controls.RemoveByKey("colRelationSwingCount");
                ChartControl.Controls.RemoveByKey("colRelationSwing");
                ChartControl.Controls.RemoveByKey("relationList");
                ChartControl.Controls.RemoveByKey("colLengthLastDuration");
                ChartControl.Controls.RemoveByKey("colLengthDuration");
                ChartControl.Controls.RemoveByKey("colLengthLastLength");
                ChartControl.Controls.RemoveByKey("colLengthLength");
                ChartControl.Controls.RemoveByKey("colLengthSwingCount");
                ChartControl.Controls.RemoveByKey("colLengthDirection");
                ChartControl.Controls.RemoveByKey("lengthList");
                ChartControl.Controls.RemoveByKey("tabSwingRelation");
                ChartControl.Controls.RemoveByKey("tabSwingLength");
                ChartControl.Controls.RemoveByKey("label");

                // Remove button and separator from the NT menu
                if (toolStripButton != null)
                    toolStrip.Items.RemoveByKey("button");
                if (toolStripSeparator != null)
                    toolStrip.Items.RemoveByKey("separator");

                // Set control elements to null
                this.tabSwingLength = null;
                this.lengthList = null;
                this.tabSwingRelation = null;
                this.colLengthDirection = null;
                this.colLengthSwingCount = null;
                this.colLengthLength = null;
                this.colLengthLastLength = null;
                this.colLengthDuration = null;
                this.colLengthLastDuration = null;
                this.relationList = null;
                this.label = null;
                this.colRelationSwing = null;
                this.colRelationSwingCount = null;
                this.colRelationHigherHigh = null;
                this.colRelationLowerHigh = null;
                this.colRelationHigherLow = null;
                this.colRelationLowerLow = null;

                toolStripButton = null;
                toolStripSeparator = null;
            }
            //=====================================================================================
            #endregion

            #region Remove riskmanagement items
            //=====================================================================================
            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                toolStrip.Items.Remove(toolStripSeparator1);
                toolStrip.Items.Remove(toolStripSeparator2);
                toolStrip.Items.Remove(toolStripSeparator3);
                toolStrip.Items.Remove(toolStripSeparator4);
                toolStrip.Items.Remove(toolStripLabelQty);
                toolStrip.Items.Remove(toolStripLabelRiskReward);
                toolStrip.Items.Remove(toolStripLabelLoss);
                toolStrip.Items.Remove(toolStripLabelProfit);
                toolStrip.Items.Remove(toolStripLabelEntry);
                toolStrip.Items.Remove(toolStripButtonLong);
                toolStrip.Items.Remove(toolStripButtonShort);
                toolStrip.Items.Remove(toolStripLabelStop);
                toolStrip.Items.Remove(toolStripLabelTarget);
                toolStrip.Items.Remove(toolStripControlHostEntry);
                toolStrip.Items.Remove(toolStripControlHostStop);
                toolStrip.Items.Remove(toolStripControlHostTarget);

                // Set control elements to null
                toolStripSeparator1 = null;
                toolStripSeparator2 = null;
                toolStripSeparator3 = null;
                toolStripSeparator4 = null;
                toolStripLabelQty = null;
                toolStripLabelRiskReward = null;
                toolStripLabelLoss = null;
                toolStripLabelProfit = null;
                toolStripLabelEntry = null;
                toolStripButtonLong = null;
                toolStripButtonShort = null;
                toolStripLabelStop = null;
                toolStripLabelTarget = null;
                toolStripNumericUpDownEntry = null;
                toolStripNumericUpDownStop = null;
                toolStripNumericUpDownTarget = null;
                toolStripControlHostEntry = null;
                toolStripControlHostStop = null;
                toolStripControlHostTarget = null;
            }
            //=====================================================================================
            #endregion

            if (riskManagementPosition == RiskManagementStyle.Tab)
            {
                ChartControl.Controls.RemoveByKey("labelQuantity");
                ChartControl.Controls.RemoveByKey("labelRiskReward");
                ChartControl.Controls.RemoveByKey("labelLoss");
                ChartControl.Controls.RemoveByKey("labelWin");
                ChartControl.Controls.RemoveByKey("labelLossValue");
                ChartControl.Controls.RemoveByKey("labelWinValue");
                ChartControl.Controls.RemoveByKey("labelEntry");
                ChartControl.Controls.RemoveByKey("labelStopLoss");
                ChartControl.Controls.RemoveByKey("labelTakeProfit");
                ChartControl.Controls.RemoveByKey("buttonEntryLong");
                ChartControl.Controls.RemoveByKey("buttonEntryShort");
                ChartControl.Controls.RemoveByKey("numericUpDownEntry");
                ChartControl.Controls.RemoveByKey("numericUpDownStopLoss");
                ChartControl.Controls.RemoveByKey("numericUpDownTakeProfit");
                ChartControl.Controls.RemoveByKey("tabRiskManagement");
                this.labelQuantity = null;
                this.labelRiskReward = null;
                this.labelLoss = null;
                this.labelWin = null;
                this.labelLossValue = null;
                this.labelWinValue = null;
                this.labelEntry = null;
                this.labelStopLoss = null;
                this.labelTakeProfit = null;
                this.buttonEntryLong = null;
                this.buttonEntryShort = null;
                this.toolStripNumericUpDownEntry = null;
                this.numericUpDownStopLoss = null;
                this.numericUpDownTakeProfit = null;
                this.tabRiskManagement = null;
            }

            if (statisticPosition != StatisticPositionStyle.False
                || riskManagementPosition == RiskManagementStyle.Tab)
            {
                ChartControl.Controls.RemoveByKey("mainTabControl");
                ChartControl.Controls.RemoveByKey("panel");
                ChartControl.Controls.RemoveByKey("splitter");
                this.panel = null;
                this.mainTabControl = null;
                this.splitter = null;
            }

            if (statisticPosition != StatisticPositionStyle.False
                || riskManagementPosition == RiskManagementStyle.ToolStrip)
                toolStrip = null;

            base.OnTermination();
        }
        //#########################################################################################
        #endregion

        #region Statistic
        //#########################################################################################
        #region UpStatistic()
        //=========================================================================================
        private void UpStatistic()
        {
            int upCount = swingHighs.Count - 1;
            if (upCount == 0)
                return;

            overallUpLength = overallUpLength + swingHighs[upCount].length;
            overallAvgUpLength = Math.Round(overallUpLength / upCount, 0, 
                MidpointRounding.AwayFromZero);

            overallUpDuration = overallUpDuration + swingHighs[upCount].duration;
            overallAvgUpDuration = Math.Round(overallUpDuration / upCount, 0, 
                MidpointRounding.AwayFromZero);

            if (upCount >= statisticLength)
            {
                upLength = 0;
                upDuration = 0;
                for (int i = 0; i < statisticLength; i++)
                {
                    upLength = upLength + swingHighs[upCount - i].length;
                    upDuration = upDuration + swingHighs[upCount - i].duration;
                }
                avgUpLength = Math.Round(upLength / statisticLength, 0, 
                    MidpointRounding.AwayFromZero);
                avgUpDuration = Math.Round(upDuration / statisticLength, 0, 
                    MidpointRounding.AwayFromZero);
            }

            lengthList[1, 0].Value = upCount;
            lengthList[2, 0].Value = overallAvgUpLength;
            lengthList[3, 0].Value = avgUpLength;
            lengthList[4, 0].Value = overallAvgUpDuration;
            lengthList[5, 0].Value = avgUpDuration;

            if (swingHigh.LastRelation == 1)
            {
                hhCount++;

                if (swingHigh.CurRelation == 1) hhCountHH++;
                if (swingHigh.CurRelation == -1) hhCountLH++;
                if (swingLow.LastRelation == 1) hhCountHL++;
                if (swingLow.LastRelation == -1) hhCountLL++;

                hhCountLHPercent = Math.Round(100.0 / hhCount * hhCountLH, 1, 
                    MidpointRounding.AwayFromZero);
                hhCountHHPercent = Math.Round(100.0 / hhCount * hhCountHH, 1, 
                    MidpointRounding.AwayFromZero);
                hhCountHLPercent = Math.Round(100.0 / hhCount * hhCountHL, 1, 
                    MidpointRounding.AwayFromZero);
                hhCountLLPercent = Math.Round(100.0 / hhCount * hhCountLL, 1, 
                    MidpointRounding.AwayFromZero);
            }

            if (swingHigh.LastRelation == -1)
            {
                lhCount++;

                if (swingHigh.CurRelation == 1) lhCountHH++;
                if (swingHigh.CurRelation == -1) lhCountLH++;
                if (swingLow.LastRelation == 1) lhCountHL++;
                if (swingLow.LastRelation == -1) lhCountLL++;

                lhCountLHPercent = Math.Round(100.0 / lhCount * lhCountLH, 1,
                    MidpointRounding.AwayFromZero);
                lhCountHHPercent = Math.Round(100.0 / lhCount * lhCountHH, 1,
                    MidpointRounding.AwayFromZero);
                lhCountHLPercent = Math.Round(100.0 / lhCount * lhCountHL, 1, 
                    MidpointRounding.AwayFromZero);
                lhCountLLPercent = Math.Round(100.0 / lhCount * lhCountLL, 1, 
                    MidpointRounding.AwayFromZero);
            }

            if (swingHigh.LastRelation == 0)
            {
                dtCount++;

                if (swingHigh.CurRelation == 1) dtCountHH++;
                if (swingHigh.CurRelation == -1) dtCountLH++;
                if (swingLow.LastRelation == 1) dtCountHL++;
                if (swingLow.LastRelation == -1) dtCountLL++;

                dtCountLHPercent = Math.Round(100.0 / dtCount * dtCountLH, 1, 
                    MidpointRounding.AwayFromZero);
                dtCountHHPercent = Math.Round(100.0 / dtCount * dtCountHH, 1, 
                    MidpointRounding.AwayFromZero);
                dtCountHLPercent = Math.Round(100.0 / dtCount * dtCountHL, 1, 
                    MidpointRounding.AwayFromZero);
                dtCountLLPercent = Math.Round(100.0 / dtCount * dtCountLL, 1, 
                    MidpointRounding.AwayFromZero);
            }
            relationList[1, 0].Value = hhCount;
            relationList[2, 0].Value = hhCountHHPercent + "%";
            relationList[3, 0].Value = hhCountLHPercent + "%";
            relationList[4, 0].Value = hhCountHLPercent + "%";
            relationList[5, 0].Value = hhCountLLPercent + "%";

            relationList[1, 1].Value = lhCount;
            relationList[2, 1].Value = lhCountHHPercent + "%";
            relationList[3, 1].Value = lhCountLHPercent + "%";
            relationList[4, 1].Value = lhCountHLPercent + "%";
            relationList[5, 1].Value = lhCountLLPercent + "%";

            relationList[1, 2].Value = dtCount;
            relationList[2, 2].Value = dtCountHHPercent + "%";
            relationList[3, 2].Value = dtCountLHPercent + "%";
            relationList[4, 2].Value = dtCountHLPercent + "%";
            relationList[5, 2].Value = dtCountLLPercent + "%";
        }
        //=========================================================================================
        #endregion

        #region DownStatistic()
        //=========================================================================================
        private void DownStatistic()
        {
            int dnCount = swingLows.Count - 1;
            if (dnCount == 0)
                return;

            overallDnLength = overallDnLength + swingLows[dnCount].length;
            overallAvgDnLength = Math.Round(overallDnLength / dnCount, 0, 
                MidpointRounding.AwayFromZero);

            overallDnDuration = overallDnDuration + swingLows[dnCount].duration;
            overallAvgDnDuration = Math.Round(overallDnDuration / dnCount, 0, 
                MidpointRounding.AwayFromZero);

            if (dnCount >= statisticLength)
            {
                dnLength = 0;
                dnDuration = 0;
                for (int i = 0; i < statisticLength; i++)
                {
                    dnLength = dnLength + swingLows[dnCount - i].length;
                    dnDuration = dnDuration + swingLows[dnCount - i].duration;
                }
                avgDnLength = Math.Round(dnLength / statisticLength, 0, 
                    MidpointRounding.AwayFromZero);
                avgDnDuration = Math.Round(dnDuration / statisticLength, 0, 
                    MidpointRounding.AwayFromZero);
            }

            lengthList[1, 1].Value = dnCount;
            lengthList[2, 1].Value = overallAvgDnLength;
            lengthList[3, 1].Value = avgDnLength;
            lengthList[4, 1].Value = overallAvgDnDuration;
            lengthList[5, 1].Value = avgDnDuration;

            if (swingLow.LastRelation == -1)
            {
                llCount++;

                if (swingHigh.LastRelation == 1) llCountHH++;
                if (swingHigh.LastRelation == -1) llCountLH++;
                if (swingLow.CurRelation == 1) llCountHL++;
                if (swingLow.CurRelation == -1) llCountLL++;

                llCountLHPercent = Math.Round(100.0 / llCount * llCountLH, 1, 
                    MidpointRounding.AwayFromZero);
                llCountHHPercent = Math.Round(100.0 / llCount * llCountHH, 1, 
                    MidpointRounding.AwayFromZero);
                llCountHLPercent = Math.Round(100.0 / llCount * llCountHL, 1, 
                    MidpointRounding.AwayFromZero);
                llCountLLPercent = Math.Round(100.0 / llCount * llCountLL, 1, 
                    MidpointRounding.AwayFromZero);
            }

            if (swingLow.LastRelation == 1)
            {
                hlCount++;

                if (swingHigh.LastRelation == 1) hlCountHH++;
                if (swingHigh.LastRelation == -1) hlCountLH++;
                if (swingLow.CurRelation == 1) hlCountHL++;
                if (swingLow.CurRelation == -1) hlCountLL++;

                hlCountLHPercent = Math.Round(100.0 / hlCount * hlCountLH, 1, 
                    MidpointRounding.AwayFromZero);
                hlCountHHPercent = Math.Round(100.0 / hlCount * hlCountHH, 1, 
                    MidpointRounding.AwayFromZero);
                hlCountHLPercent = Math.Round(100.0 / hlCount * hlCountHL, 1, 
                    MidpointRounding.AwayFromZero);
                hlCountLLPercent = Math.Round(100.0 / hlCount * hlCountLL, 1, 
                    MidpointRounding.AwayFromZero);
            }

            if (swingLow.LastRelation == 0)
            {
                dbCount++;

                if (swingHigh.LastRelation == 1) dbCountHH++;
                if (swingHigh.LastRelation == -1) dbCountLH++;
                if (swingLow.CurRelation == 1) dbCountHL++;
                if (swingLow.CurRelation == -1) dbCountLL++;

                dbCountLHPercent = Math.Round(100.0 / dbCount * dbCountLH, 1, 
                    MidpointRounding.AwayFromZero);
                dbCountHHPercent = Math.Round(100.0 / dbCount * dbCountHH, 1, 
                    MidpointRounding.AwayFromZero);
                dbCountHLPercent = Math.Round(100.0 / dbCount * dbCountHL, 1, 
                    MidpointRounding.AwayFromZero);
                dbCountLLPercent = Math.Round(100.0 / dbCount * dbCountLL, 1, 
                    MidpointRounding.AwayFromZero);
            }
            relationList[1, 3].Value = llCount;
            relationList[2, 3].Value = llCountHHPercent + "%";
            relationList[3, 3].Value = llCountLHPercent + "%";
            relationList[4, 3].Value = llCountHLPercent + "%";
            relationList[5, 3].Value = llCountLLPercent + "%";

            relationList[1, 4].Value = hlCount;
            relationList[2, 4].Value = hlCountHHPercent + "%";
            relationList[3, 4].Value = hlCountLHPercent + "%";
            relationList[4, 4].Value = hlCountHLPercent + "%";
            relationList[5, 4].Value = hlCountLLPercent + "%";

            relationList[1, 5].Value = dbCount;
            relationList[2, 5].Value = dbCountHHPercent + "%";
            relationList[3, 5].Value = dbCountLHPercent + "%";
            relationList[4, 5].Value = dbCountHLPercent + "%";
            relationList[5, 5].Value = dbCountLLPercent + "%";
        }
        //=========================================================================================
        #endregion
        //#########################################################################################
        #endregion

        #region CalculateLongRisk
        //#########################################################################################
        /// <summary>
        /// Calculate risk for a long trade.
        /// </summary>
        private void CalculateLongRisk(double entryPrice, double stopLossPrice, 
            double takeProfitPrice)
        {
            double loss = Math.Round(((entryPrice - stopLossPrice) * 
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            int quantity = (int)Math.Truncate(accountSize / 100.0 * accountRisk / loss);
            double win = Math.Round(((takeProfitPrice - entryPrice) * 
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            double riskReward = Math.Round(win / loss, 2, MidpointRounding.AwayFromZero);

            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                this.toolStripNumericUpDownEntry.Value = (decimal)entryPrice;
                this.toolStripNumericUpDownStop.Value = (decimal)stopLossPrice;
                this.toolStripNumericUpDownTarget.Value = (decimal)takeProfitPrice;

                this.toolStripLabelQty.Text = "Qty: " + quantity;
                this.toolStripLabelQty.ForeColor = Color.Green;

                this.toolStripLabelRiskReward.Text = "R/R: " + riskReward;
                this.toolStripLabelLoss.Text = "Loss: " + loss;
                this.toolStripLabelProfit.Text = "Win: " + win;
                if (riskReward < 1.0)
                    this.toolStripLabelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.toolStripLabelRiskReward.ForeColor = Color.Black;
                else
                    this.toolStripLabelRiskReward.ForeColor = Color.Green;
            }
            else
            {
                this.numericUpDownEntry.Value = (decimal)entryPrice;
                this.numericUpDownStopLoss.Value = (decimal)stopLossPrice;
                this.numericUpDownTakeProfit.Value = (decimal)takeProfitPrice;
                this.labelQuantity.Text = "Qty: " + quantity;
                this.labelQuantity.ForeColor = Color.Green;

                this.labelRiskReward.Text = "R/R: " + riskReward;
                this.labelLossValue.Text = "-" + String.Format("{0:F" + 2 + "}", loss);
                this.labelWinValue.Text = String.Format("{0:F" + 2 + "}", win);
                if (riskReward < 1.0)
                    this.labelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.labelRiskReward.ForeColor = Color.Black;
                else
                    this.labelRiskReward.ForeColor = Color.Green;
            }
        }
        //#########################################################################################
        #endregion

        #region CalculateShortRisk
        //#########################################################################################
        /// <summary>
        /// Calculate risk for a short trade.
        /// </summary>
        private void CalculateShortRisk(double entryPrice, double stopLossPrice,
            double takeProfitPrice)
        {
            double loss = Math.Round(((stopLossPrice - entryPrice) *
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            int quantity = (int)Math.Truncate(accountSize / 100.0 * accountRisk / loss);
            double win = Math.Round(((entryPrice - takeProfitPrice) *
                Instrument.MasterInstrument.PointValue), 2, MidpointRounding.AwayFromZero);
            double riskReward = Math.Round(win / loss, 2, MidpointRounding.AwayFromZero);

            if (riskManagementPosition == RiskManagementStyle.ToolStrip)
            {
                this.toolStripNumericUpDownEntry.Value = (decimal)entryPrice;
                this.toolStripNumericUpDownStop.Value = (decimal)stopLossPrice;
                this.toolStripNumericUpDownTarget.Value = (decimal)takeProfitPrice;

                this.toolStripLabelQty.Text = "Qty: " + quantity;
                this.toolStripLabelQty.ForeColor = Color.Red;

                this.toolStripLabelRiskReward.Text = "R/R: " + riskReward;
                this.toolStripLabelLoss.Text = "Loss: " + loss;
                this.toolStripLabelProfit.Text = "Win: " + win;
                if (riskReward < 1.0)
                    this.toolStripLabelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.toolStripLabelRiskReward.ForeColor = Color.Black;
                else
                    this.toolStripLabelRiskReward.ForeColor = Color.Green;
            }
            else
            {
                this.numericUpDownEntry.Value = (decimal)entryPrice;
                this.numericUpDownStopLoss.Value = (decimal)stopLossPrice;
                this.numericUpDownTakeProfit.Value = (decimal)takeProfitPrice;

                this.labelQuantity.Text = "Qty: " + quantity;
                this.labelQuantity.ForeColor = Color.Red;

                this.labelRiskReward.Text = "R/R: " + riskReward;
                this.labelLossValue.Text = "-" + String.Format("{0:F" + 2 + "}", loss);
                this.labelWinValue.Text = String.Format("{0:F" + 2 + "}", win);
                if (riskReward < 1.0)
                    this.labelRiskReward.ForeColor = Color.Red;
                else if (riskReward < 1.6)
                    this.labelRiskReward.ForeColor = Color.Black;
                else
                    this.labelRiskReward.ForeColor = Color.Green;
            }
        }
        //#########################################################################################
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PriceActionSwingPro[] cachePriceActionSwingPro = null;

        private static PriceActionSwingPro checkPriceActionSwingPro = new PriceActionSwingPro();

        /// <summary>
        /// PriceActionSwingPro calculates swings and visualize them in different ways and display several information about them. Features: ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic | Divergence.
        /// </summary>
        /// <returns></returns>
        public PriceActionSwingPro PriceActionSwingPro(int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return PriceActionSwingPro(Input, dtbStrength, swingSize, swingType, useCloseValues);
        }

        /// <summary>
        /// PriceActionSwingPro calculates swings and visualize them in different ways and display several information about them. Features: ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic | Divergence.
        /// </summary>
        /// <returns></returns>
        public PriceActionSwingPro PriceActionSwingPro(Data.IDataSeries input, int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            if (cachePriceActionSwingPro != null)
                for (int idx = 0; idx < cachePriceActionSwingPro.Length; idx++)
                    if (cachePriceActionSwingPro[idx].DtbStrength == dtbStrength && Math.Abs(cachePriceActionSwingPro[idx].SwingSize - swingSize) <= double.Epsilon && cachePriceActionSwingPro[idx].SwingType == swingType && cachePriceActionSwingPro[idx].UseCloseValues == useCloseValues && cachePriceActionSwingPro[idx].EqualsInput(input))
                        return cachePriceActionSwingPro[idx];

            lock (checkPriceActionSwingPro)
            {
                checkPriceActionSwingPro.DtbStrength = dtbStrength;
                dtbStrength = checkPriceActionSwingPro.DtbStrength;
                checkPriceActionSwingPro.SwingSize = swingSize;
                swingSize = checkPriceActionSwingPro.SwingSize;
                checkPriceActionSwingPro.SwingType = swingType;
                swingType = checkPriceActionSwingPro.SwingType;
                checkPriceActionSwingPro.UseCloseValues = useCloseValues;
                useCloseValues = checkPriceActionSwingPro.UseCloseValues;

                if (cachePriceActionSwingPro != null)
                    for (int idx = 0; idx < cachePriceActionSwingPro.Length; idx++)
                        if (cachePriceActionSwingPro[idx].DtbStrength == dtbStrength && Math.Abs(cachePriceActionSwingPro[idx].SwingSize - swingSize) <= double.Epsilon && cachePriceActionSwingPro[idx].SwingType == swingType && cachePriceActionSwingPro[idx].UseCloseValues == useCloseValues && cachePriceActionSwingPro[idx].EqualsInput(input))
                            return cachePriceActionSwingPro[idx];

                PriceActionSwingPro indicator = new PriceActionSwingPro();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.DtbStrength = dtbStrength;
                indicator.SwingSize = swingSize;
                indicator.SwingType = swingType;
                indicator.UseCloseValues = useCloseValues;
                Indicators.Add(indicator);
                indicator.SetUp();

                PriceActionSwingPro[] tmp = new PriceActionSwingPro[cachePriceActionSwingPro == null ? 1 : cachePriceActionSwingPro.Length + 1];
                if (cachePriceActionSwingPro != null)
                    cachePriceActionSwingPro.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachePriceActionSwingPro = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// PriceActionSwingPro calculates swings and visualize them in different ways and display several information about them. Features: ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic | Divergence.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceActionSwingPro PriceActionSwingPro(int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return _indicator.PriceActionSwingPro(Input, dtbStrength, swingSize, swingType, useCloseValues);
        }

        /// <summary>
        /// PriceActionSwingPro calculates swings and visualize them in different ways and display several information about them. Features: ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic | Divergence.
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceActionSwingPro PriceActionSwingPro(Data.IDataSeries input, int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return _indicator.PriceActionSwingPro(input, dtbStrength, swingSize, swingType, useCloseValues);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// PriceActionSwingPro calculates swings and visualize them in different ways and display several information about them. Features: ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic | Divergence.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceActionSwingPro PriceActionSwingPro(int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            return _indicator.PriceActionSwingPro(Input, dtbStrength, swingSize, swingType, useCloseValues);
        }

        /// <summary>
        /// PriceActionSwingPro calculates swings and visualize them in different ways and display several information about them. Features: ABC pattern recognition | Risk management | Fibonacci retracements/-extensions | Statistic | Divergence.
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceActionSwingPro PriceActionSwingPro(Data.IDataSeries input, int dtbStrength, double swingSize, SwingStyle swingType, bool useCloseValues)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PriceActionSwingPro(input, dtbStrength, swingSize, swingType, useCloseValues);
        }
    }
}
#endregion
